// Model.h

#ifndef _MODEL_H_
#define _MODEL_H_

#include "Project.h"
#include "MsMatrix.h"
#include "MsTimer.h"
#include "MsVector.h"

typedef struct Mesh
{	int  materialIndex;
	int  numTriangles;
	int *pTriangleIndices;
}MESH;

		//	Material properties
typedef struct Material
{	float  ambient[4];
	float  diffuse[4];
	float  specular[4];
	float  emissive[4];
	float  shininess;
	GLuint texture;
	char * pTextureFilename;
}MATERIAL;

		//	Triangle structure
typedef struct Triangle
{	float vertexNormals[3][3];
	float s[3];
	float t[3];
	int   vertexIndices[3];
}TRIANGLE;

//	Vertex structure
typedef struct Vertex
{	char  boneID;	// for skeletal animation
	float location[3];
}VERTEX;

//	Animation keyframe information
typedef struct Keyframe
{	int   jointIndex;
	float time;	// in milliseconds
	float parameter[3];
}KEYFRAME;

//	Skeleton bone joint
typedef struct Joint
{	float localRotation[3];
	float localTranslation[3];
	float mat_absolute[16];
	float mat_relative[16];

	int   numRotationKeyframes;
	int   numTranslationKeyframes;
	KEYFRAME *pTranslationKeyframes;
	KEYFRAME *pRotationKeyframes;

	int   currentTranslationKeyframe;
	int   currentRotationKeyframe;
	float mat_final[16];

	int   parent;
}JOINT;

typedef struct Model
{
	char * pModelName;

	int   numMeshes;
	MESH *pMeshes;

	int       numMaterials;
	MATERIAL *pMaterials;

	int       numTriangles;
	TRIANGLE *pTriangles;

	int     numVertices;
	VERTEX *pVertices;

	int    numJoints;
	JOINT *pJoints;

	TIMER *pTimer;
	double totalTime;
	BOOL looping;
} MODEL;

void Model_New(MODEL * mod);
void Model_Delete(MODEL * mod);
void Model_Draw(MODEL * mod);
void Model_ReloadTextures(MODEL * mod);
void Model_SetJointKeyframe(MODEL * mod, int jointIndex, int keyframeIndex, float time, float *parameter, BOOL isRotation );
void Model_SetupJoints(MODEL * mod);
void Model_Restart(MODEL * mod);
void Model_AdvanceAnimation(MODEL * mod, float start, float end);

#endif // _MODEL_H_
