// MsMatrix.c

#include "math.h"
#include "MsMatrix.h"
#include <string.h>

#define PI	3.14159268

void MsMatrix_Set(float * this, const float * pMsMatrix )
{
	memcpy( this, pMsMatrix, sizeof( float )*16 );
}

void MsMatrix_LoadIdentity(float * this)
{
	memset( this, 0, sizeof( float )*16 );
	this[0] = this[5] = this[10] = this[15] = 1.0f;
}

void MsMatrix_PostMultiply(float * this, const float * MsMatrix )
{
	float newMsMatrix[16];

	newMsMatrix[0] = this[0] * MsMatrix[0] + this[4] * MsMatrix[1] + this[8] * MsMatrix[2];
	newMsMatrix[1] = this[1] * MsMatrix[0] + this[5] * MsMatrix[1] + this[9] * MsMatrix[2];
	newMsMatrix[2] = this[2] * MsMatrix[0] + this[6] * MsMatrix[1] + this[10] * MsMatrix[2];
	newMsMatrix[3] = 0;

	newMsMatrix[4] = this[0] * MsMatrix[4] + this[4] * MsMatrix[5] + this[8] * MsMatrix[6];
	newMsMatrix[5] = this[1] * MsMatrix[4] + this[5] * MsMatrix[5] + this[9] * MsMatrix[6];
	newMsMatrix[6] = this[2] * MsMatrix[4] + this[6] * MsMatrix[5] + this[10] * MsMatrix[6];
	newMsMatrix[7] = 0;

	newMsMatrix[8] = this[0] * MsMatrix[8] + this[4] * MsMatrix[9] + this[8] * MsMatrix[10];
	newMsMatrix[9] = this[1] * MsMatrix[8] + this[5] * MsMatrix[9] + this[9] * MsMatrix[10];
	newMsMatrix[10] = this[2] * MsMatrix[8] + this[6] * MsMatrix[9] + this[10] * MsMatrix[10];
	newMsMatrix[11] = 0;

	newMsMatrix[12] = this[0] * MsMatrix[12] + this[4] * MsMatrix[13] + this[8] * MsMatrix[14] + this[12];
	newMsMatrix[13] = this[1] * MsMatrix[12] + this[5] * MsMatrix[13] + this[9] * MsMatrix[14] + this[13];
	newMsMatrix[14] = this[2] * MsMatrix[12] + this[6] * MsMatrix[13] + this[10] * MsMatrix[14] + this[14];
	newMsMatrix[15] = 1;

	MsMatrix_Set(this, newMsMatrix );
}

void MsMatrix_SetTranslation(float * this, const float *translation )
{
	this[12] = translation[0];
	this[13] = translation[1];
	this[14] = translation[2];
}

void MsMatrix_SetInverseTranslation(float * this, const float *translation )
{
	this[12] = -translation[0];
	this[13] = -translation[1];
	this[14] = -translation[2];
}

void MsMatrix_SetRotationDegrees(float * this, const float *angles )
{
	float vec[3];
	vec[0] = ( float )( angles[0]*180.0/PI );
	vec[1] = ( float )( angles[1]*180.0/PI );
	vec[2] = ( float )( angles[2]*180.0/PI );
	MsMatrix_SetRotationRadians(this, vec );
}

void MsMatrix_SetInverseRotationRadians(float * this, const float * angles )
{
	double cr = cos( angles[0] );
	double sr = sin( angles[0] );
	double cp = cos( angles[1] );
	double sp = sin( angles[1] );
	double cy = cos( angles[2] );
	double sy = sin( angles[2] );

	this[0] = ( float )( cp*cy );
	this[4] = ( float )( cp*sy );
	this[8] = ( float )( -sp );

	double srsp = sr*sp;
	double crsp = cr*sp;

	this[1] = ( float )( srsp*cy-cr*sy );
	this[5] = ( float )( srsp*sy+cr*cy );
	this[9] = ( float )( sr*cp );

	this[2] = ( float )( crsp*cy+sr*sy );
	this[6] = ( float )( crsp*sy-sr*cy );
	this[10] = ( float )( cr*cp );
}

void MsMatrix_SetInverseRotationDegrees(float * this, const float *angles )
{
	float vec[3];
	vec[0] = ( float )( angles[0]*180.0/PI );
	vec[1] = ( float )( angles[1]*180.0/PI );
	vec[2] = ( float )( angles[2]*180.0/PI );
	MsMatrix_SetInverseRotationRadians( this, vec );
}

void MsMatrix_SetRotationRadians(float * this, const float *angles )
{
	double cr = cos( angles[0] );
	double sr = sin( angles[0] );
	double cp = cos( angles[1] );
	double sp = sin( angles[1] );
	double cy = cos( angles[2] );
	double sy = sin( angles[2] );

	this[0] = ( float )( cp*cy );
	this[1] = ( float )( cp*sy );
	this[2] = ( float )( -sp );
	this[3] = ( float )( 0.0f );

	double srsp = sr*sp;
	double crsp = cr*sp;

	this[4] = ( float )( srsp*cy-cr*sy );
	this[5] = ( float )( srsp*sy+cr*cy );
	this[6] = ( float )( sr*cp );

	this[8] = ( float )( crsp*cy+sr*sy );
	this[9] = ( float )( crsp*sy-sr*cy );
	this[10] = ( float )( cr*cp );
}

void MsMatrix_RotateVect(float * this, float * vec )
{
	float tmpvec[3];

	tmpvec[0] = vec[0] * this[0] + vec[1] * this[4] + vec[2] * this[8];
	tmpvec[1] = vec[0] * this[1] + vec[1] * this[5] + vec[2] * this[9];
	tmpvec[2] = vec[0] * this[2] + vec[1] * this[6] + vec[2] * this[10];

	memcpy( vec, tmpvec, sizeof( float )*3 );
}

void MsMatrix_InverseRotateVect(float * this, float *vec )
{
	float tmpvec[3];

	tmpvec[0] = vec[0] * this[0] + vec[1] * this[1] + vec[2] * this[2];
	tmpvec[1] = vec[0] * this[4] + vec[1] * this[5] + vec[2] * this[6];
	tmpvec[2] = vec[0] * this[8] + vec[1] * this[9] + vec[2] * this[10];

	memcpy( vec, tmpvec, sizeof( float )*3 );
}

void MsMatrix_TranslateVect(float * this, float *vec )
{
	vec[0] += this[12];
	vec[1] += this[13];
	vec[2] += this[14];
}

void MsMatrix_InverseTranslateVect(float * this, float *vec )
{
	vec[0] -= this[12];
	vec[1] -= this[13];
	vec[2] -= this[14];
}


