// MsVector.c

#include <math.h>
#include "MsVector.h"

void MsVector_Reset(float * this)
{
	this[0] = this[1] = this[2] = 0.0f;
	this[3] = 1.0f;
}

void MsVector_Add(float * this, const float * MsVector )
{
	this[0] += MsVector[0];
	this[1] += MsVector[1];
	this[2] += MsVector[2];
	this[3] += MsVector[3];
}

void MsVector_Sub(float * this, const float * MsVector )
{
	this[0] -= MsVector[0];
	this[1] -= MsVector[1];
	this[2] -= MsVector[2];
	this[3] -= MsVector[3];
}

void MsVector_Min(float * this, const float * MsVector)
{
  	this[0] = this[0] < MsVector[0] ? this[0] : MsVector[0];
	this[1] = this[1] < MsVector[1] ? this[1] : MsVector[1];
  	this[2] = this[2] < MsVector[2] ? this[2] : MsVector[2];
}

void MsVector_Max(float * this, const float * MsVector)
{
  	this[0] = this[0] > MsVector[0] ? this[0] : MsVector[0];
	this[1] = this[1] > MsVector[1] ? this[1] : MsVector[1];
  	this[2] = this[2] > MsVector[2] ? this[2] : MsVector[2];
}

void MsVector_Lerp(float * this, const float * MsVector, float s)
{
	this[0] = s * (MsVector[0] - this[0]);
	this[1] = s * (MsVector[1] - this[1]);
	this[2] = s * (MsVector[2] - this[2]);
}

void MsVector_Normalize(float * this)
{
	float len = MsVector_Length( this );

	this[0] /= len;
	this[1] /= len;
	this[2] /= len;
}

float MsVector_Length(float * this)
{
	return ( float )sqrt( this[0] * this[0] + this[1] * this[1] + this[2] * this[2] );
}

float MsVector_LengthSq(float * this)
{
	return ( float ) ( this[0] * this[0] + this[1] * this[1] + this[2] * this[2] );
}

void MsVector_Set(float * this, const float *values )
{
	this[0] = values[0];
	this[1] = values[1];
	this[2] = values[2];
}

void MsVector_MsVector(float * this, const float *values )
{
	MsVector_Set( this, values );
	this[3] = 1;
}

void MsVector_Transform(float * this, const float * mat )
{
	double MsVector[4];

	MsVector[0] = this[0] * mat[0] + this[1] * mat[4] + this[2] * mat[8] + mat[12];
	MsVector[1] = this[0] * mat[1] + this[1] * mat[5] + this[2] * mat[9] + mat[13];
	MsVector[2] = this[0] * mat[2] + this[1] * mat[6] + this[2] * mat[10] + mat[14];
	MsVector[3] = this[0] * mat[3] + this[1] * mat[7] + this[2] * mat[11] + mat[15];

	this[0] = ( float )( MsVector[0] );
	this[1] = ( float )( MsVector[1] );
	this[2] = ( float )( MsVector[2] );
	this[3] = ( float )( MsVector[3] );
}

void MsVector_Transform3(float * this, const float * mat )
{
	double MsVector[3];

	MsVector[0] = this[0] * mat[0] + this[1] * mat[4] + this[2] * mat[8];
	MsVector[1] = this[0] * mat[1] + this[1] * mat[5] + this[2] * mat[9];
	MsVector[2] = this[0] * mat[2] + this[1] * mat[6] + this[2] * mat[10];

	this[0] = ( float )( MsVector[0] );
	this[1] = ( float )( MsVector[1] );
	this[2] = ( float )( MsVector[2] );
	this[3] = 1.0f;
}

