#include "dmf.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int GetToken(FILE *fp, char *buffer)
{
    int c;
    unsigned int i = 0;

    while(1)
    {
        if(feof(fp))
            break;

        c = fgetc(fp);

        if(c != '\n' && c != '\t')
            buffer[i++] = c;

        if(i > DMF_MAX_TOKEN_SIZE)
            break;

        if(c == ',' || c == ';')
            break;

    };

    if(i > 0)
    {
        buffer[i-1] = '\0';
        return 1;
    }
    else
    {
        buffer[0] = '\0';
        return 0;
    }
}

int LoadMaterials(FILE *fp, dmf_file_t *df, dmf_progress_cb callback, dmf_texture_cb tcallback)
{
    unsigned int i, j, id;
    char buffer[DMF_MAX_TOKEN_SIZE];
	char	lmbuffer[50];
	char 	file[25],tmp[25],ext[4];

    // Get Material Count
    if(!GetToken(fp, buffer))
        return DMF_TOKEN_ERROR;

    df->numMaterials = atoi(buffer);

    if(atoi(buffer) < 0)
        return DMF_FORMAT_ERROR;

    df->materialList = (dmf_material_t*)malloc(sizeof(dmf_material_t) * df->numMaterials);
    if(0 == df->materialList)
        return DMF_MEMORY_ERROR;

    memset(df->materialList, 0, sizeof(dmf_material_t) * df->numMaterials);

    // Get Material Information
    for(i = 0; i < df->numMaterials; ++i)
    {
        if(callback != 0)
            callback(DMF_LOAD_STAGE_MATERIALS, (i * 100) / df->numMaterials);

        // Get Material ID
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        // use id index instead of array index in case file index is not in order.
        id = atoi(buffer);
        df->materialList[id].id = id;

        // Get Material Name
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->materialList[id].name = (char*)malloc(sizeof(char) * strlen(buffer) + 1);
        if(0 == df->materialList[id].name)
            return DMF_MEMORY_ERROR;

        memcpy(df->materialList[id].name, buffer, strlen(buffer) + 1);

        // Get Material Category
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->materialList[id].category = (char*)malloc(sizeof(char) * strlen(buffer) + 1);
        if(0 == df->materialList[id].category)
            return DMF_MEMORY_ERROR;

        memcpy(df->materialList[id].category, buffer, strlen(buffer) + 1);

        // Reserved
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        // Get Material Texture Layers
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->materialList[id].numLayers = atoi(buffer);
        df->materialList[id].layers = (dmf_layer_t*)malloc(sizeof(dmf_layer_t) * df->materialList[id].numLayers);
        if(0 == df->materialList[id].layers)
            return DMF_MEMORY_ERROR;

        memset(df->materialList[id].layers, 0, sizeof(dmf_layer_t) * df->materialList[id].numLayers);

        for(j = 0; j < df->materialList[id].numLayers; ++j)
        {
            df->materialList[id].layers[j].type = DMF_LAYER_TYPE_UNUSED;
            df->materialList[id].layers[j].texture = 0;
            df->materialList[id].layers[j].blendop = 0;
            df->materialList[id].layers[j].textureIndex = 0;

            // Get Material Layer Type, Filename, Blend Operation.
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            if(buffer[0] != '\0')
            {
                df->materialList[id].layers[j].type = atoi(buffer);

                if(0 == GetToken(fp, buffer))
                    return DMF_TOKEN_ERROR;



				//printf("test lm : %s \n",buffer);
                df->materialList[id].layers[j].texture = (char*)malloc(sizeof(char) * strlen(buffer) + 1);
                if(0 == df->materialList[id].layers[j].texture)
                    return DMF_MEMORY_ERROR;




                memcpy(df->materialList[id].layers[j].texture, buffer, strlen(buffer) + 1);

                if(0 == GetToken(fp, buffer))
                    return DMF_TOKEN_ERROR;

                df->materialList[id].layers[j].blendop = atoi(buffer);
                if(0 != tcallback)
                    df->materialList[id].layers[j].textureIndex = tcallback(&df->materialList[id].layers[j]);
                else
                    df->materialList[id].layers[j].textureIndex = 0;
            }
        }
    }

    return DMF_NO_ERROR;
}

int LoadHeader(FILE *fp, dmf_file_t *df, dmf_progress_cb callback)
{
    char buffer[DMF_MAX_TOKEN_SIZE];
    char c[3];

    if(0 != callback)
        callback(DMF_LOAD_STAGE_HEADER, 0);

    // Get File Header
    if(0 == GetToken(fp, buffer))
        return DMF_TOKEN_ERROR;

    if(strstr(buffer, DMF_FILE_HEADER) == 0)
        return DMF_FORMAT_ERROR;

    // Get File Version
    if(0 == GetToken(fp, buffer))
        return DMF_TOKEN_ERROR;

    if(strstr(buffer, DMF_CHAR_VERSION_0_9_1) != NULL)
        df->version = DMF_VERSION_0_9_1;
    else if(strstr(buffer, DMF_CHAR_VERSION_1_0) != NULL)
        df->version = DMF_VERSION_1_0;
    else if(strstr(buffer, DMF_CHAR_VERSION_1_1) != NULL)
        df->version = DMF_VERSION_1_1;
    else
        return DMF_VERSION_ERROR;

    // Get Map Name
    if(0 == GetToken(fp, buffer))
        return DMF_TOKEN_ERROR;

    df->mapName = (char*)malloc(sizeof(char) * strlen(buffer) + 1);
    if(0 == df->mapName)
        return DMF_MEMORY_ERROR;

    memcpy(df->mapName, buffer, strlen(buffer) + 1);

    // Get Ambient Color
    if(0 == GetToken(fp, buffer))
        return DMF_TOKEN_ERROR;

    memset(c, 0, sizeof(c));
    strncpy(c, buffer, 2);
    df->ambientLight.a = (unsigned char)strtoul(c, 0, 16);

    memset(c, 0, sizeof(c));
    strncpy(c, buffer + 2, 2);
    df->ambientLight.r = (unsigned char)strtoul(c, 0, 16);

    memset(c, 0, sizeof(c));
    strncpy(c, buffer + 4, 2);
    df->ambientLight.g = (unsigned char)strtoul(c, 0, 16);

    memset(c, 0, sizeof(c));
    strncpy(c, buffer + 6, 2);
    df->ambientLight.b = (unsigned char)strtoul(c, 0, 16);

    // Get Shadow Opacity
    if(0 == GetToken(fp, buffer))
        return DMF_TOKEN_ERROR;

    df->shadowOpacity = atoi(buffer);

    return DMF_NO_ERROR;
}


int LoadLights(FILE *fp, dmf_file_t *df, dmf_progress_cb callback)
{
    char buffer[DMF_MAX_TOKEN_SIZE];
    unsigned int i;

    // Get Number of Light Sources
    if(0 == GetToken(fp, buffer))
        return DMF_TOKEN_ERROR;

    if(atoi(buffer) < 0)
        return DMF_FORMAT_ERROR;

    df->numLights = atoi(buffer);

    df->lightList = (dmf_light_t*)malloc(sizeof(dmf_light_t) * df->numLights);
    if(0 == df->lightList)
        return DMF_MEMORY_ERROR;

    memset(df->lightList, 0, sizeof(dmf_light_t) * df->numLights);

    for(i = 0; i < df->numLights; ++i)
    {
        if(0 != callback)
            callback(DMF_LOAD_STAGE_LIGHTS, (i * 100) / df->numLights);

        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].type = atoi(buffer);

        if(df->lightList[i].type == DMF_LIGHT_TYPE_POINT)
        {
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].constant = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].linear = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].quadratic = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].range = atoi(buffer);

        } // DMF_LIGHT_TYPE_POINT

        else if(df->lightList[i].type == DMF_LIGHT_TYPE_DIRECTIONAL)
        {
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].direction[0] = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].direction[1] = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].direction[2] = (float)atof(buffer);

        } // DMF_LIGHT_TYPE_DIRECTIONAL

        else if(df->lightList[i].type == DMF_LIGHT_TYPE_SPOT)
        {
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].constant = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].linear = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].quadratic = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].range = atoi(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].direction[0] = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].direction[1] = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].direction[2] = (float)atof(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].umbra = atoi(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].penumbra = atoi(buffer);
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->lightList[i].falloff = atoi(buffer);

        } // DMF_LIGHT_TYPE_SPOT

        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].center[0] = (float)atof(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].center[1] = (float)atof(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].center[2] = (float)atof(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].flash = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].active = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].diffuse[0] = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].diffuse[1] = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].diffuse[2] = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].specular[0] = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].specular[1] = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].specular[2] = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].intensity = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].id = atoi(buffer);
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->lightList[i].name = (char*)malloc(sizeof(char) * strlen(buffer) + 1);
        if(0 == df->lightList[i].name)
            return DMF_MEMORY_ERROR;

        if(strlen(buffer) > 0)
            memcpy(df->lightList[i].name, buffer, strlen(buffer) + 1);

    } // for i

    return DMF_NO_ERROR;
}

int LoadObjects(FILE *fp, dmf_file_t *df, dmf_progress_cb callback)
{
    char buffer[DMF_MAX_TOKEN_SIZE];
    unsigned int i, j, k, m, n;

    // Get Object Count
    if(0 == GetToken(fp, buffer))
        return DMF_TOKEN_ERROR;

    if(atoi(buffer) < 0)
        return DMF_FORMAT_ERROR;

    df->numObjects = atoi(buffer);
    df->objectList = (dmf_object_t*)malloc(sizeof(dmf_object_t) * df->numObjects);
    if(0 == df->objectList)
        return DMF_MEMORY_ERROR;

    memset(df->objectList, 0, sizeof(dmf_object_t) * df->numObjects);

    for(i = 0; i < df->numObjects; ++i)
    {
        if(0 != callback)
            callback(DMF_LOAD_STAGE_OBJECTS, (i * 100) / df->numObjects);

        // Get Object Name
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].name = (char*)malloc(sizeof(char) * strlen(buffer) + 1);
        if(0 == df->objectList[i].name)
            return DMF_MEMORY_ERROR;

        memcpy(df->objectList[i].name, buffer, strlen(buffer) + 1);

        // Get Object ID
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].id = atoi(buffer);

        // Get Object Type
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].type = atoi(buffer);

        // Is Object Visible
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].visible = atoi(buffer);

        // Get Object Snap Method
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].snap = atoi(buffer);

        // Get Object Group ID
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].group = atoi(buffer);

        // Get Object Auto UV
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].auto_uv = atoi(buffer);

        // Get Object Tag
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].tag = (char*)malloc(sizeof(char) * strlen(buffer) + 1);
        if(0 == df->objectList[i].tag)
            return DMF_MEMORY_ERROR;

        memcpy(df->objectList[i].tag, buffer, strlen(buffer) + 1);

        // Get Object Number Of Vertices
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].numVertices = atoi(buffer);
        df->objectList[i].vertices = (dmf_vector3_t*)malloc(sizeof(dmf_vector3_t) * df->objectList[i].numVertices);
        if(0 == df->objectList[i].vertices)
            return DMF_MEMORY_ERROR;

        memset(df->objectList[i].vertices, 0, sizeof(dmf_vector3_t) * df->objectList[i].numVertices);

        for(j = 0; j < df->objectList[i].numVertices; ++j)
        {
            // Get Vertex X
            if(0 ==GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->objectList[i].vertices[j].x = (float)atof(buffer);

            // Get Vertex Y
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->objectList[i].vertices[j].y = (float)atof(buffer);

            // Get Vertex Z
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->objectList[i].vertices[j].z = (float)atof(buffer);
        }

        // Get Object Number Of Polygons
        if(0 == GetToken(fp, buffer))
            return DMF_TOKEN_ERROR;

        df->objectList[i].numPolygons = atoi(buffer);
        df->objectList[i].polygons = (dmf_polygon_t*)malloc(sizeof(dmf_polygon_t) * df->objectList[i].numPolygons);
        if(0 == df->objectList[i].polygons)
            return DMF_MEMORY_ERROR;

        memset(df->objectList[i].polygons, 0, sizeof(dmf_polygon_t) * df->objectList[i].numPolygons);

        for(j = 0; j < df->objectList[i].numPolygons; ++j)
        {
            // Get Polygon Number Of Vertices
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->objectList[i].polygons[j].numVertices = atoi(buffer);

            // Get Polygon Material ID
            if(0 == GetToken(fp, buffer))
                return DMF_TOKEN_ERROR;

            df->objectList[i].polygons[j].materialIndex = atoi(buffer);
            m = df->objectList[i].polygons[j].materialIndex;

            df->objectList[i].polygons[j].indices = (unsigned int*)malloc(sizeof(unsigned int) * df->objectList[i].polygons[j].numVertices);
            if(0 == df->objectList[i].polygons[j].indices)
                return DMF_MEMORY_ERROR;

            memset(df->objectList[i].polygons[j].indices, 0, sizeof(unsigned int) * df->objectList[i].polygons[j].numVertices);

            for(k = 0; k < df->materialList[m].numLayers; ++k)
            {
                if(df->materialList[m].layers[k].type != DMF_LAYER_TYPE_UNUSED)
                {
                    df->objectList[i].polygons[j].texcoords[k] = (dmf_vector2_t*)malloc(sizeof(dmf_vector2_t) * df->objectList[i].polygons[j].numVertices);
                    if(0 == df->objectList[i].polygons[j].texcoords[k])
                        return DMF_MEMORY_ERROR;

                    memset(df->objectList[i].polygons[j].texcoords[k], 0, sizeof(dmf_vector2_t) * df->objectList[i].polygons[j].numVertices);
                }
            }

            for(k = 0; k < df->objectList[i].polygons[j].numVertices; ++k)
            {
                // Get Polygon Vertices
                if(0 == GetToken(fp, buffer))
                    return DMF_TOKEN_ERROR;

                df->objectList[i].polygons[j].indices[k] = atoi(buffer);

            } // for k

            for(n = 0; n < df->materialList[m].numLayers; ++n)
            {
                if(df->materialList[m].layers[n].type != DMF_LAYER_TYPE_UNUSED)
                {
                    for(k = 0; k < df->objectList[i].polygons[j].numVertices; ++k)
                    {
                        // Get Polygon Texture Coordinates
                        if(0 == GetToken(fp, buffer))
                            return DMF_TOKEN_ERROR;

                        df->objectList[i].polygons[j].texcoords[n][k].x = (float)atof(buffer);

                        // In Version 1.1 V Coordinate of UV is flipped so negate the coordinate;
                        if(0 == GetToken(fp, buffer))
                            return DMF_TOKEN_ERROR;

                        df->objectList[i].polygons[j].texcoords[n][k].y = (float)atof(buffer);

                    } // for k (numVertices)
                }
            } // for n (numLayers)

        } // for j (numPolygons)

    } // for i (df.numObjects)

    return DMF_NO_ERROR;
}

unsigned int ReadDMF(dmf_file_t *df, const char *filename, dmf_progress_cb pcallback, dmf_texture_cb tcallback)
{
    FILE *fp;
    unsigned int result;

    memset(df, 0, sizeof(dmf_file_t));
    fp = fopen(filename, "r");
    if(0 == fp)
        return DMF_FILE_ERROR;

    result = LoadHeader(fp, df, pcallback);
    if(DMF_NO_ERROR != result)
    {
        FreeDMF(df);
        return result;
    }

    result = LoadMaterials(fp, df, pcallback, tcallback);
    if(DMF_NO_ERROR != result)
    {
        FreeDMF(df);
        return result;
    }

    result = LoadObjects(fp, df, pcallback);
    if(DMF_NO_ERROR != result)
    {
        FreeDMF(df);
        return result;
    }

    result = LoadLights(fp, df, pcallback);
    if(DMF_NO_ERROR != result)
    {
        FreeDMF(df);
        return result;
    }

    fclose(fp);

    return DMF_NO_ERROR;
}

void FreeMaterial(dmf_material_t *mat)
{
    unsigned int j;

    free(mat->name);
    free(mat->category);

    for(j = 0; j < mat->numLayers; ++j)
        free(mat->layers[j].texture);

    free(mat->layers);
    memset(mat, 0, sizeof(dmf_material_t));
}

void FreePolygon(dmf_file_t *df, dmf_polygon_t *poly)
{
    unsigned int i, m;

    m = poly->materialIndex;

    for(i = 0; i < df->materialList[m].numLayers; ++i)
    {
        if(df->materialList[m].layers[i].type != DMF_LAYER_TYPE_UNUSED)
            free(poly->texcoords[i]);
    }

    free(poly->indices);
    memset(poly, 0, sizeof(dmf_polygon_t));
}

void FreeObject(dmf_file_t *df, dmf_object_t *obj)
{
    unsigned int i;

    for(i = 0; i < obj->numPolygons; ++i)
        FreePolygon(df, &obj->polygons[i]);

    free(obj->name);
    free(obj->tag);
    free(obj->polygons);
    free(obj->vertices);

    memset(obj, 0, sizeof(dmf_object_t));
}

void FreeLight(dmf_light_t *light)
{
    free(light->name);
    memset(light, 0, sizeof(dmf_light_t));
}

unsigned int FreeDMF(dmf_file_t *df)
{
    unsigned int i;

    for(i = 0; i < df->numObjects; ++i)
        FreeObject(df, &df->objectList[i]);

    for(i = 0; i < df->numMaterials; ++i)
        FreeMaterial(&df->materialList[i]);

    for(i = 0; i < df->numLights; ++i)
        FreeLight(&df->lightList[i]);

    free(df->objectList);
    free(df->materialList);
    free(df->lightList);
    free(df->mapName);
    memset(df, 0, sizeof(dmf_file_t));

    return DMF_NO_ERROR;
}

unsigned int WriteDMF(const dmf_file_t *df, const char *filename)
{
    FILE *fp;
    char buffer[DMF_MAX_TOKEN_SIZE];
    unsigned int i, j, k, m, t;

    fp = fopen(filename, "w");
    if(0 == fp)
        return DMF_FILE_ERROR;

    sprintf(buffer, "%s;\n%s;\n", DMF_FILE_HEADER, DMF_CHAR_VERSION_1_1);
    fputs(buffer, fp);

    sprintf(buffer, "%s;%02X%02X%02X%02X;%i;\n",
        df->mapName,
        df->ambientLight.a,
        df->ambientLight.r,
        df->ambientLight.g,
        df->ambientLight.b,
        df->shadowOpacity);

    fputs(buffer, fp);

    sprintf(buffer, "%i;\n", df->numMaterials);
    fputs(buffer, fp);

    for(i = 0; i < df->numMaterials; ++i)
    {
        sprintf(buffer, "%i;%s;%s;%i;%i;",
            df->materialList[i].id,
            df->materialList[i].name,
            df->materialList[i].category,
            df->materialList[i].reserved,
            df->materialList[i].numLayers);

        fputs(buffer, fp);

        for(j = 0; j < df->materialList[i].numLayers; ++j)
        {
            if(df->materialList[i].layers[j].type != DMF_LAYER_TYPE_UNUSED)
            {
                sprintf(buffer, "%i,%s,%i;",
                    df->materialList[i].layers[j].type,
                    df->materialList[i].layers[j].texture,
                    df->materialList[i].layers[j].blendop);
            }
            else
                sprintf(buffer, ";");

            fputs(buffer, fp);
        }

        fputs("\n", fp);
    }

    sprintf(buffer, "%i;\n", df->numObjects);
    fputs(buffer, fp);

    for(i = 0; i < df->numObjects; ++i)
    {
        sprintf(buffer, "%s;%i;%i;%i;%i;%i;%i;%s;\n",
            df->objectList[i].name,
            df->objectList[i].id,
            df->objectList[i].type,
            df->objectList[i].visible,
            df->objectList[i].snap,
            df->objectList[i].group,
            df->objectList[i].auto_uv,
            df->objectList[i].tag);

        fputs(buffer, fp);

        sprintf(buffer, "%i;\n", df->objectList[i].numVertices);
        fputs(buffer, fp);

        for(j = 0; j < df->objectList[i].numVertices; ++j)
        {
            sprintf(buffer, "%i;%i;%i;\n",
                (int)df->objectList[i].vertices[j].x,
                (int)df->objectList[i].vertices[j].y,
                (int)df->objectList[i].vertices[j].z);

            fputs(buffer, fp);
        }

        sprintf(buffer, "%i;\n", df->objectList[i].numPolygons);
        fputs(buffer, fp);

        for(j = 0; j < df->objectList[i].numPolygons; ++j)
        {
            sprintf(buffer, "%i;%i;",
                df->objectList[i].polygons[j].numVertices,
                df->objectList[i].polygons[j].materialIndex);

            fputs(buffer, fp);

            for(k = 0; k < df->objectList[i].polygons[j].numVertices; ++k)
            {
                sprintf(buffer, "%i;", df->objectList[i].polygons[j].indices[k]);
                fputs(buffer, fp);
            }

            m = df->objectList[i].polygons[j].materialIndex;

            for(t = 0; t < df->materialList[m].numLayers; ++t)
            {
                if(df->materialList[m].layers[t].type != DMF_LAYER_TYPE_UNUSED)
                {
                    for(k = 0; k < df->objectList[i].polygons[j].numVertices; ++k)
                    {
                        sprintf(buffer, "%0.4f;%0.4f;",
                            df->objectList[i].polygons[j].texcoords[t][k].x,
                            df->objectList[i].polygons[j].texcoords[t][k].y);
                        fputs(buffer, fp);
                    }
                }
            }

            fputs("\n", fp);

        } // for j (numPolygons)

    } // for i (numObjects)


    sprintf(buffer, "%i;\n", df->numLights);
    fputs(buffer, fp);

    for(i = 0; i < df->numLights; ++i)
    {
        sprintf(buffer, "%i;", df->lightList[i].type);
        fputs(buffer, fp);

        if(df->lightList[i].type == DMF_LIGHT_TYPE_POINT)
        {
            sprintf(buffer, "%0.f;%0.f;%E;%i;",
                df->lightList[i].constant,
                df->lightList[i].linear,
                df->lightList[i].quadratic,
                (int)df->lightList[i].range);

            fputs(buffer, fp);
        }
        else if(df->lightList[i].type == DMF_LIGHT_TYPE_DIRECTIONAL)
        {
            sprintf(buffer, "%0.f;%0.f;%0.f;",
                df->lightList[i].direction[0],
                df->lightList[i].direction[1],
                df->lightList[i].direction[2]);

            fputs(buffer, fp);
        }
        else if(df->lightList[i].type == DMF_LIGHT_TYPE_SPOT)
        {
            sprintf(buffer, "%0.f;%0.f;%E;%i;%0.f;%0.f;%0.f;%i;%i;%i;",
                df->lightList[i].constant,
                df->lightList[i].linear,
                df->lightList[i].quadratic,
                df->lightList[i].range,
                df->lightList[i].direction[0],
                df->lightList[i].direction[1],
                df->lightList[i].direction[2],
                df->lightList[i].umbra,
                df->lightList[i].penumbra,
                df->lightList[i].falloff);

            fputs(buffer, fp);
        }
        else
        {
            fclose(fp);
            return DMF_UNSUPPORTED_ERROR;
        }

        sprintf(buffer, "%i;%i;%i;%i;%i;%i;%i;%i;%i;%i;%i;%i;%i;%s;\n",
            (int)df->lightList[i].center[0],
            (int)df->lightList[i].center[1],
            (int)df->lightList[i].center[2],
            df->lightList[i].flash,
            df->lightList[i].active,
            df->lightList[i].diffuse[0],
            df->lightList[i].diffuse[1],
            df->lightList[i].diffuse[2],
            df->lightList[i].specular[0],
            df->lightList[i].specular[1],
            df->lightList[i].specular[2],
            df->lightList[i].intensity,
            df->lightList[i].id,
            df->lightList[i].name);

        fputs(buffer, fp);
    }

    fclose(fp);

    return DMF_NO_ERROR;
}

unsigned int TriangulateDMF(dmf_file_t *df)
{
    unsigned int i, j, k, m, n;
    unsigned int tcount = 0;
    dmf_polygon_t *tris;
    dmf_object_t *obj;

    for(n = 0; n < df->numObjects; ++n)
    {
        obj = &df->objectList[n];

        for(i = 0; i < obj->numPolygons; ++i)
            tcount += (obj->polygons[i].numVertices - 2);

        tris = (dmf_polygon_t*)malloc(sizeof(dmf_polygon_t) * tcount);
        memset(tris, 0, sizeof(dmf_polygon_t) * tcount);

        tcount = 0;

        for(i = 0; i < obj->numPolygons; ++i)
        {
            m = obj->polygons[i].materialIndex;

            for(j = 0; j < obj->polygons[i].numVertices - 2; ++j)
            {
                tris[tcount].materialIndex = obj->polygons[i].materialIndex;
                tris[tcount].indices = (unsigned int*)malloc(sizeof(unsigned int) * 3);
                tris[tcount].numVertices = 3;

                for(k = 0; k < df->materialList[m].numLayers; ++k)
                {
                    if(df->materialList[m].layers[k].type != DMF_LAYER_TYPE_UNUSED)
                    {
                        tris[tcount].texcoords[k] = (dmf_vector2_t*)malloc(sizeof(dmf_vector2_t) * 3);

                        tris[tcount].texcoords[k][0].x = obj->polygons[i].texcoords[k][0].x;
                        tris[tcount].texcoords[k][0].y = obj->polygons[i].texcoords[k][0].y;

                        tris[tcount].texcoords[k][1].x = obj->polygons[i].texcoords[k][1 + j].x;
                        tris[tcount].texcoords[k][1].y = obj->polygons[i].texcoords[k][1 + j].y;

                        tris[tcount].texcoords[k][2].x = obj->polygons[i].texcoords[k][2 + j].x;
                        tris[tcount].texcoords[k][2].y = obj->polygons[i].texcoords[k][2 + j].y;
                    }
                }

                tris[tcount].indices[0] = obj->polygons[i].indices[0];
                tris[tcount].indices[1] = obj->polygons[i].indices[1 + j];
                tris[tcount].indices[2] = obj->polygons[i].indices[2 + j];
                ++tcount;
            }
        }

        // free the old object as it will now be refilled with the new data
        for(i = 0; i < obj->numPolygons; ++i)
            FreePolygon(df, &obj->polygons[i]);

        free(obj->polygons);

        obj->numPolygons = tcount;
        obj->polygons = (dmf_polygon_t*)malloc(sizeof(dmf_polygon_t) * tcount);

        for(i = 0; i < tcount; ++i)
        {
            obj->polygons[i].materialIndex = tris[i].materialIndex;
            obj->polygons[i].indices = (unsigned int*)malloc(sizeof(unsigned int) * 3);
            obj->polygons[i].numVertices = 3;

            m = obj->polygons[i].materialIndex;

            for(k = 0; k < df->materialList[m].numLayers; ++k)
            {
                if(df->materialList[m].layers[k].type != DMF_LAYER_TYPE_UNUSED)
                {
                    obj->polygons[i].texcoords[k] = (dmf_vector2_t*)malloc(sizeof(dmf_vector2_t) * 3);

                    for(j = 0; j < 3; ++j)
                    {
                        obj->polygons[i].texcoords[k][j].x = tris[i].texcoords[k][j].x;
                        obj->polygons[i].texcoords[k][j].y = tris[i].texcoords[k][j].y;
                    }
                }
            }

            for(j = 0; j < 3; ++j)
                obj->polygons[i].indices[j] = tris[i].indices[j];

        }

        for(i = 0; i < tcount; ++i)
            FreePolygon(df, &tris[i]);

        free(tris);

    } // n

    return DMF_NO_ERROR;
}

unsigned int ScaleDMF(dmf_file_t *df, float x, float y, float z, float u, float v)
{
    unsigned int i, j, k, l, m;


    for(i = 0; i < df->numObjects; ++i)
    {
        // scale object vertices
        for(j = 0; j < df->objectList[i].numVertices; ++j)
        {
            df->objectList[i].vertices[j].x *= x;
            df->objectList[i].vertices[j].y *= y;
            df->objectList[i].vertices[j].z *= z;
        }

        // scale texture coordinates
        for(j = 0; j < df->objectList[i].numPolygons; ++j)
        {
            m = df->objectList[i].polygons[j].materialIndex;

            for(l = 0; l < df->materialList[m].numLayers; ++l)
            {
                if(df->materialList[m].layers[l].type != DMF_LAYER_TYPE_UNUSED)
                {
                    for(k = 0; k < df->objectList[i].polygons[j].numVertices; ++k)
                    {
                        df->objectList[i].polygons[j].texcoords[l][k].x *= u;
                        df->objectList[i].polygons[j].texcoords[l][k].y *= v;
                    }
                }
            }

        } // for j (numPolygons)

    } // for i (numObjects)

    // scale light positions
    for(i = 0; i < df->numLights; ++i)
    {
        df->lightList[i].center[0] *= x;
        df->lightList[i].center[1] *= y;
        df->lightList[i].center[2] *= z;
    }

    return DMF_NO_ERROR;
}



/* end of file */

