/*
 * 	---------------- MoJo engine --------------
 *   		---- Basic Game Engine ----
 *
 * Author: Yann PARMENTIER <kohai@kohaistyle.com>
 *
 * ALL RIGHTS RESERVED
 *
 *	Uses
 * 			DMFloader  		( www.delgine.com )
 * 			GLFW 				( glfw.sourceforge.net )
 *			Fmod Sound API 		( www.fmod.org )
 *
 * To compile this project you must include the following libraries:
 * wlib.lib uuid.lib fmodlcc.lib glfw.lib glu32.lib ole32.lib olepro32.lib opengl32.lib
 * You need also the header file glfw.h in your GL/ compiler directory.
 *
 * File header: mojoMain.c
 *
 */

// thx to PSC80, Mephisto



#include 	<stdio.h>
#include 	<stdlib.h>
#include 	<conio.h>
#include 	<math.h>
#include 	<GL/glfw.h>
#include 	<GL/glext.h>
#include 	<inc/fmod.h>
#include 	"dmf.h"
#include 	"mojomain.h"
#include 	"mojogui.h"
#include 	"mojomatrix.h"
#include 	"mojovector.h"
#include 	"mojoparticles2.h"
#include 	"mojofont.h"
#include 	"mojolog.h"
#include 	"newton.h"
#include 	"ezxml.h"
#include 	"project.h"
#include 	"milkshapemodel.h"
#include 	"gltexfont.h"

#ifdef MEMWATCH
#include 	"memwatch.h"
#endif




// The width and height of our window
int 	screen_width	= 800, screen_height=600;
int		CinemascopeLo 	= 0.0;
int		CinemascopeHi 	= 0.0;
int 	running			= 1;
int 	exiting			= 1;
int 	menuing			= 1;
int		currentid 		= 0;
int 	gamemode 		= 1;
float 	ypos 			= 400, xpos = 300;
float	level 			= 1.5;
int		volumeSfx		= 200;
int		volumeMusic		= 150;
FMUSIC_MODULE 		*music = 0;

// Player Matrix and vector
struct PLAYER	player;
MATRIX			MtxPlayer, 		MtxTmp, 		MtxLvl;
VECTOR			VplayerMove, 	VplayerStrafe,	VTmp;
int				is_jumping	= 0, 	chanJumping;
int				is_firing	= 0, 	chanFiring;
int				is_walking	= 0, 	chanWalking;
float			is_dashing	= 0.0;
float			m[16];





// Camera view vectors
double rot_x, rot_z, rot_y;




// Newton Collision World
static 				NewtonWorld* nWorld;
int					MtlDefault, MtlLvl, MtlPlyr, MtlBullet, MtlEnmBullet, MtlEnm;
NewtonBody*			contactBody0;
NewtonBody*			contactBody1;
NewtonCollision* 	newtonBulletCol;
NewtonCollision* 	newtonEnmBulletCol;

// MS3D model data
MODEL Model;		// Holds the model data
MODEL Enm[20];		// Holds the model data
MODEL Bonus;		// Holds the model data



// Multitexture Pointer
PFNGLACTIVETEXTUREARBPROC       glActiveTextureARB       = NULL;
PFNGLMULTITEXCOORD1FARBPROC		glMultiTexCoord1fARB	= NULL;
PFNGLMULTITEXCOORD2FARBPROC     glMultiTexCoord2fARB     = NULL;
PFNGLCLIENTACTIVETEXTUREARBPROC glClientActiveTextureARB = NULL;



extern texFont_t font;

#define DEBUGNWT 	0
#define DEBUG 		0

#define NEWTON_RESOLUTION      0.01f   // Newton resolution in seconds,.
#define RAND_INT(l,h) (((int)(random() * ((double)(h)-(l)+1))) + (l))



void mojoLevelCollisionCallback (const NewtonBody* bodyWithTreeCollision, const NewtonBody* body,
								 const float* vertex, int vertexstrideInBytes, int indexCount, const int* indexArray);










void mojoLevelCollisionCallback (
	const NewtonBody* bodyWithTreeCollision,
	const NewtonBody* body,
	const float* vertex,
	int vertexstrideInBytes,
	int indexCount,
	const int* indexArray)
{
 	// the aplication can use this function for debuging purpose by writing the
	// face to a global variable then display which face of the mesh are been used for collition.
    // ,,,,,,,,,,,
	// ..........

	// the application can use this function also to modify the collsion geometry by changing the face ID
	// this could be usesfull to make surface change for example from dry to wet, or breaking glass
	// for this the application should used the functions:
	// int id = NewtonTreeCollisionGetFaceAtribute (treeCollision, indexArray);
	// NewtonTreeCollisionSetFaceAtribute (treeCollision, indexArray, newid);

}



void DebugShowGeometryCollision (const NewtonBody* body, int vertexCount, const float* faceVertec, int id)
{

#if DEBUGNWT

	int i;
	i = vertexCount - 1;
	VECTOR p0,p1;



	glColor3f(1.0,1.0,1.0);



	glBegin(GL_LINES);

	p0.v[0] = faceVertec[i * 3 + 0];
	p0.v[1] = faceVertec[i * 3 + 1];
	p0.v[2] = faceVertec[i * 3 + 2];

	for (i = 0; i < vertexCount; i ++) {
		p1.v[0] = faceVertec[i * 3 + 0]; p1.v[1] = faceVertec[i * 3 + 1]; p1.v[2] =faceVertec[i * 3 + 2];

		glVertex3f (p0.v[0], p0.v[1], p0.v[2]);
		glVertex3f (p1.v[0], p1.v[1], p1.v[2]);
 		p0 = p1;
	}

	glEnd();


#endif

}



void DebugCollision (const NewtonBody* body)
{

#if DEBUGNWT
		NewtonBodyForEachPolygonDo (body, DebugShowGeometryCollision);

#endif

}





void PhysicsApplyForceAndTorque (const NewtonBody* body)
{

	float mass;
	float Ixx;
	float Iyy;
	float Izz;
	VECTOR fforce;


	// apply the gravity vector
	NewtonBodyGetMassMatrix (body, &mass, &Ixx, &Iyy, &Izz);
    VectorLoad( &fforce, 0.0f, -mass * 80.0f, 0.0f);
	NewtonBodyAddForce (body, &fforce.v[0]);

	// quick hack : add a constant rotation force
	// to have the player working in sim from the start
	VectorLoad( &VTmp, 0.0f, 0.1f, 0.0f);
	NewtonBodySetOmega (body, &VTmp.v[0]);


	if(is_jumping == 0 )
	{
		if(glfwGetKey(GLFW_KEY_SPACE) /*|| glfwGetKey(122)*/) // S
		{

			//is_jumping = 1;
    		VectorLoad( &fforce, 0.0f, 30000.0f, 0.0f);
			NewtonBodyAddForce( body, &fforce.v[0] );

			if(!FSOUND_IsPlaying(chanJumping))
				chanJumping = FSOUND_PlaySound(FSOUND_FREE, soundSlot[ playerSndJUMP ].sndPtr);
		}
	}else{
    		VectorLoad( &fforce, 0.0f, -10000.0f, 0.0f);
			NewtonBodyAddForce( body, &fforce.v[0] );

			//if(glfwGetKey(GLFW_KEY_SPACE) == GLFW_RELEASE) is_jumping = 0;

	}


	if(!is_jumping)
	{
	if(player.plyCurrentDashPower > player.plyCurrentDash)
	{

		if(is_dashing >= 0.5)
		{
			NewtonBodyGetVelocity(body, &fforce.v[0]);
			VectorMultiply(&fforce, 0.2);
	//		VectorLoad( &fforce, 0.0f, 0.0f, 0.0f);
			NewtonBodySetVelocity(body, &fforce.v[0]);
			return;
		}
	}
	}

	// Add the movement force vector
	VectorLoad( &VTmp, VplayerMove.v[0]*100.0, 0.0, VplayerMove.v[2]*100.0);
	NewtonBodyAddForce (body, &VTmp.v[0]);

/*
	if(player.plyState == stHIT)
	{
	VectorLoad( &VTmp, -1000.0, 0.0, -1000.0);
	NewtonBodyAddForce (body, &VTmp.v[0]);
	}
*/


	NewtonBodyGetVelocity(body, &fforce.v[0]);

	if(VplayerMove.v[0] == 0.0 )
	{
		if(fforce.v[0] < 15.0 || fforce.v[0] > -15.0 )
			fforce.v[0] *= 0.80;
	}

	if(VplayerMove.v[2] == 0.0 )
	{
		if(fforce.v[2] < 15.0 || fforce.v[2] > -15.0 )
			fforce.v[2] *= 0.80;
	}


	NewtonBodySetVelocity(body, &fforce.v[0]);

	//printf("velocity#%1.3f %1.3f %1.3f\n", fforce.v[0], fforce.v[1], fforce.v[2]);


}




void EnmApplyForceAndTorque (const NewtonBody* body)
{

	float mass;
	float Ixx;
	float Iyy;
	float Izz;
	VECTOR fforce;
	struct ENEMY*	enm;

	NewtonBodyGetMassMatrix (body, &mass, &Ixx, &Iyy, &Izz);
    VectorLoad( &fforce, 0.0f, -mass * 80.0f, 0.0f);
	NewtonBodyAddForce (body, &fforce.v[0]);

	enm = (struct ENEMY*)NewtonBodyGetUserData( body );

	switch(enm->enmState)
	{
		case stCHASE:
		VectorLoad( &fforce, sin(RADIAN(enm->enmAngle))*enm->enmForwardSpeed, 0.0, cos(RADIAN(enm->enmAngle))*enm->enmForwardSpeed);
		NewtonBodyAddForce (body, &fforce.v[0]);
		break;

		case stWANDER:
		VectorLoad( &fforce, sin(RADIAN(enm->enmAngle))*enm->enmForwardSpeed, 0.0, cos(RADIAN(enm->enmAngle))*enm->enmForwardSpeed);
		NewtonBodyAddForce (body, &fforce.v[0]);
		break;

		case stIDLE:
		break;

		case stHIT:
		break;

		case stDEAD:
		NewtonDestroyBody( nWorld, body);
		break;

	}

	VectorLoad( &VTmp, 0.0f, 0.1f, 0.0f);
	NewtonBodySetOmega (body, &VTmp.v[0]);



}





// add force and torque to rigid body
void PhysicsSetTransform (const NewtonBody* body, const float* matrix)
{

	// note : assignment is faster than memcpy ??

	MtxPlayer.m[0] = matrix[0] ;
	MtxPlayer.m[1] = matrix[1] ;
	MtxPlayer.m[2] = matrix[2] ;
	MtxPlayer.m[3] = matrix[3] ;

	MtxPlayer.m[4] = matrix[4] ;
	MtxPlayer.m[5] = matrix[5] ;
	MtxPlayer.m[6] = matrix[6] ;
	MtxPlayer.m[7] = matrix[7] ;

	MtxPlayer.m[8] = matrix[8] ;
	MtxPlayer.m[9] = matrix[9] ;
	MtxPlayer.m[10] = matrix[10] ;
	MtxPlayer.m[11] = matrix[11] ;

	MtxPlayer.m[12] = matrix[12] ;
	MtxPlayer.m[13] = matrix[13] ;
	MtxPlayer.m[14] = matrix[14] ;
	MtxPlayer.m[15] = matrix[15] ;

}




void 			mojoSkyboxLoad(void)
{

	int 	free;
	//char 	tmp[25],ext[4];
	char	path[50];




	free = mojoTexManGetFreeId();

	//strcpy(path,mojoResDir);
	strcpy(path,"skyfront.tga");
	mojoTexManLoad( path, free, 1, GL_CLAMP_TO_EDGE );
	skybox[0] = free;
	printf("loaded skyfront #texid %d\n",free);


	free = mojoTexManGetFreeId();

	//strcpy(path,mojoResDir);
	strcpy(path,"skyback.tga");
	mojoTexManLoad( path, free, 1, GL_CLAMP_TO_EDGE );
	skybox[1] = free;
	printf("loaded skyback #texid %d\n",free);


	free = mojoTexManGetFreeId();

	//strcpy(path,mojoResDir);
	strcpy(path,"skyup.tga");
	mojoTexManLoad( path, free, 1, GL_CLAMP_TO_EDGE );
	skybox[2] = free;
	printf("loaded skyup #texid %d\n",free);


	free = mojoTexManGetFreeId();

	//strcpy(path,mojoResDir);
	strcpy(path,"skydown.tga");
	mojoTexManLoad( path, free, 1, GL_CLAMP_TO_EDGE );
	skybox[3] = free;
	printf("loaded skydown #texid %d\n",free);


	free = mojoTexManGetFreeId();

	//strcpy(path,mojoResDir);
	strcpy(path,"skyleft.tga");
	mojoTexManLoad( path, free, 1, GL_CLAMP_TO_EDGE );
	skybox[4] = free;
	printf("loaded skyleft #texid %d\n",free);


	free = mojoTexManGetFreeId();

	//strcpy(path,mojoResDir);
	strcpy(path,"skyright.tga");
	mojoTexManLoad( path, free, 1, GL_CLAMP_TO_EDGE );
	skybox[5] = free;
	printf("loaded skyright #texid %d\n",free);


#if DEBUG

	printf("\n#call SkyboxLoad\n\r" );

#endif
}




unsigned int 	TextureLoadCallback(const dmf_layer_t *layer)
{

	char 	*buffer="",lmbuffer[255],tmp[25],ext[4];
	int 	success,i;


	if(strncmp(layer->texture,"LightMaps",9)==0)
	{
		buffer[0] = '\0';

		// _splitpath is an LCC specific function
		// use basename() on Unix
		_splitpath(layer->texture,NULL,NULL,tmp,ext);

		buffer = strtok(layer->texture,".");
		strlcpy(lmbuffer,buffer,strlen(buffer)+1);

		//printf("lightmap %s %s %s %s\n", layer->texture, lmbuffer, tmp, ext);

		strcat(lmbuffer,".tga");

		printf("lightmap %s\n",lmbuffer);

		i = mojoTexManGetTextureId( lmbuffer );

	if( i == -1 )
	{

		currentid++;
		success = mojoTexManLoad( lmbuffer, currentid, 1, GL_REPEAT );

		printf("%s #success%d #texid %d\n",tmp, success, currentid);
   		return currentid;

	}



	}else{

		// _splitpath is an LCC specific function
		// use basename() on Unix
		_splitpath(layer->texture,NULL,NULL,tmp,ext);
		strcat(tmp,".tga");

		printf("texture %s\n",tmp);

		i = mojoTexManGetTextureId( tmp );

	if( i == -1 )
	{

		currentid++;
		success = mojoTexManLoad( tmp, currentid, 1, GL_REPEAT );

		printf("%s #success%d #texid %d\n",tmp, success, currentid);
   		return currentid;

	}


	}




    return i;
}



void 			ProgressCallback(unsigned int stage, unsigned int progress)
{
    // progress is in the range 0% - 100%

    switch(stage)
    {
    case DMF_LOAD_STAGE_HEADER:
		printf("header\n");
        break;
    case DMF_LOAD_STAGE_MATERIALS:
		printf("materials\n");
        break;
    case DMF_LOAD_STAGE_OBJECTS:
		printf("objects\n");
        break;
    case DMF_LOAD_STAGE_LIGHTS:
		printf("lights\n");
        break;
    };
}




void 			resize (int width, int height)
{

	float ratio = 1.0f;


    screen_width=width; // We obtain the   float ratio = 1.0f;

  if( height > 0 )
  {
	  ratio = (float) width / (float) height;
  }

  /* Setup viewport (Place where the stuff will appear in the main window). */
  glViewport(0, 0, width, height);

  /*
   * Change to the projection matrix and set
   * our viewing volume.
   */
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(60.0, ratio, 1.0, 1024.0);

}




/* Handle key strokes */
void GLFWCALL handle_key_down(int key, int action)
{
  if( action != GLFW_PRESS )
  {
	//rot_y = 0;
	//rot_x = 0;
    //return;
  }


  switch(key) {

    case GLFW_KEY_F2:
		CLAMP(volumeSfx--, 0, 255);
		FSOUND_SetSFXMasterVolume( volumeSfx );
      break;

    case GLFW_KEY_F3:
		CLAMP(volumeSfx++, 0, 255);
		FSOUND_SetSFXMasterVolume( volumeSfx );
      break;

    case GLFW_KEY_F4:
		CLAMP(volumeMusic--, 0, 255);
		FMUSIC_SetMasterVolume( music, volumeMusic );
      break;

    case GLFW_KEY_F5:
		CLAMP(volumeMusic++, 0, 255);
		FMUSIC_SetMasterVolume( music, volumeMusic );
      break;

/*    case GLFW_KEY_RIGHT:
		rot_y -= 2.0;
      break;
    case GLFW_KEY_UP:
		rot_x -= 2.0;
      break;
    case GLFW_KEY_DOWN:
		rot_x += 2.0;
      break;
    case GLFW_KEY_LSHIFT:

		if(gamemode == 1)
		{
			gamemode = 2;
			glfwEnable( GLFW_MOUSE_CURSOR );
			glfwDisable( GLFW_KEY_REPEAT );
		}else{
			gamemode = 1;
			glfwDisable( GLFW_MOUSE_CURSOR );
			glfwEnable( GLFW_KEY_REPEAT );
		}

		break;*/
    case GLFW_KEY_ESC:


    	switch( gamemode )
    	{
        	case 1:						// in play
			mojoFadeSetState( 1 );
	    	//exiting = 0;
			break;
        	case 2:						// in menu
			menuing = 0;
			break;
		}
      	break;
    default:
      break;
  }
}




void GLFWCALL MousePosFun( int x, int y )
{



    // Depending on which view was selected, rotate around different axes
    switch( gamemode )
    {
        case 1:
            rot_x += (2.0*(y - ypos))*0.15;
            rot_y -= (2.0*(x - xpos))*0.15;

		    xpos = screen_width/2;
		    ypos = screen_height/2;

			glfwSetMousePos(xpos,ypos);

            break;
        case 2:
			xpos = x;
			ypos = y;
			printf("mouse #%d #%d \r", x, y);
            break;
        default:
            // Do nothing for perspective view, or if no view is selected
            break;
    }

	//printf("mouse x#%f y#%f \n",rot_x, rot_y);

    // Remember mouse position


}




void GLFWCALL MouseWheelFun(int pos)
{

	level -= (float)pos * 0.1;


	CLAMP(level, 0.1, 1.1);

	glfwSetMouseWheel( 0 );

	printf("mwheel msg x#%d %f\n", pos, level);

}



void GLFWCALL MouseButtonFun( int button, int action )
{
    // Button clicked?
    if( ( button == GLFW_MOUSE_BUTTON_LEFT ) && action == GLFW_PRESS )
    {
		//printf(" clicked\n");
		is_firing = 1;
    }// Button released?
    else if( button == GLFW_MOUSE_BUTTON_LEFT )
    {
		//printf(" released\n");
		is_firing = 0;
    }


    if( ( button == GLFW_MOUSE_BUTTON_RIGHT ) && action == GLFW_PRESS )
    {
		is_dashing = 1.0;
    }// Button released?
    else if( ( button == GLFW_MOUSE_BUTTON_RIGHT ) )
    {
		//is_dashing=0;
    }// Button released?

}




void            mojoGlInit( void)
{

   	glClearColor(0.0, 0.0, 0.0, 0.0);					//This clear the background color to black
   	glShadeModel(GL_SMOOTH);							// Type of shading for the polygons Projection yransformation
	glHint(GL_PERSPECTIVE_CORRECTION_HINT,GL_NICEST);	// Really Nice Perspective Calculations
	glHint(GL_POINT_SMOOTH_HINT,GL_NICEST);				// Really Nice Point Smoothing
   	glMatrixMode(GL_PROJECTION);						// Specifies which matrix stack is the target for matrix operations
   	glLoadIdentity();									// We initialize the projection matrix as identity
   	glEnable(GL_DEPTH_TEST);							// We enable the depth test (also called zbuffer)
   	glDepthFunc(GL_LESS);
   	glDisable(GL_FOG);
   	glPolygonMode(GL_FRONT, GL_FILL);					// Polygon rasterization mode (polygon / filled)
   	glCullFace(GL_BACK);								// Which face do we cull
	glEnable(GL_CULL_FACE); 		   					// Enable back face culling
	glDisable(GL_LIGHTING);
	glViewport(0, 0, screen_width, screen_height);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluPerspective(60.0, screen_width/screen_height, 1.0, 1024.0);

}




GLvoid          mojoGlOrthoBegin( void )
{
   glMatrixMode(GL_PROJECTION);							// Select The Projection Matrix
   glPushMatrix();										// Store The Projection Matrix
   glLoadIdentity();									// Reset The Projection Matrix
   glOrtho(0, screen_width, 0, screen_height, -1, 1);	// Set Up An Ortho Screen
   glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
   glPushMatrix();										// Store The Modelview Matrix
   glLoadIdentity();									// Reset The Modelview Matrix
}





GLvoid          mojoGlOrthoEnd( void )
{
   glMatrixMode(GL_PROJECTION);							// Select The Projection Matrix
   glPopMatrix();										// Restore The Old Projection Matrix
   glMatrixMode(GL_MODELVIEW);							// Select The Modelview Matrix
   glPopMatrix();										// Restore The Old Projection Matrix
}





void			mojoCompileDmf(dmf_file_t dmf, int listNb, NewtonWorld* nWorld, NewtonTreeCollisionCallback levelCallback )
{

    dmf_object_t 	*dmfobj;
	dmf_polygon_t 	*dmfpoly;
	dmf_material_t 	*dmfmatl;
	int				i, j;
	unsigned int	texid;
	int 			numpoly=0;
	float m[16];

	float			newtonVertex[9];
	NewtonBody*		m_level;
	NewtonCollision* collision;

	// create the collsion tree geometry
	collision = NewtonCreateTreeCollision(nWorld, NULL/*mojoLevelCollisionCallback*/);

	// prepare to create collsion geometry
	NewtonTreeCollisionBeginBuild(collision);


	dmfobj = dmf.objectList;
	dmfmatl = dmf.materialList;
	glNewList(listNb, GL_COMPILE);


	glColor3f(1.0, 1.0, 1.0);

		for(i=0;i<(int)dmf.numObjects;i++)
		{





			dmfpoly = dmfobj->polygons;



			for(j=0;j<(int)dmfobj->numPolygons;j++)
			{

				dmf_vector2_t 	*nuv, *luv;
				int				hasMulti;


				hasMulti = 0;


				if(dmfmatl[dmfpoly->materialIndex].numLayers > 1)
					hasMulti = 1;


				texid = dmfmatl[dmfpoly->materialIndex].layers[0].textureIndex;

				glActiveTextureARB( GL_TEXTURE0_ARB );
	   			glEnable (GL_TEXTURE_2D);
				glBindTexture( GL_TEXTURE_2D, texid );
//	   			glDisable(GL_TEXTURE_2D);
//				glEnable(GL_BLEND);


				texid = dmfmatl[dmfpoly->materialIndex].layers[3].textureIndex;

				if( hasMulti )
				{
					glActiveTextureARB( GL_TEXTURE1_ARB );
					glEnable(GL_BLEND);
					glBlendFunc(GL_ONE, GL_ZERO);
	   				glEnable (GL_TEXTURE_2D);
					glBindTexture( GL_TEXTURE_2D, texid );
				}

				glBegin(GL_TRIANGLES);


				//----------------------------------------------- vtx 1
				luv = NULL;
				nuv = &(dmfpoly->texcoords[0])[0];
				luv = &(dmfpoly->texcoords[3])[0];


				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, nuv->x, nuv->y);
				if(hasMulti)
				{
					glMultiTexCoord2fARB(GL_TEXTURE1_ARB, luv->x, luv->y);
				}

				glVertex3f(	dmfobj->vertices[dmfpoly->indices[0] ].x,
							dmfobj->vertices[dmfpoly->indices[0] ].y,
							dmfobj->vertices[dmfpoly->indices[0] ].z);


				//----------------------------------------------- vtx 3
				luv = NULL;
				nuv = &(dmfpoly->texcoords[0])[2];
				luv = &(dmfpoly->texcoords[3])[2];


				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, nuv->x, nuv->y);
				if(hasMulti)
				{
					glMultiTexCoord2fARB(GL_TEXTURE1_ARB, luv->x, luv->y);
				}

				glVertex3f(	dmfobj->vertices[dmfpoly->indices[2] ].x,
							dmfobj->vertices[dmfpoly->indices[2] ].y,
							dmfobj->vertices[dmfpoly->indices[2] ].z);

				//----------------------------------------------- vtx 2
				luv = NULL;
				nuv = &(dmfpoly->texcoords[0])[1];
				luv = &(dmfpoly->texcoords[3])[1];

				glMultiTexCoord2fARB(GL_TEXTURE0_ARB, nuv->x, nuv->y);
				if(hasMulti)
				{
					glMultiTexCoord2fARB(GL_TEXTURE1_ARB, luv->x, luv->y);
				}

				glVertex3f(	dmfobj->vertices[dmfpoly->indices[1] ].x,
							dmfobj->vertices[dmfpoly->indices[1] ].y,
							dmfobj->vertices[dmfpoly->indices[1] ].z);



				newtonVertex[0] = dmfobj->vertices[dmfpoly->indices[0] ].x;
				newtonVertex[1] = dmfobj->vertices[dmfpoly->indices[0] ].y;
				newtonVertex[2] = dmfobj->vertices[dmfpoly->indices[0] ].z;

				newtonVertex[3] = dmfobj->vertices[dmfpoly->indices[2] ].x;
				newtonVertex[4] = dmfobj->vertices[dmfpoly->indices[2] ].y;
				newtonVertex[5] = dmfobj->vertices[dmfpoly->indices[2] ].z;

				newtonVertex[6] = dmfobj->vertices[dmfpoly->indices[1] ].x;
				newtonVertex[7] = dmfobj->vertices[dmfpoly->indices[1] ].y;
				newtonVertex[8] = dmfobj->vertices[dmfpoly->indices[1] ].z;

				NewtonTreeCollisionAddFace(collision, 3, &newtonVertex[0], sizeof(float)*3, 1);

				numpoly++;
				dmfpoly++;

				glEnd();
			}


			dmfobj++;

		}


		glActiveTextureARB( GL_TEXTURE1_ARB );
		glDisable(GL_BLEND);
		glDisable (GL_TEXTURE_2D);

		glEndList();

		printf("numpoly : %d\n", numpoly);

		NewtonTreeCollisionEndBuild(collision, 1);

		// create the level ridid body
		m_level = NewtonCreateBody(nWorld, collision);

		m[0]=1.0;	m[4]=0.0;	m[8]=0.0;	m[12]=0.0;
		m[1]=0.0;	m[5]=1.0;	m[9]=0.0;	m[13]=-60.0;
		m[2]=0.0;	m[6]=0.0;	m[10]=1.0;	m[14]=0.0;
		m[3]=0.0;	m[7]=0.0;	m[11]=0.0;	m[15]=1.0;


		// set the global position of this body
		NewtonBodySetMatrix ( m_level, (float*) &m[0] );

		VECTOR boxP0;
		VECTOR boxP1;
		// get the position of the aabb of this geometry
		NewtonCollisionCalculateAABB (collision, (float*) &m[0], &boxP0.v[0], &boxP1.v[0]);

		// add some extra padding the world size
		boxP0.v[0] -= 10.0f;
		boxP0.v[1] -= 10.0f;
		boxP0.v[2] -= 10.0f;
		boxP1.v[0] += 10.0f;
		boxP1.v[1] += 10.0f;
		boxP1.v[2] += 10.0f;


		// set the world size
		NewtonSetWorldSize (nWorld, &boxP0.v[0], &boxP1.v[0]);


		// release the collision tree (this way the application does not have to do book keeping of Newton objects
		NewtonReleaseCollision (nWorld, collision);

}




void LogMatrix(MATRIX *m) {
  char buf[256];

  printf("Matrix:\n");
  sprintf(buf, "%1.3f %1.3f %1.3f %1.3f\n", m->m[0], m->m[4], m->m[8],  m->m[12]);
  printf(buf);
  sprintf(buf, "%1.3f %1.3f %1.3f %1.3f\n", m->m[1], m->m[5], m->m[9],  m->m[13]);
  printf(buf);
  sprintf(buf, "%1.3f %1.3f %1.3f %1.3f\n", m->m[2], m->m[6], m->m[10], m->m[14]);
  printf(buf);
  sprintf(buf, "%1.3f %1.3f %1.3f %1.3f\n", m->m[3], m->m[7], m->m[11], m->m[15]);
  printf(buf);
}



float RayCast (const NewtonBody* body, const float* normal, int collisionID, void* userData, float intersetParam)
{
	float* paramPtr;

	float mass;
	float Ixx;
	float Iyy;
	float Izz;

	NewtonBodyGetMassMatrix (body, &mass, &Ixx, &Iyy, &Izz);
	if (mass == 0.0f) {
		// pick any static rigid nody
		paramPtr = (float*)userData;
		paramPtr[0] = intersetParam;
		return intersetParam;
	}
	// else continue the search
	return 1.0;
}



float mojoGetFloor (float x, float y,float z)
{
	float parameter;

	// shot a vertical ray from a high altitude and collect the intersection parameter.
	VECTOR 	p0, p1;

   	p0.v[0] = x;
	p0.v[1] = y;
	p0.v[2] = z;

   	p1.v[0] = x;
	p1.v[1] = -500.0f;
	p1.v[2] = z;

	parameter = 1.2f;
	NewtonWorldRayCast (nWorld, &p0.v[0], &p1.v[0], RayCast, &parameter);

	// the intersection is the interpolated value
	return y - (y + 500.0f) * parameter;
}




double mojoGetIntersect (float x, float y,float z, float x1,  float y1, float z1, float castDist)
{
	float parameter;

	// shot a horizontal ray from pos x,y,z and collect the intersection parameter.
	VECTOR 	p0, p1;


   	p0.v[0] = x;
	p0.v[1] = y;
	p0.v[2] = z;

   	p1.v[0] = x1;
	p1.v[1] = y1;
	p1.v[2] = z1;


	parameter = 1.0f;
	NewtonWorldRayCast (nWorld, &p0.v[0], &p1.v[0], RayCast, &parameter);

	// the intersection is the interpolated value
	return (double)castDist * parameter;
}





int 			main(int argc,char *argv[])
{


  	/* Dimensions of our window. */
  	int 				width, height;
  	/* Style of our window. */
  	int 				mode= GLFW_FULLSCREEN;
  	/* Frame time */
   	double          	t, t0, t1, fps, elapsed, frames;
   	char            	StrBuffer[200];
	char 				*exten;
	char				*fileEntity;
	char				*filePreset;
	char				*fileDMF;
	char				*fileMusic;
	char				*fileNextLevel = "def/lvl1.xml";
	char				*fileLevelTitle;

	int 				success=0, fontTex, titleTex;
    dmf_file_t 			dmf;
	int					animCurr = 0, animCurrEnm;
	int					enmNum = 0;
	int					gameStat = 0; 	// 0 loose, 1 win
	float				pingpong = -1;		// yeah, funny name, isn't it ? :)

    unsigned int 		result;
	double 				strafeangle = 0.0, forwardspeed = 0.0, tmpFloat = 0.0;
	float				speedfactor = 6.0, distToFloor;
	double				distToStatic;
	float				m[16];
	VECTOR 				Veye, Vview, VplyrMoveNorm;


	int					i, dashBillboardId;
	NewtonBody*			newtonPlayer;
	NewtonBody*			newtonTmp;
	NewtonBody*			newtonEnmBullet;
 	NewtonBody*			newtonBullet;

	NewtonCollision* 	newtonPlayerCol;
	NewtonCollision* 	newtonTmpCol[50];
	double 				mAccumulativeLoopTime = 0.0;         // Accumulates the loop time for Newton resolution

    //ezxml_t 		xmlNode, enemyNode;

	struct ENEMY		*enemyList=NULL, *tmpEnmList = NULL;
	struct particleStm	*StaticPStm=NULL, *PresetPStm = NULL, *DynPStm = NULL, *DynPStmList = NULL;
	struct BIND			*currentBind = NULL, *bindList = NULL;

	int		button1, button2;
	struct 	mojoGuiEl *mbutton1= NULL, *mbutton2 = NULL;
	struct 	mojoGuiEvent *myEvent;




	if (MessageBox(NULL,"Would You Like To Run In Fullscreen Mode?", "Start FullScreen?",MB_YESNO|MB_ICONQUESTION)==IDNO)
	{
		mode=GLFW_WINDOW;							// Windowed Mode
	}





  	////////////////////////////////// [ Initialize Globals and Window ]

	mojolog ("- mojo \tSTART : Starting Application !\n");

  	/* Desired window properties */
  	width  = screen_width;
  	height = screen_height;

	CinemascopeLo = (int)( ( screen_height * 5.0 ) /100 );
	CinemascopeHi = ( screen_height - CinemascopeLo ) - CinemascopeLo;

	/* Initialize GLFW */
	if(glfwInit() == GL_FALSE)
	{
    	fprintf(stderr, "GLFW initialization failed\n");
		mojolog ("! mojo \terror : GLFW initialization failed !\n");
    	exit(-1);
  	}

	mojolog ("- mojo \tlog : GLFW initialization Success !\n");


  	/* Open window */
  	if( glfwOpenWindow(width,height,8,8,8,8,16,0,mode) == GL_FALSE )
  	{
  	  	fprintf(stderr, "Could not open window\n");
		mojolog ("- mojo \terror : Could not open window !\n");
  	  	glfwTerminate();
  	  	exit(-1);
  	}


	mojolog ("- mojo \tlog : Window Open Success !\n");

	glfwSwapInterval(0);
	/* Window resize handler */
	glfwSetWindowSizeCallback( resize );

	/* Keyboard handler */
	glfwSetKeyCallback( handle_key_down );
	glfwEnable( GLFW_KEY_REPEAT );

	/* Mouse handler */
    glfwSetMousePosCallback( MousePosFun );
	glfwSetMouseWheelCallback( MouseWheelFun );
	glfwDisable( GLFW_MOUSE_CURSOR );

	/* Mouse button handler */
	glfwSetMouseButtonCallback( MouseButtonFun );





  	////////////////////////////////// [ Initialize Extensions ]

	// Get Gfx Brand Info
	exten = (char *) glGetString(GL_VENDOR);
	sprintf(StrBuffer,"- mojo \tVendor : %s\n",exten);
	mojolog (StrBuffer);

	// Get Gfx Chip Info
	exten = (char *) glGetString(GL_RENDERER);
	sprintf(StrBuffer,"- mojo \tGCard : %s\n",exten);
	mojolog (StrBuffer);

	if(glfwExtensionSupported( "GL_ARB_multitexture" ) == GL_FALSE){
		MessageBox(GetActiveWindow(),"No multi-texturing support !","_inFrame_",MB_ICONASTERISK);
		mojolog ("- mojo \terror : No multi-texturing support !\n");
		exit(1);
	}
	mojolog ("- mojo \tlog : Multitexturing Support !\n");

	// Get Multi Texturing PFN
	glActiveTextureARB       = (PFNGLCLIENTACTIVETEXTUREARBPROC)glfwGetProcAddress("glActiveTextureARB");
	glMultiTexCoord2fARB     = (PFNGLMULTITEXCOORD2FARBPROC)glfwGetProcAddress("glMultiTexCoord2fARB");
	glClientActiveTextureARB = (PFNGLACTIVETEXTUREARBPROC)glfwGetProcAddress("glClientActiveTextureARB");




  	////////////////////////////////// [ Initialize Sound ]

	mojoSoundInit();



	//printf("fmod -> %i\n", i);
  	////////////////////////////////// [ Initialize GL screen ]

	mojoGlInit();




  	////////////////////////////////// [ Start of Outer Game Loop ]


while(menuing)
{


	// font texture
	// -----------------------------------------------------
	strcpy(mojoResDir,"./textures/\0");
	fontTex = mojoTexManGetFreeId();
	//success = mojoTexManLoad("Font.tga", fontTex, 1, GL_CLAMP_TO_EDGE );
	//mojoFontInit(fontTex);
	mojoTexManSetTexture( fontTex, "font.tga", fontTex );
 	fontLoad ("textures/font.tga", fontTex);


	titleTex = mojoTexManGetFreeId();
	success = mojoTexManLoad("title_tex.tga", titleTex, 1, GL_CLAMP_TO_EDGE );
	mojoTexManSetTexture( titleTex, "tile_tex.tga", titleTex );



	Sleep(300);
	glfwPollEvents();
	glfwDisable( GLFW_KEY_REPEAT );

	gamemode = 2;
	exiting = 1;
	fileNextLevel = "def/lvl1.xml";


  	////////////////////////////////// [ Title Screen ]

	while(!glfwGetKey(GLFW_KEY_SPACE))
	{


		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 	// This clear the background


		t = glfwGetTime(); 										// Get time position
		elapsed = (t - t1);

		t1 = t;

		printf("%f\r",pingpong);

		mojoGlOrthoBegin();

		glDisable(GL_DEPTH_TEST);
		glEnable(GL_TEXTURE_2D);

		glBindTexture(GL_TEXTURE_2D, titleTex);					// Select Our background Texture
		glColor4f(1.0f,1.0f,1.0f,1.0f);

		glBegin(GL_QUADS);										// Start Drawing Our Quad
			glTexCoord2d(0,0);	glVertex2d(0,0);				// Top Left Of Screen
			glTexCoord2d(1,0);	glVertex2d( width, 0);			// Bottom Right Of Screen
			glTexCoord2d(1,1);	glVertex2d( width,height);		// Top Right Of Screen
			glTexCoord2d(0,1);	glVertex2d(	0, height);			// Bottom Left Of Screen
		glEnd();

		glDisable(GL_TEXTURE_2D);
		glEnable(GL_DEPTH_TEST);


		mojoCinemascope( width, height, 100, 400 );

		mojoGlOrthoEnd();



		fontSize (14);
		fontShadow ();
		fontDrawString (15, 280, "F2/F3 - music volume up/down");

		fontSize (14);
		fontShadow ();
		fontDrawString (15, 260, "F4/F5 - sfx volume up/down");



		pingpong += elapsed;

		fontShadow ();

		if(pingpong>1.0) pingpong = -1.0;

		if(pingpong>0)											// pingpong for flashing text
			mojoGlPrintCenter(width, (height-36)/4, "press space to start", 1, 18);


   		glfwSwapBuffers(); 										// In double buffered mode we invert the positions

		if(!menuing)											// Esc has been pressed in gamemode 2 ( title screen ), so General Exit
		{
			exiting = 0;
			break;
		}
	}

  	////////////////////////////////// [ End of Title Screen ]



	glfwEnable( GLFW_KEY_REPEAT );
	mojoTexManFreeList();

while(exiting)
{

	if(strcmp(fileNextLevel,"end")==0)
	{
		gameStat = 2;
		exiting = 0;
		break;
	}

	running 	= 1;
	gamemode 	= 1;
	enmNum		= 0;
	tmpEnmList 	= NULL;
	enemyList 	= NULL;
	gameStat	= -1;



	mojolog ("- mojo \tlog : Loading new level !\n");


	VectorLoad( &Veye, 0.0, 300.0, -400.0);

	VectorLoad( &Vview, 0.0, 0.0, 0.0);

	VectorLoad( &VplayerMove, 0.0, 0.0, 0.0);

	VectorLoad( &VplayerStrafe, 0.0, 0.0, 0.0);


	MatrixLoadIdentity( &MtxPlayer );
	MatrixTranslate( &MtxPlayer, 0.0, 0.0, 0.0);


	MatrixLoadIdentity( &MtxTmp );
	MatrixTranslate( &MtxTmp, 0.0, 0.0, 0.0);


	player.plyTotalLife = 100;
	player.plyRadiusCol = 2.5;

	player.plyCurrentWeapon = 1;
	player.plyCurrentWPower = 10.0;
	player.plyCurrentDash = 15.0;		// this the dash wave radius
	player.plyCurrentDashPower = 100.0;	// this the dash remaining power


  	////////////////////////////////// [ Initialize general resources ]

	// font texture
	// -----------------------------------------------------
	strcpy(mojoResDir,"./textures/\0");
	fontTex = mojoTexManGetFreeId();
	//success = mojoTexManLoad("Font.tga", fontTex, 1, GL_CLAMP_TO_EDGE );
	//mojoFontInit(fontTex);
	mojoTexManSetTexture( fontTex, "font.tga", fontTex );
 	fontLoad ("textures/font.tga", fontTex);

	//font.texId = fontTex;

	printf("font tex #%d #succes %d\n", fontTex, success );
	mojoGlPrintLoading(width, (height-16)/2, "Loading font", fontTex, 1, 12);

	//sleep(300);



	mojoGlOrthoBegin();




	// Newton world Vars
	// -----------------------------------------------------
	nWorld = NewtonCreate (NULL, NULL);

	NewtonSetSolverModel( nWorld, 3 ) ;
	NewtonSetFrictionModel( nWorld, 1 );

	mojolog ("- mojo \tlog : Newton Creation Success !\n");
	mojoGlPrintLoading(width, (height-16)/2, "Loading NGD", fontTex, 1, 12);
	//sleep(300);




	// skybox texture
	// -----------------------------------------------------
	strcpy(mojoResDir,"./skybox/\0");
 	mojoSkyboxLoad();
	mojolog ("- mojo \tlog : Skybox Loading Success !\n");
	mojoGlPrintLoading(width, (height-16)/2, "Loading skybox", fontTex, 1, 12);


	// Particles init
	// -----------------------------------------------------

	strcpy(mojoResDir,"./textures/\0");


	// Bullets init
	// -----------------------------------------------------
	mojoFreeBulletList();


	// Render to texture init
	// -----------------------------------------------------
//	mojoGlowRTCreate( 256, 0);
//	mojoGlowRTCreate( 128, 1);
//	mojoGlowRTCreate( 64, 2);
//	mojoGlowRTCreate( 32, 3);




	// Xml Cfg Vars
	// -----------------------------------------------------


	fileEntity = mojoLoadLvlXmlCfg( fileNextLevel ,0);
	printf(" res value : %s\n", fileEntity);

	filePreset = mojoLoadLvlXmlCfg( fileNextLevel ,1);
	printf(" res value : %s\n", filePreset);

	fileDMF = mojoLoadLvlXmlCfg( fileNextLevel ,2);
	printf(" res value : %s\n", fileDMF);

	fileMusic = mojoLoadLvlXmlCfg( fileNextLevel ,3);
	printf(" res value : %s\n", fileMusic);

	fileLevelTitle = mojoLoadLvlXmlCfg( fileNextLevel ,5);



	fileNextLevel = mojoLoadLvlXmlCfg( fileNextLevel ,4);
	printf(" res value : %s\n", fileNextLevel);


// -------------------------------------------------------------------------------------------------------

	// dmf map object
	// -----------------------------------------------------

	// Set resource dir
	strcpy(mojoResDir,"./textures/\0");


    result = ReadDMF(&dmf, fileDMF , ProgressCallback, TextureLoadCallback);
    if(DMF_NO_ERROR != result)
    {
		mojolog ("- mojo \terror : DMF loading failed !\n");
        exit(1);
    }

	mojolog ("- mojo \tlog : DMF loading Success !\n");

    TriangulateDMF(&dmf);

    if(dmf.version >= DMF_VERSION_1_1)
        ScaleDMF(&dmf, .2f, .2f, .2f, 1.f, -1.f);
    else
        ScaleDMF(&dmf, .2f, .2f, .2f, 1.f, 1.f);

	dmf.glDisplayList = 1;
	mojoCompileDmf( dmf, dmf.glDisplayList, nWorld, mojoLevelCollisionCallback );

	mojoGlPrintLoading(width, (height-16)/2, "Loading level", fontTex, 1, 12);
	//sleep(300);

// -------------------------------------------------------------------------------------------------------



	music = FMUSIC_LoadSong( fileMusic );


	mojoLoadResXmlCfg( fileEntity );				// in Entity file	( game entities )

	mojoGlPrintLoading(width, (height-16)/2, "Loading entities", fontTex, 1, 12);


	mojoLoadResXmlCfg( filePreset );				// in Preset file	( particles presets )

	mojoGlPrintLoading(width, (height-16)/2, "Loading particles presets", fontTex, 1, 12);


	mojoLoadBonusPresetXmlCfg( filePreset );		// in Preset file	( bonus presets )

	mojoGlPrintLoading(width, (height-16)/2, "Loading bonus presets", fontTex, 1, 12);


	enemyList = mojoLoadEnmXmlCfg( fileEntity ); // in Entity file	( enemy list )

	if(enemyList == NULL)
	{
		mojolog ("- mojo \terror : No enemy list loaded !\n");
		exit(1);

	}



	StaticPStm = mojoLoadPStmXmlCfg( fileEntity );// in Preset file	( pstm list )

	if(StaticPStm == NULL)
	{
		mojolog ("- mojo \terror : No PStm list loaded !\n");
		exit(1);

	}



	PresetPStm = mojoLoadPStmPresetXmlCfg( filePreset );

	if(StaticPStm == NULL)
	{
		mojolog ("- mojo \terror : No PStm Preset list loaded !\n");
		exit(1);

	}


	//strcpy(mojoResDir,"\0"); // get to local folder for models textures !!!
	strcpy(mojoResDir,"./models/\0");

	Model_New(&Model);

	if ( Model_LoadModelData(&Model, "models/brigand.ms3d", "test" ) == FALSE )		// Loads The Model And Checks For Errors
	{
		MessageBox( NULL, "Couldn't load the model data\\zombie.ms3d", "Error", MB_OK | MB_ICONERROR );
		return 0;										// If Model Didn't Load Quit
	}


	Model_New(&Bonus);

	if ( Model_LoadModelData(&Bonus, "models/bonus.ms3d", "test" ) == FALSE )		// Loads The Model And Checks For Errors
	{
		MessageBox( NULL, "Couldn't load the model data\\bonus.ms3d", "Error", MB_OK | MB_ICONERROR );
		return 0;										// If Model Didn't Load Quit
	}











	// Newton player Vars
	// -----------------------------------------------------
	newtonPlayer = NewtonCreateSphere (nWorld, player.plyRadiusCol*2.0, player.plyRadiusCol*2.0, player.plyRadiusCol*2.0, NULL);
	newtonPlayerCol = NewtonCreateBody (nWorld, newtonPlayer);

	mojolog ("- mojo \tlog : Adding Player to Newton !\n");

	NewtonBodySetMassMatrix (newtonPlayerCol, 40.0f, 1.0f, 1.0f, 1.0f);
	NewtonBodySetForceAndTorqueCallback (newtonPlayerCol, PhysicsApplyForceAndTorque);
	NewtonBodySetTransformCallback (newtonPlayerCol, PhysicsSetTransform);


	NewtonWorldUnfreezeBody (nWorld, newtonPlayerCol);
	NewtonBodySetAutoFreeze (newtonPlayerCol, 0);


		m[0]=1.0;	m[4]=0.0;	m[8]=0.0;	m[12]=100.0;
		m[1]=0.0;	m[5]=1.0;	m[9]=0.0;	m[13]=10.0;
		m[2]=0.0;	m[6]=0.0;	m[10]=1.0;	m[14]=0.0;
		m[3]=0.0;	m[7]=0.0;	m[11]=0.0;	m[15]=1.0;


	//NewtonBodySetMatrix ( newtonPlayerCol, (float*) &MtxPlayer->m[0]);
	NewtonBodySetMatrix ( newtonPlayerCol, (float*) &m[0]);

	// Add an up vector constraint to help in keeping the body upright
	//NewtonJoint* upVector;
	//float upDirection[3]={ 0.0f, 2.0f, 0.0f};
	//upVector = NewtonConstraintCreateUpVector (nWorld, &upDirection[0], newtonPlayerCol);

	NewtonReleaseCollision (nWorld, newtonPlayer);


	MtlDefault  = NewtonMaterialGetDefaultGroupID(nWorld);


	NewtonMaterialSetDefaultSoftness (nWorld, MtlDefault, MtlDefault, 0.1f);
	NewtonMaterialSetDefaultElasticity (nWorld, MtlDefault, MtlDefault, 0.1f);
	NewtonMaterialSetDefaultFriction (nWorld, MtlDefault, MtlDefault, 0.9f, 0.3f);
	NewtonMaterialSetDefaultCollidable (nWorld, MtlDefault, MtlDefault, 1);

	MtlLvl 		= MtlDefault;
	//MtlLvl 		= NewtonMaterialCreateGroupID(nWorld);
	MtlPlyr 	= NewtonMaterialCreateGroupID(nWorld);
	MtlEnm		= NewtonMaterialCreateGroupID(nWorld);


	NewtonMaterialSetDefaultFriction(nWorld, MtlPlyr, MtlLvl , 0.9, 0.3);
	NewtonMaterialSetDefaultElasticity(nWorld, MtlPlyr, MtlLvl, 0.01);

	NewtonBodySetMaterialGroupID( newtonPlayerCol, MtlPlyr );

	NewtonBodySetLinearDamping( newtonPlayerCol, 1.0f);


	// Newton enemy Vars
	// -----------------------------------------------------


	tmpEnmList = enemyList;
	i = 0;






	while(enemyList != NULL )
	{

		mojolog ("- mojo \tlog : Adding Enemy to Newton !\n");

		tmpFloat = enemyList->enmRadiusCol * 2;

		newtonTmp = NewtonCreateSphere (nWorld, tmpFloat, tmpFloat, tmpFloat, NULL);
		newtonTmpCol[i] = NewtonCreateBody (nWorld, newtonTmp);

		NewtonBodySetMassMatrix (newtonTmpCol[i], 20.0f, 1.0f, 1.0f, 1.0f);
		NewtonBodySetForceAndTorqueCallback (newtonTmpCol[i], EnmApplyForceAndTorque);

		m[0]=1.0;	m[4]=0.0;	m[8]=0.0;	m[12]=enemyList->enmMtx.m[12];
		m[1]=0.0;	m[5]=1.0;	m[9]=0.0;	m[13]=enemyList->enmMtx.m[13];
		m[2]=0.0;	m[6]=0.0;	m[10]=1.0;	m[14]=enemyList->enmMtx.m[14];
		m[3]=0.0;	m[7]=0.0;	m[11]=0.0;	m[15]=1.0;

		NewtonBodySetMatrix ( newtonTmpCol[i], (float*) &m[0]);

		//NewtonBodySetMaterialGroupID( newtonTmpCol[i], MtlPlyr );
		NewtonBodySetLinearDamping( newtonTmpCol[i], 0.5f);

		NewtonBodySetUserData( newtonTmpCol[i], enemyList );

		NewtonBodySetMaterialGroupID( newtonTmpCol[i], MtlEnm );

		NewtonReleaseCollision (nWorld, newtonTmp);


		Model_New(&Enm[i]);

		switch(enemyList->enmType)
		{
		case 1:
			if ( Model_LoadModelData(&Enm[i], "models/zumbiz.ms3d", "test" ) == FALSE )		// Loads The Model And Checks For Errors
			{
				MessageBox( NULL, "Couldn't load the model data\\zumbiz.ms3d", "Error", MB_OK | MB_ICONERROR );
				return 0;										// If Model Didn't Load Quit
			}
			enmNum++;
		break;

		case 2:
			if ( Model_LoadModelData(&Enm[i], "models/ninbotz.ms3d", "test" ) == FALSE )		// Loads The Model And Checks For Errors
			{
				MessageBox( NULL, "Couldn't load the model data\\ninbotz.ms3d", "Error", MB_OK | MB_ICONERROR );
				return 0;										// If Model Didn't Load Quit
			}
			enmNum++;
		break;

		case 3:
			if ( Model_LoadModelData(&Enm[i], "models/golem2.ms3d", "test" ) == FALSE )		// Loads The Model And Checks For Errors
			{
				MessageBox( NULL, "Couldn't load the model data\\golem.ms3d", "Error", MB_OK | MB_ICONERROR );
				return 0;										// If Model Didn't Load Quit
			}
			enmNum++;
		break;
		}

		enemyList->id = i;
		enemyList->enmState = 0;
		i++;


		enemyList = enemyList->next;
	}

	enemyList = tmpEnmList;






	// game vars
	// -----------------------------------------------------
	frames = fps = 0;
	t0 = t1 = glfwGetTime();




	// gui vars
	// -----------------------------------------------------
/*
	mojoGuiSetScreen( screen_width, screen_height );


	mbutton1 = mojoGuiAddButton( NULL );
	mbutton1->textureId = fontTex;
	mbutton1->posx = 100.0;
	mbutton1->posy = 100.0;
	mbutton1->sizex = 100.0;
	mbutton1->sizey = 10.0;
	mbutton1->alpha = 0.3;
	button1 = mbutton1->id;			// store the id


	mbutton2 = mojoGuiAddButton( mbutton1 );
	mbutton2->textureId = fontTex;
	mbutton2->posx = 250.0;
	mbutton2->posy = 100.0;
	mbutton2->sizex = 100.0;
	mbutton2->sizey = 10.0;
	button2 = mbutton2->id;			// store the id
*/
	// hack : we advance the particle timer so that they are drawn completely on start of rendering
	mojo_particleDrawStatic( StaticPStm, 0.0, 0.0, 0.0, 0.1);


	mojoGlOrthoEnd();


	mojoFadeSetState( -1 );

	FSOUND_SetSFXMasterVolume( volumeSfx );
	FMUSIC_SetMasterVolume( music, volumeMusic );
    FMUSIC_SetLooping(music, TRUE);
    FMUSIC_PlaySong(music);








  	////////////////////////////////// [ Start of INNER Game Loop ]


  	/* Main loop */
	// -----------------------------------------------------
	while(running)
	{



		// OpenGL frame setup
		/////////////////////////////////////////////////////////////
		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // This clear the background
  		glViewport(0, 0, width, height);

  		glMatrixMode(GL_PROJECTION);
  		glLoadIdentity();
  		gluPerspective(60.0, width/height, 1.0, 1024.0);


        // Select and setup the modelview matrix
  		glMatrixMode( GL_MODELVIEW );
        glLoadIdentity();



		// Frametime
		// ///////////////////////////////////////////////////////////
		t = glfwGetTime(); 						// Get time position
		elapsed = (t - t1);

		if ((t - t0) > 1.0 )					// Calculate and display FPS (frames per second)
		{
			fps = frames / (t - t0);
			t0 = t;
			frames = 0;
		}

		frames++;
		t1 = t;




		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Input polling ]
		//VectorLoad( &VplayerMove, 0.0, 0.0, 0.0);
		strafeangle = forwardspeed = 0.0;
		tmpFloat = 10.0 * speedfactor;
		//VectorLoad( &VplayerMove, 0.0, 0.0, 0.0);
		VectorLoad( &VplayerStrafe, 0.0, 0.0, 0.0);

		success = 0;


		rot_y = WRAP(rot_y);						// wrap player angle ( 0 / 360 )


/*
		if(glfwGetKey(GLFW_KEY_UP))
		{
			rot_x -= tmpFloat;
		}

		if(glfwGetKey(GLFW_KEY_DOWN))
		{
			rot_x += tmpFloat;
		}

		if(glfwGetKey(GLFW_KEY_LEFT))
		{
			rot_y += tmpFloat/6.0;
		}

		if(glfwGetKey(GLFW_KEY_RIGHT))
		{
			rot_y -= tmpFloat/6.0;
		}
*/
		if(glfwGetKey(90) || glfwGetKey(GLFW_KEY_UP)) // Z -> Forward
		{
			success = 1;					// simple value to indicate we need to animate the model
			forwardspeed = tmpFloat;
			if(animCurr!=1)
			{
				animCurr = 1;
				Model_Restart(&Model);
			}

		}

		if(glfwGetKey(83) || glfwGetKey(GLFW_KEY_DOWN)) // S -> Backward
		{
			success = 1;					// simple value to indicate we need to animate the model
			forwardspeed = -tmpFloat/3.0;
			if(animCurr!=1)
			{
				animCurr = 1;
				Model_Restart(&Model);
			}

		}

		if(glfwGetKey(81)  || glfwGetKey(GLFW_KEY_LEFT)) // Q -> Left
		{
			success = 1;					// simple value to indicate we need to animate the model
			strafeangle = 90.0;
			VectorLoad( &VplayerStrafe, sin(RADIAN(rot_y+strafeangle))*(tmpFloat/2.0), 0.0, cos(RADIAN(rot_y+strafeangle))*(tmpFloat/2.0));
			if(animCurr!=1)
			{
				animCurr = 1;
				Model_Restart(&Model);
			}

		}

		if(glfwGetKey(68) || glfwGetKey(GLFW_KEY_RIGHT)) // D -> Right
		{
			success = 1;					// simple value to indicate we need to animate the model
			strafeangle = -90.0;
			VectorLoad( &VplayerStrafe, sin(RADIAN(rot_y+strafeangle))*(tmpFloat/2.0), 0.0, cos(RADIAN(rot_y+strafeangle))*(tmpFloat/2.0));
			if(animCurr!=1)
			{
				animCurr = 1;
				Model_Restart(&Model);
			}

		}



		if(!success)					// if not, anim is 'idle'
		{
			if(animCurr!=2)
			{
				animCurr = 2;
				Model_Restart(&Model);
			}
			Model_AdvanceAnimation(&Model, 50.0,60.0);

		}else if(success){				// else it's 'normal' walk

			if(!FSOUND_IsPlaying(chanWalking) && !is_jumping )
				chanWalking = FSOUND_PlaySound(FSOUND_FREE, soundSlot[ playerSndSTEP ].sndPtr);

			Model_AdvanceAnimation(&Model, 5.0,20.0);
		}


		if(player.plyState == stHIT)
			Model_AdvanceAnimation(&Model, 40.0,45.0);



		if(glfwGetKey(GLFW_KEY_INSERT) ) //
		{
			DynPStm = mojoAddDynPStmSlot( DynPStm, PresetPStm, 2 );

		}


		//VectorLoad( &VplayerMove, sin(RADIAN(rot_y))*forwardspeed, 0.0, cos(RADIAN(rot_y))*forwardspeed);
		VectorLoad( &VplayerMove, sin(RADIAN(rot_y)), 0.0, cos(RADIAN(rot_y)));
		VectorCopy( &VplyrMoveNorm, &VplayerMove );

		VectorMultiply( &VplayerMove, forwardspeed);
		VectorMultiply( &VplyrMoveNorm, 100.0);


		VectorAdd( &VplayerMove, &VplayerStrafe);


		distToStatic = 	mojoGetIntersect( MtxPlayer.m[12], MtxPlayer.m[13]+6.0, MtxPlayer.m[14],
						VplyrMoveNorm.v[0]*500.0,  MtxPlayer.m[13]+6.0, VplyrMoveNorm.v[2]*500.0, 500.0f);

		distToFloor = mojoGetFloor( MtxPlayer.m[12], MtxPlayer.m[13]+5.0, MtxPlayer.m[14]);

		distToFloor = MtxPlayer.m[13] - distToFloor;

		//printf("dist #%f \r", distToFloor );


		if(distToFloor < 7.0 )  is_jumping = 0;

		if(distToFloor > 18.0 )  is_jumping = 1;

		if(distToFloor > 80.0 )
		{
			if(!is_jumping)
		   		player.plyTotalLife -= 0.5;
		}


		if(abs(player.plyTotalLife) <= 0.0)
		{
			if(mojoFadeGetState()!=1)
			{
				printf("dead lost");
				mojoFadeSetState( 1 );
				gameStat = 0;
			}
		}







		//printf("dist %f\n", distToStatic*100.0);

		// Update Newton physics world
		// We must update Newton (or any physics engine) at a constant resolution so that all
		// the clients have the same results
		// Many thx to PSC80 for the explanation here !!!

		mAccumulativeLoopTime += elapsed;            // Accumulate the LoopTime (time slice)
		while (mAccumulativeLoopTime > NEWTON_RESOLUTION)
		{
  	 		NewtonUpdate(nWorld, NEWTON_RESOLUTION);
  	 		mAccumulativeLoopTime -= NEWTON_RESOLUTION;
		} // while loop

		NewtonBodyGetMatrix( newtonPlayerCol, (float*) &MtxPlayer.m[0]);





		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Camera Position ]

		float rot_Tmp, dashValue;

		rot_Tmp = WRAP(rot_y+180.0);
		VectorLoad( &Veye, MtxPlayer.m[12]+sin(RADIAN(rot_Tmp))*100.0*level, MtxPlayer.m[13]+100.0*level, MtxPlayer.m[14]+cos(RADIAN(rot_Tmp))*100.0*level);
		//VectorRotateX( &Veye, rot_x );

		if(!is_jumping)
		{
		if(player.plyCurrentDashPower > player.plyCurrentDash)
		{

			if(is_dashing)
			{

				dashValue = player.plyCurrentDash/3.0;					// camera shaking when dashing
				success = RAND_INT(-dashValue,dashValue)*is_dashing;
				Veye.v[0]+=success;
				Veye.v[0]-=success;
				Veye.v[0]+=success;
			}
		}
		}

        gluLookAt( 	Veye.v[0], 			Veye.v[1],			Veye.v[2],    // Eye-position
                   	MtxPlayer.m[12],	MtxPlayer.m[13],	MtxPlayer.m[14],
					/*Vview.v[0], 		Vview.v[1], 		Vview.v[2],   // View-point*/
                   	0.0f, 				1.0f, 				0.0f 		);  // Up-vector






		// optim : we only push/pop the matrix stack between ple skybox and the pleyr rendering,
		// because the skybox transform matrix is the same as the player, to center the view.

		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Skybox rendering ]

		glPushMatrix();

		glActiveTextureARB( GL_TEXTURE1_ARB );
		glDisable(GL_BLEND);
		glDisable (GL_TEXTURE_2D);
		glActiveTextureARB( GL_TEXTURE0_ARB );

		mojoSkyBoxDraw(MtxPlayer.m[12], MtxPlayer.m[13], MtxPlayer.m[14], 1000, 600, 1000, skybox);

		//glPopMatrix();







		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Player rendering ]

		MatrixLoadNull( &MtxTmp );

		MtxTmp.m[12] = 	MtxPlayer.m[12]+VplyrMoveNorm.v[0]*distToStatic;
		MtxTmp.m[13] = 	MtxPlayer.m[13];
		MtxTmp.m[14] = 	MtxPlayer.m[14]+VplyrMoveNorm.v[2]*distToStatic;


#if DEBUG
		// Visual Debug : this shows us the 'player to static' raycast
		// used in the bullet algorythm
		glColor3f(1.0f, 0.0f, 0.0f);
		glBegin(GL_LINES);
		glVertex3f( MtxPlayer.m[12], MtxPlayer.m[13], MtxPlayer.m[14]);
		glVertex3f( MtxTmp.m[12], MtxTmp.m[13], MtxTmp.m[14]);
		glEnd();
#endif

		glPushMatrix();

		mojoGlBodyDraw( &MtxTmp );
		glColor3f( 1.0, 0.0, 1.0);
		mojoBoxDraw( 0.5, 0.5, 0.5, 0.0, 0.0, 0.0);

		glPopMatrix();


//		glPushMatrix();

		MatrixLoadNull( &MtxPlayer );
		MatrixTranslate(&MtxPlayer, 0.0, -5.0, 0.0); 	// Start model positionning Fix is need be
		MatrixRotateY( &MtxPlayer, rot_y );

		glColor3f( 1.0, 1.0, 1.0);
		glActiveTextureARB( GL_TEXTURE0_ARB );

		glDisable(GL_BLEND);

		if(player.plyState == stHIT )
		glEnable(GL_BLEND);

		//glEnable(GL_TEXTURE_2D);
		//glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, /*GL_BLEND*/ GL_MODULATE);

		mojoGlBodyDraw( &MtxPlayer );
		//glScalef(1.0,1.0,1.0);
		Model_Draw(&Model);

		player.plyState	= stIDLE;
		//glDisable(GL_BLEND);
		//glDisable(GL_TEXTURE_2D);
		//glScalef(1.0,1.0,1.0);

		glPopMatrix();
		MatrixTranslate(&MtxPlayer, 0.0, 5.0, 0.0);	// End model positionning Fix is need be

#if DEBUGNWT

		glPushMatrix();

		NewtonWorldForEachBodyDo( nWorld, DebugCollision );

		glPopMatrix();

#endif




		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Bullets rendering ]

		if( is_firing )
		{
			//printf(" addedbullet\n");
			// Hack : we sub 6.0 for the player collision sphere, for accurate result !!!

			mojoAddBulletSlot( elapsed, rot_y, distToStatic*100.0-6.0, MtxPlayer.m[12], MtxPlayer.m[13], MtxPlayer.m[14] );
		}

		mojoUpdateBulletList( elapsed );





		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Enemies rendering ]

		VECTOR 	BotHeading, BotPlayer, CrossResult;
		tmpEnmList = enemyList;
		int colResult = 0, hitResult = 0;
		//glEnable(GL_TEXTURE_2D);

		glColor3f( 1.0, 1.0, 1.0);

		while(enemyList != NULL)
		{

			colResult = 0;
			animCurrEnm = enemyList->enmState;

			if(enemyList->enmState < stDEAD)
			{

			// fill the Enm structure with relevant infos
			NewtonBodyGetMatrix ( newtonTmpCol[ enemyList->id ], (float*) &m[0]);


			enemyList->enmAngleToPlayer = mojoGetAngle( m[12] - MtxPlayer.m[12] , m[14] - MtxPlayer.m[14] );
			enemyList->enmAngleToPlayer = WRAP( (-1*enemyList->enmAngleToPlayer - 90.0));
			//enemyList->enmDistToPlayer = mojoGetDistance2D(m[12], m[14], MtxPlayer.m[12], MtxPlayer.m[14]);
			enemyList->enmDistToPlayer = mojoGetDistance3D(m[12], m[13], m[14],
		   								 MtxPlayer.m[12], MtxPlayer.m[13], MtxPlayer.m[14]);

			//tmpFloat = fabs(enemyList->enmAngle - enemyList->enmAngleToPlayer);
			//tmpFloat2 = WRAP(360.0 + tmpFloat) ;
			//enemyList->enmAngle = enemyList->enmAngleToPlayer;

			MtxTmp.m[12] = m[12];
			MtxTmp.m[13] = m[13];
			MtxTmp.m[14] = m[14];
			MtxTmp.m[15] = m[15];


			if(enemyList->enmState != stDEAD)
			{
				// check bullet/enemies collision
				colResult = mojoCheckBulletCollision( enemyList->enmRadiusCol, m[12], m[13], m[14]);
			}


			// handle the FSM logic
			//mojoEnmFSM( enemyList );

			// _______________________________________ enemy collision with bullets
			if(colResult /*|| hitResult*/)
			{
				enemyList->enmState = stHIT;									// switch to HIT mode
				glColor3f( 1.0, 1.0, 1.0);
				//printf("enm id%d hit\n",enemyList->id);
				enemyList->enmTotalLife -= 2.0;									// decrease enemyLife

				if(enemyList->enmTotalLife<0.0)
				{
					enemyList->enmState = stDEAD;								// switch to DEAD mode

					enmNum--;


					player.plyScore += 200;
																				// if enemyLife <0.0
					DynPStm = mojoAddDynPStmSlot( DynPStm, PresetPStm, 3 );		// spawn a PStm
					DynPStm->x = MtxTmp.m[12];
					DynPStm->y = MtxTmp.m[13];
					DynPStm->z = MtxTmp.m[14];

					i = FSOUND_PlaySound(FSOUND_FREE, soundSlot[ bonusSndSPAWN ].sndPtr); 			// bonus spawn sound


					mojoAddBonusSlot( RAND_INT(0,1), MtxTmp.m[12], MtxTmp.m[13], MtxTmp.m[14] ); 	// spawn a bonus

					//printf("bonus : %d\n",RAND_INT(0,1));
				}

			}else{

				if(enemyList->enmDistToPlayer < 150 )
				{
			// _______________________________________ enemy in CHASE mode

					//if(animCurrEnm != stCHASE )
						//Model_Restart(&Enm[enemyList->id]);

					enemyList->enmState = stCHASE;
					//glColor3f( 1.0, 0.0, 0.0);

					//Model_AdvanceAnimation(&Enm[enemyList->id], 5.0,20.0);

				}else{
			// _______________________________________ enemy back in IDLE mode
					if(enemyList->enmState != stWANDER)
					{
						enemyList->enmState = stIDLE;
						//Model_AdvanceAnimation(&Enm[enemyList->id], 50.0,60.0);		// we play the idle anim
					}else if(enemyList->enmState == stWANDER){
						//Model_AdvanceAnimation(&Enm[enemyList->id], 5.0,20.0);		// we play the walk/chase anim
					}

					//glColor3f( 0.0, 1.0, 0.0);
				}
			}


			// _______________________________________ enemy in ATTACK mode
			if(enemyList->enmDistToPlayer < 10 )
			{

				//if(animCurrEnm != stATTACK )
					//Model_Restart(&Enm[enemyList->id]);

				Model_AdvanceAnimation(&Enm[enemyList->id], 25.0,35.0);

				enemyList->enmRandomTimer = 0.5;
				enemyList->enmState = stATTACK;
				//glColor3f( 0.0, 0.0, 1.0);

				player.plyState = stHIT;
				player.plyTotalLife -= enemyList->enmDamage*0.005;
				//glColor3f( 1.0, 0.0, 0.0);
			}


		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ FSM Logic ]



			// _______________________________________ enemy in CHASE mode

			if(enemyList->enmState >= stCHASE)
			{

				// Hack : Calculate the Cross Product of the enemy heading angle,
				// and enemy player angle. The sign of the component gives us the direction of the angle
				// So can simply increment/decrement as we go, and the enemy interpolate to the player direction
				// this only works for a single axis though, but saves us the matrix/quaternion transformation.
				// ( see quaternion for Arbitrary axis angle interpolation )
				// Many thx to Mephisto for the idea !!!
				VectorLoad( &BotHeading, sin(RADIAN(enemyList->enmAngle))*10.0, 0.0, cos(RADIAN(enemyList->enmAngle))*10.0);
				VectorNormalize( &BotHeading );
				VectorLoad( &BotPlayer, MtxPlayer.m[12] - m[12], 0.0, MtxPlayer.m[14] - m[14]);
				VectorNormalize( &BotPlayer );
				VectorCrossProduct( &CrossResult, &BotHeading, &BotPlayer );

				//if(tmpFloat > 0.1)
				{
					//if(enemyList->enmAngleToPlayer<180.0)
					if(CrossResult.v[1]>0.0)
					{
						enemyList->enmAngle += enemyList->enmTurnSpeed;
					}else{
						enemyList->enmAngle -= enemyList->enmTurnSpeed;
					}
				}

			}


			// _______________________________________ enemy in IDLE mode

			if(enemyList->enmState == stIDLE){

				enemyList->enmRandomTimer -= elapsed;

				if(animCurrEnm != stIDLE )
					Model_Restart(&Enm[enemyList->id]);

				Model_AdvanceAnimation(&Enm[enemyList->id], 50.0,60.0);

				if(enemyList->enmRandomTimer <= 0.0)
				{
					enemyList->enmRandomAngle = (float)RAND_INT(0,360);
					enemyList->enmRandomTimer = (float)RAND_INT(0,6);
					enemyList->enmState = stWANDER;
				}

			}




			// _______________________________________ enemy in CHASE mode
			if(enemyList->enmState == stCHASE){

				if(animCurrEnm != stCHASE )
					Model_Restart(&Enm[enemyList->id]);

				Model_AdvanceAnimation(&Enm[enemyList->id], 5.0,20.0);
			}


			// _______________________________________ enemy in HIT mode
			if(enemyList->enmState == stHIT){

				if(animCurrEnm != stHIT )
					Model_Restart(&Enm[enemyList->id]);

				Model_AdvanceAnimation(&Enm[enemyList->id], 40.0,45.0);
			}


			// _______________________________________ enemy in ATTACK mode
			if(enemyList->enmState == stATTACK){

				if(animCurrEnm != stATTACK )
					Model_Restart(&Enm[enemyList->id]);

				Model_AdvanceAnimation(&Enm[enemyList->id], 25.0,35.0);
			}



			// _______________________________________ enemy in WANDER mode

			if(enemyList->enmState == stWANDER){

				//printf("enemy wander random timer%f", enemyList->enmRandomTimer);
				enemyList->enmRandomTimer -= elapsed;

				VectorLoad( &BotHeading, sin(RADIAN(enemyList->enmAngle))*10.0, 0.0, cos(RADIAN(enemyList->enmAngle))*10.0);
				VectorNormalize( &BotHeading );
				VectorLoad( &BotPlayer, sin(RADIAN(enemyList->enmRandomAngle))*10.0, 0.0, cos(RADIAN(enemyList->enmRandomAngle))*10.0);
				VectorNormalize( &BotPlayer );
				VectorCrossProduct( &CrossResult, &BotHeading, &BotPlayer );

				//if(tmpFloat > 0.1)
				{
					//if(enemyList->enmAngleToPlayer<180.0)
					if(CrossResult.v[1]>0.0)
					{
						enemyList->enmAngle += enemyList->enmTurnSpeed;
					}else{
						enemyList->enmAngle -= enemyList->enmTurnSpeed;
					}
				}


				if(animCurrEnm != stWANDER )
					Model_Restart(&Enm[enemyList->id]);

				Model_AdvanceAnimation(&Enm[enemyList->id], 5.0,20.0);

				if(enemyList->enmRandomTimer <= 0.0)
				{
					//enemyList->enmRandomAngle = RAND_INT(0,360);
					enemyList->enmRandomTimer = (float)RAND_INT(0,6);
					enemyList->enmState = stIDLE;
					//Model_Restart(&Enm[enemyList->id]);
				}


			}



			enemyList->enmAngle = WRAP(enemyList->enmAngle);

			// position and draw
			MatrixLoadNull( &MtxTmp );
			MatrixRotateY( &MtxTmp, enemyList->enmAngle);



#if DEBUG


			glColor3f(1.0f, 0.0f, 0.0f);
			glBegin(GL_LINES);
			glVertex3f( MtxTmp.m[12], MtxTmp.m[13], MtxTmp.m[14]);
			glVertex3f( MtxTmp.m[12]+sin(RADIAN(enemyList->enmAngle))*10.0, MtxTmp.m[13], MtxTmp.m[14]+cos(RADIAN(enemyList->enmAngle))*10.0);
			glEnd();

			glColor3f(1.0f, 0.0f, 0.0f);
			glBegin(GL_LINES);
			glVertex3f( MtxTmp.m[12], MtxTmp.m[13], MtxTmp.m[14]);
			glVertex3f( MtxTmp.m[12]+sin(RADIAN(enemyList->enmAngleToPlayer))*10.0, MtxTmp.m[13], MtxTmp.m[14]+cos(RADIAN(enemyList->enmAngleToPlayer))*10.0);
			glEnd();

			glColor3f(1.0f, 0.0f, 0.0f);
			glBegin(GL_LINES);
			glVertex3f( MtxTmp.m[12], MtxTmp.m[13], MtxTmp.m[14]);
			glVertex3f( MtxTmp.m[12]+sin(RADIAN(enemyList->enmRandomAngle))*10.0, MtxTmp.m[13], MtxTmp.m[14]+cos(RADIAN(enemyList->enmRandomAngle))*10.0);
			glEnd();



#endif




			glPushMatrix();

			glColor4f( 1.0, 1.0, 1.0,1.0);

			// _______________________________________ enemy positioning and drawing
			mojoGlBodyDraw( &MtxTmp );

			//mojoBoxDraw( 5.0, 10.0, 5.0, 0.0, 5.0, 0.0);

			Model_Draw(&Enm[enemyList->id]);

			glPopMatrix();

		}//endif enmState

			enemyList = enemyList->next;

		}

		enemyList = tmpEnmList;

#if DEBUG

		//printf("angle# %f AtP# %f diff1# %f CrossY# %f \r",enemyList->enmAngle, enemyList->enmAngleToPlayer, tmpFloat, CrossResult.v[1]);
		//printf("angle# %f AtP# %f diff# %f\r",enemyList->enmAngle, enemyList->enmAngleToPlayer, tmpFloat);
		//glDisable(GL_TEXTURE_2D);
#endif


		if(enmNum == 0)
		{
			if(mojoFadeGetState()!=1)
			{
				printf("dead win");
				mojoFadeSetState( 1 );

				if(gameStat!=2)
					gameStat = 1;
			}
		}





		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Map rendering ]
		glPushMatrix();

		MatrixLoadIdentity( &MtxLvl );
		//MatrixTranslate( &MtxPlayer, 0.0, 0.0, 0.0);
		MtxLvl.m[13] = -60.0;
		mojoGlBodyDraw( &MtxLvl );

		glActiveTextureARB( GL_TEXTURE1_ARB );
		glEnable(GL_BLEND);
		glEnable (GL_TEXTURE_2D);


		glColor3f(1.0,1.0,1.0);
		glCallList(dmf.glDisplayList);
		glPopMatrix();



		glActiveTextureARB( GL_TEXTURE1_ARB );
		glDisable(GL_BLEND);
		glDisable (GL_TEXTURE_2D);

		glActiveTextureARB( GL_TEXTURE0_ARB );
		//glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, /*GL_BLEND*/ GL_MODULATE);
		glEnable(GL_TEXTURE_2D);






		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Bonus rendering ]

		success = -1;
		mojoUpdateBonusList( elapsed );
		success = mojoCheckBonusCollision( player.plyRadiusCol, MtxPlayer.m[12], MtxPlayer.m[13], MtxPlayer.m[14]);

		if(success>=0)
		{

			mojoBulletSetVar((4.0-player.plyCurrentWeapon)/10.0,0.5);

			i = FSOUND_PlaySound(FSOUND_FREE, soundSlot[ bonusSndpICKUP ].sndPtr);
			mojoPlayerUpdate( &player, success);
			printf("player Score: %i\n",player.plyScore);
		}





		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Particles rendering ]

		// NOTES : we group all the texUnit functions for particles and hud rendering
		// cos only the blend modes differ. So we disable the second texture unit only once.
		// particles special : don't forget to disable the DepthMask writing, to have proper Z-buffer for particles

		if(!is_jumping)
		{
			if(player.plyCurrentDashPower > player.plyCurrentDash)
			{
				if(is_dashing >= 1.0)
				{
					player.plyCurrentDashPower -= player.plyCurrentDash;
					dashBillboardId = mojoAddBillboardSlot(0, 0.5, mojoTexManGetTextureId("ring_blue.tga"), MtxPlayer.m[12], MtxPlayer.m[13]-3.0, MtxPlayer.m[14], player.plyCurrentDash );
					FSOUND_PlaySound(FSOUND_FREE, soundSlot[ playerSndDASH ].sndPtr);

				}
			}
		}


		is_dashing -= elapsed;
		if(is_dashing<0.0) is_dashing = 0.0;


		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		glEnable(GL_ALPHA_TEST);

		glDepthMask(GL_FALSE);

		glPushMatrix();

		Prepare_Billboarding();


		DynPStmList = mojoGetDynPStm();
		mojo_particleDrawStatic( StaticPStm, MtxPlayer.m[12], MtxPlayer.m[13], MtxPlayer.m[14], elapsed);
		mojo_particleDrawDyn( DynPStmList, MtxPlayer.m[12], MtxPlayer.m[13], MtxPlayer.m[14], elapsed);

		mojoUpdateBillboardList( elapsed );

		glPopMatrix();

		glDepthMask(GL_TRUE);
		glDisable(GL_ALPHA_TEST);
		//glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);







		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ 2D/HUD rendering ]

		mojoGlOrthoBegin();

		glActiveTextureARB( GL_TEXTURE1_ARB );
		glDisable(GL_BLEND);
		glDisable (GL_TEXTURE_2D);

		glActiveTextureARB( GL_TEXTURE0_ARB );
		glDisable(GL_TEXTURE_2D);
		//glDisable(GL_BLEND);

		//glDisable(GL_DEPTH_TEST);

		//myEvent = mojoGuiUpdate( mbutton1, xpos, ypos, 0 );

		//glEnable(GL_DEPTH_TEST);

		mojoCinemascope( width, height, CinemascopeLo, CinemascopeHi );


		if(mojoFadeGetState() == -1)
		{
			mojoFadeOut( elapsed*0.5, width, height);
	 		fontSize (36);
			fontColor (1, 0, 0);
			fontShadow ();
			fontShadowColor (0, 0, 0);
			mojoGlPrintCenter(width, (height-36)/2, fileLevelTitle, 1, 36);

		}

		success = 0;

		if(mojoFadeGetState() == 1)
		{
			success = mojoFadeIn( elapsed, width, height);
		}

		if(success)
		{
			running = 0;
			//exiting = 0;
		}



		mojoGlOrthoEnd();

		// END of NOTES : this is where we should set up TexUnits.
/*


		glBlendFunc(GL_ONE, GL_ONE);


		glBindTexture( GL_TEXTURE_2D, fontTex );
		glColor3f(1.0,1.0,1.0);					// for colored font, we should use alpha testing
		sprintf(StrBuffer, "mojoGl V0.5 (%.1f FPS)\0", fps);
		mojoGlPrint(10, 10, StrBuffer, 1, 0.8);
*/

		// 2D Debug Textual Infos ( should be stripped of for release version )
		/////////////////////////////////////////////////////////////

		// Notes : these flags are valid for text
		// ....... [ 	\\i+, \\i- 		] italic
		// ....... [ 	\\b+, \\b- 		] bold
		// ....... [ 	\\c(r g b) 		] text color
		// ....... [ 	\\a(r g b a) 	] text+alpha color
		// ....... [ 	\t 				] tab
		// ....... [ 	\n 				] newline
		// function calls :
		/*
		void fontShadow ();
		void fontColor (float r, float g, float b);
		void fontColorA (float r, float g, float b, float a);
		void fontColorp (float *clr);
		void fontColorAp (float *clr);
		void fontShadowColor (float r, float g, float b);
		void fontShadowColorA (float r, float g, float b, float a);
		void fontShadowColorp (float *clr);
		void fontShadowColorAp (float *clr);
		void fontGradientColor (float r, float g, float b);
		void fontGradientColorA (float r, float g, float b, float a);
		void fontGradientColorp (float *clr);
		void fontGradientColorAp (float *clr);
		*/

/*
		fontSize (18);
		fontColor (1.0, 1.0, 1.0);
		fontDrawString (0, 0, "MojoGL v0.7 ( %.1f fps )", fps);
*/
		fontSize (20); //_______________________________________[ draws life stat ]
		fontColor (1, 1, 1);
		fontShadow ();
		fontDrawString (width-100, height-35, "Life");

		fontSize (30);
		fontColor (1, 1, 1);
		fontShadow ();
		fontDrawString (width-100, height-60, "%d", player.plyTotalLife);


		fontSize (20); //_______________________________________[ draws score stat ]
		fontColor (1, 1, 1);
		fontShadow ();
		fontDrawString (0, height-35, "Score");

		fontSize (30);
		fontColor (1, 1, 1);
		fontShadow ();
		fontDrawString (0, height-60, "%d", player.plyScore);


		fontSize (20); //_______________________________________[ draws enemy stat ]
		fontColor (1, 1, 1);
		fontShadow ();
		fontDrawString ((width-140)/2, height-35, "Enemies");

		fontSize (30);
		fontColor (1, 1, 1);
		fontShadow ();
		fontDrawString (width/2, height-60, "%d", enmNum);


		//sprintf(StrBuffer, "test lorem ipsum est\0", fps);
		//mojoGlPrintCenter(width, 100, StrBuffer, 1, 1.0);

 		fontSize (20); //_______________________________________[ draws dash stat ]
		fontColor (1, 1, 1);
		fontShadow ();
		fontShadowColor (0, 0, 0);
		//fontGradient ();
		//fontGradientColor (0.75, 0.5, 0.5);
		fontDrawString (0, 10, "Dash \\c(255 20 20)%i", (int)player.plyCurrentDashPower);


#if DEBUG


		sprintf(StrBuffer, "Frametime#%.6f", elapsed);
		mojoGlPrint(10, 30, StrBuffer, 1, 0.8);

		sprintf(StrBuffer, "pAngle#%.1f ", rot_y);
		mojoGlPrint(10, 50, StrBuffer, 0, 0.8);

		sprintf(StrBuffer, "PlayerMove#%.1f #%.1f #%.1f", VplayerMove.v[0], VplayerMove.v[1], VplayerMove.v[2]);
		mojoGlPrint(10, 70, StrBuffer, 0, 0.8);

		sprintf(StrBuffer, "MtxPlayer#%.1f #%.1f #%.1f", MtxPlayer.m[12], MtxPlayer.m[13], MtxPlayer.m[14]);
		//sprintf(StrBuffer, "MtxPlayer#%.1f #%.1f #%.1f", m[12], m[13], m[14]);
		mojoGlPrint(10, 90, StrBuffer, 0, 0.8);

		sprintf(StrBuffer, "MtxPlayer#%.1f #%.1f #%.1f", MtxPlayer.m[8], MtxPlayer.m[9], MtxPlayer.m[10]);
		//sprintf(StrBuffer, "MtxPlayer#%.1f #%.1f #%.1f", m[8], m[9], m[10]);
		mojoGlPrint(10, 110, StrBuffer, 0, 0.8);

		sprintf(StrBuffer, "MtxPlayer#%.1f #%.1f #%.1f", MtxPlayer.m[4], MtxPlayer.m[5], MtxPlayer.m[6]);
		//sprintf(StrBuffer, "MtxPlayer#%.1f #%.1f #%.1f", m[4], m[5], m[6]);
		mojoGlPrint(10, 130, StrBuffer, 0, 0.8);

		sprintf(StrBuffer, "MtxPlayer#%.1f #%.1f #%.1f", MtxPlayer.m[0], MtxPlayer.m[1], MtxPlayer.m[2]);
		//sprintf(StrBuffer, "MtxPlayer#%.1f #%.1f #%.1f", m[0], m[1], m[2]);
		mojoGlPrint(10, 150, StrBuffer, 0, 0.8);

		sprintf(StrBuffer, "distance to floor #%.3f )", distToFloor);
		mojoGlPrint(10, 170, StrBuffer, 0, 0.8);
#endif
/*
		//glDisable(GL_BLEND);
		glDisable(GL_TEXTURE_2D);



		mojoGlOrthoEnd();
*/


		// ///////////////////////////////////////////////////////////////////////////////
  		//////////////////////////////////////////////////////////////// [ Rendering pipeline Flush ]

    	//glFlush(); 							// This force the execution of OpenGL commands
    	glfwSwapBuffers(); 					// In double buffered mode we invert the positions


    	/* Still running? */
    	running = running && glfwGetWindowParam( GLFW_OPENED );
  	}

  	////////////////////////////////// [ End of INNER Game Loop ]





	// ///////////////////////////////////////////////////////////////////////////////
	//////////////////////////////////////////////////////////////// [ Exit/Free resources ]



	//glDeleteTextures(1,&textureGlowSlot[0].texId);
	//glDeleteTextures(1,&textureGlowSlot[1].texId);
	//glDeleteTextures(1,&textureGlowSlot[2].texId);
	//glDeleteTextures(1,&textureGlowSlot[3].texId);

    FMUSIC_FreeSong(music);

	// destroy the DMF/lvl display list
	glDeleteLists(dmf.glDisplayList, 1);

	// free gui resource
	mojoGuiFree( mbutton1 );

	// destroy the Newton world
	NewtonDestroy (nWorld);

	// free enemy list
	mojoFreeEnmList( enemyList );
	enemyList = NULL;

	// free bind list
	mojoFreeBindList( bindList );
	bindList = NULL;

	// free PStm Static list
	mojoFreePStmList( StaticPStm );
	StaticPStm = NULL;

	// free PStm Preset list
	mojoFreePStmList( PresetPStm );
	PresetPStm = NULL;

	// free PStm Dyn list
	mojoFreePStmList( DynPStmList );
	DynPStmList = NULL;

	// free Bonus list
	mojoFreeBonusList();

	// free MS3D list
	Model_Delete(&Model);

	// free font display list/texture
	mojoFontKill( fontTex );

	// free the DMF/lvl datas
    FreeDMF(&dmf);

	// free gltexture list
	mojoTexManFreeList();



	switch(gameStat)
	{

		case -1:
			exiting = 0;
			running = 0;

			while(!glfwGetKey(GLFW_KEY_ESC) && !glfwGetKey(GLFW_KEY_SPACE))
			{
				glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // This clear the background
  		glViewport(0, 0, width, height);

				mojoGlPrintLoading(width, (height-16)/2, "Back to menu ( press Escape / Space)", fontTex, 1, 20);
		   		glfwSwapBuffers(); 					// In double buffered mode we invert the positions
			}


		break;

		case 0:
			exiting = 0;
			running = 0;
			fileNextLevel = "";

 			fontSize (20);

			while(!glfwGetKey(GLFW_KEY_SPACE) )
			{
				glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // This clear the background
  		glViewport(0, 0, width, height);

				mojoGlPrintLoading(width, (height-16)/2, "Too Bad ! ( press Space )", fontTex, 1, 20);
		   		glfwSwapBuffers(); 					// In double buffered mode we invert the positions
			}



		break;

		case 1:
			exiting = 1;

 			fontSize (20);

			while(!glfwGetKey(GLFW_KEY_SPACE))
			{
				glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // This clear the background
  		glViewport(0, 0, width, height);

				mojoGlPrintLoading(width, (height-16)/2, "Good Job ! ( press Space )", fontTex, 1, 20);
		   		glfwSwapBuffers(); 					// In double buffered mode we invert the positions
			}



		break;

		case 2:
			exiting = 1;

 			fontSize (20);

			while(!glfwGetKey(GLFW_KEY_SPACE) && !glfwGetKey(GLFW_KEY_ESC))
			{
				glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // This clear the background

				mojoGlPrintLoading(width, (height-16)/2, "Congratulations !", fontTex, 1, 20);
				mojoGlPrintLoading(width, (height-16)/2, "( press Space / Esc )", fontTex, 1, 20);
		   		glfwSwapBuffers(); 					// In double buffered mode we invert the positions
			}



		break;

	}




}

  	////////////////////////////////// [ End of OUTER Game Loop ]



}
	// free sound manager
	mojoSoundClose();


	mojolog ("- mojo \tEND : Closing Application !\n");


	// Close the Application window
  	glfwTerminate();




	return 0;
}


