/*
 * 	---------------- MoJo engine --------------
 *   		---- Basic Game Engine ----
 *
 * Author: Yann PARMENTIER <kohai@kohaistyle.com>
 *
 * ALL RIGHTS RESERVED
 *
 *	Uses
 * 			DMFloader  		( www.delgine.com )
 * 			GLFW 				( glfw.sourceforge.net )
 *			Fmod Sound API 		( www.fmod.org )
 *
 * To compile this project you must include the following libraries:
 * wlib.lib uuid.lib fmodlcc.lib glfw.lib glu32.lib ole32.lib olepro32.lib opengl32.lib
 * You need also the header file glfw.h in your GL/ compiler directory.
 *
 * File header: mojoGui.c
 *
 */

#include 	<stdio.h>
#include 	<stdlib.h>
#include 	<GL/glfw.h>
#include 	"mojogui.h"



float mojoGuiRatiox, mojoGuiRatioy, mojoGuiHeight = 480, mojoGuiWidth = 640;





struct mojoGuiEl	*mojoGuiAddButton( struct mojoGuiEl *guiElement)
{

	struct mojoGuiEl *newElement;

	if((newElement = malloc(sizeof(struct mojoGuiEl))) == NULL )
	{
		printf("out of memory\n");
		exit(1);
	}


	newElement->next = NULL;
	newElement->id = 0;
	newElement->alpha = 1.0;
	newElement->state = mojoIdle;
	newElement->active = 1.0;
	if(guiElement!=NULL)
	{
		guiElement->next = newElement;
		newElement->id = guiElement->id+1;

	}

	return newElement;

}



int 				mojoGuiFree( struct mojoGuiEl *guiElement)
{

	struct mojoGuiEl *tmp;

	while(guiElement != NULL )
	{
		printf("#dealloc gui : %d\n", guiElement->id);
		tmp = guiElement->next;
		free(guiElement);
		guiElement = tmp;
	}

	return 0;

}



struct mojoGuiEvent	*mojoGuiUpdate( struct mojoGuiEl *guiElement, float mousex, float mousey, int click)
{

	int 	state=0;
	struct 	mojoGuiEvent *event = NULL;
	float   midx, midy;
	float	xmin, ymin, xmax, ymax;
	float	localmousey;

	localmousey = mojoGuiHeight - mousey;


	while(guiElement != NULL )
	{

		guiElement->state = mojoIdle;

		midx = (guiElement->sizex / 2)*mojoGuiRatiox;
		midy = (guiElement->sizey / 2)*mojoGuiRatioy;

		xmax = (guiElement->posx*mojoGuiRatiox)+midx;
		xmin = (guiElement->posx*mojoGuiRatiox)-midx;

		ymax = (guiElement->posy*mojoGuiRatioy)+midy;
		ymin = (guiElement->posy*mojoGuiRatioy)-midy;


		if((mousex < xmax) && (mousex > xmin) && ( localmousey < ymax) && ( localmousey > ymin))
		{
			guiElement->state = mojoHover;
		}

		mojoGuiRender( guiElement );
		guiElement = guiElement->next;

	}

	glEnable(GL_TEXTURE_2D);
	return event;

}



void			mojoGuiSetScreen( float width, float height )
{

	mojoGuiWidth = width;
	mojoGuiHeight = height;

	mojoGuiRatiox = mojoGuiWidth / 800;
	mojoGuiRatioy = mojoGuiHeight / 600;

}


int				mojoGuiRender( struct mojoGuiEl *guiElement)
{

	float             midx, midy;



   // Render Element
   // ///////////////////////////////////////////////////////////

   if (guiElement->active)
   {

		 glDisable(GL_BLEND);

	  	if (guiElement->alpha < 1.0 )
	  	{
			glEnable(GL_BLEND);
			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);	// Select The Type Of Blending
	  	}


	  		glColor4f(guiElement->alpha, guiElement->alpha, guiElement->alpha,guiElement->alpha);


	  	midx = (guiElement->sizex / 2)*mojoGuiRatiox;
		midy = (guiElement->sizey / 2)*mojoGuiRatioy;


		glEnable(GL_TEXTURE_2D);
		glBindTexture(GL_TEXTURE_2D, guiElement->textureId);
		glLoadIdentity();

		glTranslatef(guiElement->posx*mojoGuiRatiox,guiElement->posy*mojoGuiRatioy,0.0);

		if(guiElement->state == mojoHover)
		{
	  		glColor4f(1.0, 1.0, 1.0,1.0);
		glBegin(GL_LINE_LOOP);						// Start Drawing Our Quad
		glVertex2f(-midx, -midy);		// Top Left Of Screen
		glVertex2f(midx, -midy);		// Bottom Right Of Screen
		glVertex2f(midx, midy);		// Top Right Of Screen
		glVertex2f(-midx, midy);		// Bottom Left Of Screen
		glEnd();
		printf("hover");
		return 0;
		}

		glBegin(GL_QUADS);						// Start Drawing Our Quad
		glTexCoord2d(0.0, 0.0);
		glVertex2f(-midx, -midy);		// Top Left Of Screen
		glTexCoord2d(1.0, 0.0);
		glVertex2f(midx, -midy);		// Bottom Right Of Screen
		glTexCoord2d(1.0, 1.0);
		glVertex2f(midx, midy);		// Top Right Of Screen
		glTexCoord2d(0.0, 1.0);
		glVertex2f(-midx, midy);		// Bottom Left Of Screen
		glEnd();

		glDisable(GL_TEXTURE_2D);



	}

	glEnable(GL_BLEND);

	return 0;
}
