/*
 * 	---------------- MoJo engine --------------
 *   		---- Basic Game Engine ----
 *
 * Author: Yann PARMENTIER <kohai@kohaistyle.com>
 *
 * ALL RIGHTS RESERVED
 *
 *	Uses
 * 			DMFloader  		( www.delgine.com )
 * 			GLFW 				( glfw.sourceforge.net )
 *			Fmod Sound API 		( www.fmod.org )
 *
 * To compile this project you must include the following libraries:
 * wlib.lib uuid.lib fmodlcc.lib glfw.lib glu32.lib ole32.lib olepro32.lib opengl32.lib
 * You need also the header file glfw.h in your GL/ compiler directory.
 *
 * File header: mojomain.h
 *
 */


//////////////////////////////////////////////////////////////////////////////////[ TYPES ]

#pragma once

#include 	<inc/fmod.h>
#include 	"mojomatrix.h"
#ifdef MEMWATCH
#include 	"memwatch.h"
#endif


#define 		MAX_TEX_SLOTS 		60
#define 		MAX_ENM_SLOTS 		30
#define 		MAX_SOUND_SLOTS 	20



// _______________________________________________________[ Struct TextureSlot ]
typedef struct{
	GLuint			texId;
	char	 		texName[50];
	int				texLoaded;
} TEXTURE;

TEXTURE		textureSlot[MAX_TEX_SLOTS];



typedef struct{
	GLuint			texId;
	int	 			size;
} GLOWTEX;

GLOWTEX		textureGlowSlot[4];






// _______________________________________________________[ Struct SoundSlot ]
typedef struct{
	int				sndId;
	char	 		sndName[20];
	int				sndLoaded;
	int				sndChannel;
	FSOUND_SAMPLE 	*sndPtr;
} MOJOSOUND;

MOJOSOUND		soundSlot[MAX_SOUND_SLOTS];






enum MOJOGAMESOUND
{
	zumbizSndIDLE,
	zumbizSndHIT,
	zumbizSndDEATH,

	ninjaSndIDLE,
	ninjaSndHIT,
	ninjaSndDEATH,

	golemSndIDLE,
	golemSndHIT,
	golemSndDEATH,

	playerSndSPAWN,
	playerSndSTEP,
	playerSndDASH,
	playerSndJUMP,
	playerSndFIRE,
	playerSndHIT,
	playerSndDEATH,

	bonusSndSPAWN,
	bonusSndpICKUP,

	bulletSndMISS
};




// _______________________________________________________[ Struct PlayerSlot ]
struct PLAYER{
	int				plyType;
	int				id;
	MATRIX			plyMtx;
	float			plyAngle;
	int				plyState;
	int				plyScore;
	int				plyCurrentWeapon;
	float			plyCurrentWPower;
	float			plyCurrentDash;
	float			plyCurrentDashPower;
	float			plyForwardSpeed;
	float			plyTurnSpeed;
	float			plySpeedFactor;
	float			plyRandomTimer;
	float			plyRadiusCol;
	int				plyTotalLife;
	int				bindId;
	int 			plyModelId;
};





// _______________________________________________________[ Struct EnemySlot ]
struct ENEMY{
	int				enmType;
	int				id;
	MATRIX			enmMtx;
	float			enmAngle;
	int				enmState;
	float			enmDistToPlayer;
	float			enmAngleToPlayer;
	float			enmForwardSpeed;
	float			enmTurnSpeed;
	float			enmSpeedFactor;
	float			enmRandomTimer;
	float			enmRandomAngle;
	float			enmRadiusCol;
	int				enmTotalLife;
	int				enmDamage;
	int				bindId;
	int 			enmModelId;
	struct ENEMY	*next;
};






// _______________________________________________________[ Struct BindSlot ]
struct BIND{
	int				id;
	MATRIX			Mtx;
	struct BIND		*next;
};


extern struct BIND *bindList;

// FSM states



enum STATES
{
	stIDLE,
	stWANDER,
	stCHASE,
	stATTACK,
	stHIT,
	stDEAD,
	stDOWN
};





char			mojoResDir[50];
int				skybox[6];



// mojotexman.c
int 			mojoTexManLoad( char *filename, int tex_id, int mipmap, GLint flag  );
int 			mojoTexManGetFreeId( void );
void 			mojoTexManSetTexture( int slot, char *name, int texId );
int 			mojoTexManGetTextureId( char *name );
void 			mojoTexManFreeTexture( int slot );
int				mojoLoadResXmlCfg(char *filename);
void 			mojoTexManFreeList( void );


// mojosound.c
int				mojoSoundInit( void );
int				mojoSoundClose( void );
int 			mojoSoundLoad( char *filename, int sndId);
int				mojoSoundGetFreeId( void );
void 			mojoSoundSet( int slot, char *name, int sndId );
int 			mojoSoundGetSoundId( char *name );
void 			mojoSoundFreeSound( int slot );
void 			mojoSoundFreeList( void );

// mojosupport.c
void 			mojoGlBodyDraw(MATRIX *p);
void 			mojoSetResourceDir(void);
void 			mojoSkyboxLoad(void);
void 			mojoSkyBoxDraw(float x, float y, float z, float width, float height, float length, int texid[]);
void			mojoQuadDraw2( float x, float y, float x1, float y1, int texId, float alpha);
void 			mojoBoxDraw(float length, float width, float height, float offX, float offY, float offZ);
void 			mojoCinemascope( int resX, int resY, int cinemascopeLo, int cinemascopeHi );
void 			mojoGlowRTCreate(int textureSize, int RTid);
int				mojoFadeIn( float elapsed, float resX, float resY);
void			mojoFadeOut( float elapsed, float resX, float resY);
int				mojoFadeGetState( void );
void			mojoFadeSetState( int state /* -1=fadeOut, 0=none/stop, 1=fadeIn */ );
void			mojoAddBonusSlot( int bonusPresetId, float x, float y, float z );
int				mojoCheckBonusCollision( float radius, float x, float y, float z);
void			mojoUpdateBonusList( double elapsed );
void			mojoPlayerUpdate(struct PLAYER *player, int presetId);
int				mojoLoadBonusPresetXmlCfg(char *filename);
void			mojoFreeBonusList( void );
// mojomain.c
float			mojoGetAngle(float dx, float dy);
float			mojoGetDistance2D(float sx, float sy, float dx, float dy);
float			mojoGetDistance3D(float sx, float sy, float sz, float dx, float dy, float dz);
int				mojoGetBoxCollide(	float size1, float x1, float y1, float z1,
									float size2, float x2, float y2, float z2);

struct ENEMY	*mojoLoadEnmXmlCfg(char *filename);
void			mojoRemEnmSlot(struct ENEMY *oldhead, int id);
struct ENEMY	*mojoAddEnmSlot(struct ENEMY *oldhead);
int 			mojoFreeEnmList( struct ENEMY *head);

struct BIND		*mojoAddBindSlot(struct BIND *oldhead);
int 			mojoUpdateBindSlot( struct BIND *head, int bindId, MATRIX mtx);
int 			mojoFreeBindList( struct BIND *head);

void			mojoBulletSetVar( float delay, float life);
void			mojoAddBulletSlot( float elapsed, float angle, float dist, float x, float y, float z );
void			mojoUpdateBulletList( double elapsed );
int				mojoCheckBulletCollision( float radius, float x, float y, float z);
void			mojoFreeBulletList( void );

char			*mojoLoadLvlXmlCfg(char *filename, int node);

// random_fl
double 			random (void);          /* return the next random number x: 0 <= x < 1*/
void  			rand_seed (int);         /* seed the generator */
