#ifndef __PLASMA_MATRIX_H
#define __PLASMA_MATRIX_H

#include 	"mojovector.h"
/* Matrixes can be thought of as:
 * 0  4  8  12
 * 1  5  9  13
 * 2  6  10 14
 * 3  7  11 15
 */

/* alternately:
 X'x Y'x Z'x Tx
 X'y Y'y Z'y Ty
 X'z Y'z Z'z Tz
 0   0   0   1
*/
struct MatrixStruct{
  double m[16];
};


typedef struct MatrixStruct MATRIX;

extern __inline void   	VectorTransform(VECTOR *v, MATRIX *m);
extern __inline void    MatrixLoadIdentity(MATRIX *m);
extern __inline void 	MatrixLoadNull(MATRIX *m);
extern __inline void    MatrixLoadGLModelview(MATRIX *m);
extern __inline void    MatrixLoadGLProjection(MATRIX *m);
extern __inline void    MatrixLoadGLTransform(MATRIX *m);
extern __inline void    MatrixLoadRotation(MATRIX *m, float x, float y, float z, float angle);
extern __inline void    MatrixCopy(MATRIX *m, MATRIX *n);
extern __inline void    MatrixMultiply(MATRIX *m, MATRIX *n);
extern __inline void    MatrixInverse(MATRIX *m);
extern __inline void    MatrixTranspose(MATRIX *m);
extern __inline VECTOR *MatrixGetXPrime(MATRIX *m);
extern __inline VECTOR *MatrixGetYPrime(MATRIX *m);
extern __inline VECTOR *MatrixGetZPrime(MATRIX *m);
extern __inline VECTOR *MatrixGetTranslation(MATRIX *m);
extern __inline void    MatrixRotate(MATRIX *m, float x, float y, float z, float angle);
extern __inline void    MatrixYaw(MATRIX *m, float yaw);
extern __inline void    MatrixPitch(MATRIX *m, float pitch);
extern __inline void    MatrixRoll(MATRIX *m, float roll);
extern __inline void    MatrixRotateX(MATRIX *m, float angle);
extern __inline void    MatrixRotateY(MATRIX *m, float angle);
extern __inline void    MatrixRotateZ(MATRIX *m, float angle);
extern __inline void    MatrixTranslate(MATRIX *m, float x, float y, float z);
extern __inline void    MatrixScale(MATRIX *m, float x, float y, float z);
extern __inline void    MatrixNormalize(MATRIX *m);
extern __inline float   MatrixGetYaw(MATRIX *m);
extern __inline float   MatrixGetPitch(MATRIX *m);
extern __inline float   MatrixGetRoll(MATRIX *m);

#endif

