/*
 * 	---------------- MoJo engine --------------
 *   		---- Basic Game Engine ----
 *
 * Author: Yann PARMENTIER <kohai@kohaistyle.com>
 *
 * ALL RIGHTS RESERVED
 *
 *	Uses
 * 			DMFloader  		( www.delgine.com )
 * 			GLFW 				( glfw.sourceforge.net )
 *			Fmod Sound API 		( www.fmod.org )
 *
 * To compile this project you must include the following libraries:
 * wlib.lib uuid.lib fmodlcc.lib glfw.lib glu32.lib ole32.lib olepro32.lib opengl32.lib
 * You need also the header file glfw.h in your GL/ compiler directory.
 *
 * File header: mojoParticles2.c
 *
 */



#include <windows.h>
#include <stdio.h>
#include <math.h>
#include <GL/glfw.h>

#include 	"mojomain.h"
#include 	"mojoparticles2.h"
#include 	"mojolog.h"
#include 	"ezxml.h"

#define	MAX_PARTICLES	1000	// Number Of Particles To Create



int	loop;						// Misc Loop Variable

	float	Cslowdown=2.0f;				// Slow Down Particles
	float	Cxspeed=0.0;						// Base X Speed (To Allow Keyboard Direction Of Tail)
	float	Cyspeed=20.0;						// Base Y Speed (To Allow Keyboard Direction Of Tail)
	float	Czspeed=0.0;						// Base X Speed (To Allow Keyboard Direction Of Tail)
	BOOL	rainbow=TRUE;				// Rainbow Mode?
//	NOTES : giving a constant fade value -> explosion particle
//	giving a range -> stream particle
//


//particles particle[MAX_PARTICLES];	// Particle Array (Room For Particle Info)

static GLfloat colors[16][3]=		// Rainbow Of Colors
{
	{1.0f,0.5f,0.5f},{1.0f,0.75f,0.5f},{1.0f,1.0f,0.5f},{0.75f,1.0f,0.5f},
	{0.5f,1.0f,0.5f},{0.5f,1.0f,0.75f},{0.5f,1.0f,1.0f},{0.5f,0.75f,1.0f},
	{0.5f,0.5f,1.0f},{0.75f,0.5f,1.0f},{1.0f,0.5f,1.0f},{1.0f,0.5f,0.75f},
	{1.0f,1.0f,1.0f},{0.0f,0.0f,0.0f},{0.0f,0.0f,0.0f},{0.0f,0.0f,0.0f}
};


float 			bb_rg_x,bb_rg_y,bb_rg_z;
float 			bb_up_x,bb_up_y,bb_up_z;
int				iPStm=0;	// total number of PStm
unsigned int	dynPStmNum = 0;

struct particleStm *FirstDynPStm = NULL;





struct particleStm	*mojoLoadPStmXmlCfg(char *filename)
{

    ezxml_t 		xmlNode, tmpNode, PStmNode;
    char 			*xmlValue;
   	char            StrBuffer[200];
	FILE 			*cfgFile;
	int				first=0;
	struct particleStm	*tmpPStm=NULL;
	struct particleStm	*firstPStm=NULL;

		// Xml Cfg Vars
	// -----------------------------------------------------
	if ( (cfgFile = fopen(filename, "rt")) == NULL ){

		mojolog ("! mojo \tlog : Xml Definition parsing Error !\n");
		return firstPStm;
	}

    xmlNode = ezxml_parse_fp( cfgFile );

	sprintf(StrBuffer,"- mojo \tlog : Xml Definition parsing Success ! [ loading PStm in %s ]!\n",filename);
	mojolog (StrBuffer);

	fclose( cfgFile );

	PStmNode = NULL;
	PStmNode = ezxml_child(xmlNode, "particle");

	mojolog ("- mojo \tlog : Creating Static PStm List!\n");


	while(PStmNode!=NULL)
	{

		tmpNode = NULL;

		// Nb Particles
		tmpNode = ezxml_child(PStmNode, "nbpart");

		tmpPStm = mojoAddPStmSlot( tmpPStm, atoi(tmpNode->txt) );

		tmpPStm->nbpart = atoi(tmpNode->txt);
		tmpPStm->id = iPStm;
		// PStm type
		tmpNode = ezxml_child(PStmNode, "type");
		tmpPStm->type = atoi(tmpNode->txt);
		// PStm posX
		tmpNode = ezxml_child(PStmNode, "posx");
		tmpPStm->x = atof(tmpNode->txt);
		// PStm posY
		tmpNode = ezxml_child(PStmNode, "posy");
		tmpPStm->y = atof(tmpNode->txt);
		// PStm posZ
		tmpNode = ezxml_child(PStmNode, "posz");
		tmpPStm->z = atof(tmpNode->txt);

		tmpNode = ezxml_child(PStmNode, "duration");
		tmpPStm->duration = atof(tmpNode->txt);

		tmpNode = ezxml_child(PStmNode, "alpha");
		tmpPStm->alpha = atof(tmpNode->txt);

		tmpNode = ezxml_child(PStmNode, "time");
		tmpPStm->time = atof(tmpNode->txt);

		// Individual Particle Size
		tmpNode = ezxml_child(PStmNode, "size");
		tmpPStm->size = atof(tmpNode->txt);

		// Slowdown factor
		tmpNode = ezxml_child(PStmNode, "slowdown");
		tmpPStm->Cslowdown = atof(tmpNode->txt);

		tmpNode = ezxml_child(PStmNode, "cx");
		tmpPStm->Cxspeed=atof(tmpNode->txt);				// Base X Speed
		tmpNode = ezxml_child(PStmNode, "cy");
		tmpPStm->Cyspeed=atof(tmpNode->txt);				// Base X Speed
		tmpNode = ezxml_child(PStmNode, "cz");
		tmpPStm->Czspeed=atof(tmpNode->txt);				// Base X Speed

		tmpNode = ezxml_child(PStmNode, "gx");
		tmpPStm->gx=atof(tmpNode->txt);						// Base X gravity
		tmpNode = ezxml_child(PStmNode, "gy");
		tmpPStm->gy=atof(tmpNode->txt);						// Base X gravity
		tmpNode = ezxml_child(PStmNode, "gz");
		tmpPStm->gz=atof(tmpNode->txt);						// Base X gravity

		tmpPStm->delay = 25;								// color cycle delay
		tmpPStm->accum = 0.0;								// color cycle delay
		tmpPStm->bindId = -1;								// color cycle delay

		tmpNode = ezxml_child(PStmNode, "texname");
		xmlValue = tmpNode->txt;
		tmpPStm->texid = mojoTexManGetTextureId(xmlValue);

		tmpNode = ezxml_child(PStmNode, "color");

		if( atoi(tmpNode->txt)== -1)
		{
			tmpPStm->col = 0;
			tmpPStm->rainbow = 1;
		}else{
			tmpPStm->col = atoi(tmpNode->txt)%12;
			tmpPStm->rainbow = 0;
		}

		//printf("xml node value : %s\n", xmlValue);
		printf(" Alloc PStm : #time.%f #texid %d #texname %s\n", tmpPStm->time, tmpPStm->texid, xmlValue);
		PStmNode = PStmNode->next;


		iPStm++;

		mojo_particleInit( tmpPStm, tmpPStm->partList );

		if(first == 0)
		{
			firstPStm = tmpPStm;
			first = 1;
		}


	}

	//ezxml_free( tmpNode );		// free nodes in reverse order, to prevent an exception
	ezxml_free( PStmNode );
	ezxml_free( xmlNode );
	//free( xmlValue );


	return firstPStm;
}




struct particleStm	*mojoLoadPStmPresetXmlCfg(char *filename)
{

    ezxml_t 		xmlNode, tmpNode, PStmNode;
    char 			*xmlValue;
   	char            StrBuffer[200];
	FILE 			*cfgFile;
	int				first=0, tmp=0;
	struct particleStm	*tmpPStm=NULL;
	struct particleStm	*firstPStm=NULL;

		// Xml Cfg Vars
	// -----------------------------------------------------
	if ( (cfgFile = fopen(filename, "rt")) == NULL ){

		mojolog ("! mojo \tlog : Xml Definition parsing Error !\n");
		return firstPStm;
	}

    xmlNode = ezxml_parse_fp( cfgFile );

	sprintf(StrBuffer,"- mojo \tlog : Xml Definition parsing Success ! [ loading particle presets in %s ]!\n",filename);
	mojolog (StrBuffer);

	fclose( cfgFile );

	PStmNode = NULL;
	PStmNode = ezxml_child(xmlNode, "particle");

	mojolog ("- mojo \tlog : Creating Preset PStm List!\n");


	while(PStmNode!=NULL)
	{

		tmpNode = NULL;

		// Nb Particles
		tmpNode = ezxml_child(PStmNode, "nbpart");

		tmpPStm = mojoAddPStmSlot( tmpPStm, atoi(tmpNode->txt) );

		tmpPStm->nbpart = atoi(tmpNode->txt);
		tmpPStm->id = tmp;
		// PStm type
		//tmpNode = ezxml_child(PStmNode, "type");
		//tmpPStm->type = atoi(tmpNode->txt);
		// PStm posX
		tmpNode = ezxml_child(PStmNode, "posx");
		tmpPStm->x = atof(tmpNode->txt);
		// PStm posY
		tmpNode = ezxml_child(PStmNode, "posy");
		tmpPStm->y = atof(tmpNode->txt);
		// PStm posZ
		tmpNode = ezxml_child(PStmNode, "posz");
		tmpPStm->z = atof(tmpNode->txt);

		tmpNode = ezxml_child(PStmNode, "duration");
		tmpPStm->duration = atof(tmpNode->txt);

		tmpNode = ezxml_child(PStmNode, "alpha");
		tmpPStm->alpha = atof(tmpNode->txt);

		tmpNode = ezxml_child(PStmNode, "time");
		tmpPStm->time = atof(tmpNode->txt);

		// Individual Particle Size
		tmpNode = ezxml_child(PStmNode, "size");
		tmpPStm->size = atof(tmpNode->txt);

		// Slowdown factor
		tmpNode = ezxml_child(PStmNode, "slowdown");
		tmpPStm->Cslowdown = atof(tmpNode->txt);

		tmpNode = ezxml_child(PStmNode, "cx");
		tmpPStm->Cxspeed=atof(tmpNode->txt);				// Base X Speed
		tmpNode = ezxml_child(PStmNode, "cy");
		tmpPStm->Cyspeed=atof(tmpNode->txt);				// Base X Speed
		tmpNode = ezxml_child(PStmNode, "cz");
		tmpPStm->Czspeed=atof(tmpNode->txt);				// Base X Speed

		tmpNode = ezxml_child(PStmNode, "gx");
		tmpPStm->gx=atof(tmpNode->txt);						// Base X gravity
		tmpNode = ezxml_child(PStmNode, "gy");
		tmpPStm->gy=atof(tmpNode->txt);						// Base X gravity
		tmpNode = ezxml_child(PStmNode, "gz");
		tmpPStm->gz=atof(tmpNode->txt);						// Base X gravity

		tmpPStm->delay = 25;								// color cycle delay
		tmpPStm->accum = 0.0;								// color cycle delay
		tmpPStm->bindId = -1;								// color cycle delay

		tmpNode = ezxml_child(PStmNode, "texname");
		xmlValue = tmpNode->txt;
		tmpPStm->texid = mojoTexManGetTextureId(xmlValue);

		tmpNode = ezxml_child(PStmNode, "color");

		if( atoi(tmpNode->txt)== -1)
		{
			tmpPStm->col = 0;
			tmpPStm->rainbow = 1;
		}else{
			tmpPStm->col = atoi(tmpNode->txt)%12;
			tmpPStm->rainbow = 0;
		}

		//printf("xml node value : %s\n", xmlValue);
		printf(" Alloc Preset PStm : #time.%f #presetid %d #texname %s\n", tmpPStm->time, tmpPStm->id, xmlValue);
		PStmNode = PStmNode->next;


		tmp++;

		if(first == 0)
		{
			firstPStm = tmpPStm;
			first = 1;
		}


	}

	//ezxml_free( tmpNode );		// free nodes in reverse order, to prevent an exception
	ezxml_free( PStmNode );
	ezxml_free( xmlNode );
	//free( xmlValue );


	return firstPStm;
}





struct particleStm	*mojoAddPStmSlot(struct particleStm *oldhead, int nbpart)
{

	struct particleStm 	*newnode;
	particles			*pnewnode;


	if((newnode = malloc(sizeof(struct particleStm))) == NULL )
	{
		printf("out of memory on Static PStm alloc\n");
		exit(1);
	}


	if((pnewnode = malloc(sizeof(particles)*nbpart)) == NULL )
	{
		printf("out of memory on Static PartList alloc\n");
		exit(1);
	}

	newnode->partSysNext = NULL;
	newnode->partList = pnewnode;


	if(oldhead!=NULL)
	{
		oldhead->partSysNext = newnode;
	}

	return newnode;

}






// be sure to test(NULL) the 'oldhead' before calling : if oldhead = NULL, it causes a segfault !
// should fix that quickly !
void	mojoRemPStmSlot(struct particleStm *oldhead, int id)
{

      struct particleStm* prevnode=  NULL;
      struct particleStm* tmp = NULL;

      while(oldhead != NULL)
      {
           if(oldhead->id == id) /* Si on trouve l'id */
           {
                tmp = oldhead; /* On memorise */

			printf("#dealloc PStm : %d, #nb part %d\n", oldhead->id, oldhead->nbpart);

                if (prevnode) /* Si ce n'est pas le premier */
                     prevnode->partSysNext = oldhead->partSysNext;
                else /* Si c'est le premier */
                     oldhead = oldhead->partSysNext;


                free(tmp->partList); /* On detruit les particules */
                free(tmp); /* On detruit le System de particules */
				tmp = NULL;
				return;
           }

           prevnode = oldhead; /* On memorise */
           oldhead = oldhead->partSysNext; /* Prochain System */
      }

}



int 		mojoFreePStmList( struct particleStm *head)
{

	struct particleStm 	*tmp;

	while(head != NULL )
	{
		printf("#dealloc PStm : %d, pos #%f #%f #%f, #nb part %d\n", head->id, head->x, head->y, head->z, head->nbpart);
		tmp = head->partSysNext;
		// free the associated particle list
		free(head->partList);
		free(head);
		head = tmp;
		iPStm = 0;
		dynPStmNum = 0;
		FirstDynPStm = NULL;
	}

	return 0;

}


int mojo_particleInit( struct particleStm *PStm, particles *particle )
{

	loop = 0;


	for (loop=0;loop<PStm->nbpart ;loop++)				// Initialize All The Textures
	{
		particle[loop].active=TRUE;								// Make All The Particles Active
		particle[loop].life=1.0f;								// Give All The Particles Full Life
		particle[loop].fade= (1.0-PStm->duration)+rand()/(float)RAND_MAX;//0.3;//(float)(rand()%100)/10.0f+0.003f;	// Random Fade Speed
		particle[loop].r=colors[loop*(12/PStm->nbpart)][0];	// Select Red Rainbow Color
		particle[loop].g=colors[loop*(12/PStm->nbpart)][1];	// Select Red Rainbow Color
		particle[loop].b=colors[loop*(12/PStm->nbpart)][2];	// Select Red Rainbow Color
		particle[loop].xi=(float)((rand()%50)-26.0f)*2.0f;		// Random Speed On X Axis
		particle[loop].yi=(float)((rand()%50)-25.0f)*2.0f;		// Random Speed On Y Axis
		particle[loop].zi=(float)((rand()%50)-25.0f)*2.0f;		// Random Speed On Z Axis
		particle[loop].xg=PStm->gx;									// Set Horizontal Pull To Zero
		particle[loop].yg=PStm->gy;								// Set Vertical Pull Downward
		particle[loop].zg=PStm->gz;									// Set Pull On Z Axis To Zero
		particle[loop].rot=(rand()/(float)RAND_MAX)*360.0f;								// local rotation

	}

	return TRUE;

}




struct particleStm	*mojoAddDynPStmSlot(struct particleStm *oldhead, struct particleStm *preset, int presetNum)
{

	struct particleStm 	*newnode, *tmpnode;
	particles			*pnewnode;

	tmpnode = preset;

	while( tmpnode!= NULL )
	{
		if(tmpnode->id == presetNum)
			break;

		tmpnode = tmpnode->partSysNext;
	}

	if((newnode = malloc(sizeof(struct particleStm))) == NULL )
	{
		printf("out of memory on Dyn PStm alloc\n");
		exit(1);
	}


	if((pnewnode = malloc(sizeof(particles)*tmpnode->nbpart)) == NULL )
	{
		printf("out of memory on Dyn PartList alloc\n");
		exit(1);
	}

	// copy the preset to newly allocated memory
	memcpy(newnode, tmpnode, sizeof(struct particleStm));
	iPStm++;
	dynPStmNum++;

	//printf(" testpreset : #nbpart %d #texid %d #id %d\n", preset.nbpart, newnode->texid, preset.id);
	// set the desired component
	newnode->partSysNext = NULL;
	newnode->partList = pnewnode;
	newnode->id = dynPStmNum;
	newnode->nbpart = tmpnode->nbpart;


	printf(" Adding Dyn PStm : #nbpart %d #addr %x #id %d\n", newnode->nbpart, newnode, newnode->id);

	mojo_particleInit( newnode, newnode->partList );

	if(oldhead!=NULL)
	{
		oldhead->partSysNext = newnode;
	}else{
		FirstDynPStm = newnode;
	}

	return newnode;

}




struct particleStm	*mojoGetDynPStm( void )
{

	return FirstDynPStm;

}




int mojo_particleDrawStatic(struct particleStm *PStm, float emitx, float emity, float emitz, float time)
{

	particles		*particle;
	float			pAlpha;
	int				pActiveNum;

	while( PStm != NULL ){

		//PStm->accum += time;
		glBindTexture( GL_TEXTURE_2D, PStm->texid );

		//Prepare_Billboarding();

		if(PStm->rainbow && (PStm->delay>25))			// Space Or Rainbow Mode
		{
			PStm->delay=0;								// Reset The Rainbow Color Cycling Delay
			PStm->col++;								// Change The Particle Color
			//if (PStm->col>11)	PStm->col=0;			// If Color Is Too High Reset It
			PStm->col=PStm->col%15;
		}

		PStm->delay++;

		particle = PStm->partList;

		pActiveNum = 0;

		for (loop=0;loop<PStm->nbpart;loop++)					// Loop Through All The Particles
		{

			if (particle[loop].active)							// If The Particle Is Active
			{

				float x=particle[loop].x;						// Grab Our Particle X Position
				float y=particle[loop].y;						// Grab Our Particle Y Position
				float z=particle[loop].z;						// Particle Z Pos

				particle[loop].x+=(particle[loop].xi/PStm->Cslowdown)*time;// Move On The X Axis By X Speed
				particle[loop].y+=(particle[loop].yi/PStm->Cslowdown)*time;// Move On The Y Axis By Y Speed
				particle[loop].z+=(particle[loop].zi/PStm->Cslowdown)*time;// Move On The Z Axis By Z Speed


				particle[loop].xi+=particle[loop].xg*time;			// Take Pull On X Axis Into Account
				particle[loop].yi+=particle[loop].yg*time;			// Take Pull On Y Axis Into Account
				particle[loop].zi+=particle[loop].zg*time;			// Take Pull On Z Axis Into Account
				particle[loop].life-=particle[loop].fade*time;		// Reduce Particles Life By 'Fade'

				// Draw The Particle Using Our RGB Values, Fade The Particle Based On It's Life
				if(PStm->alpha == 1.0 )
					pAlpha = particle[loop].life;
				else
					pAlpha = PStm->alpha;


				glColor4f(particle[loop].r,particle[loop].g,particle[loop].b,pAlpha);
				QuadBillboarded(PStm->size, x, y, z, particle[loop].rot);


				if (particle[loop].life<0.0f)					// If Particle Is Burned Out
				{

					if(PStm->time >= 0.0)
					{

						if((PStm->accum < PStm->time) ){
							particle[loop].life=1.0f;// Give It New Life
						}else{
							particle[loop].life=0.0f;// Give It death state
							particle[loop].active=0;
						}
					}else{
						particle[loop].life=1.0f;// Give It New Life
					}

						particle[loop].fade=(1.0-PStm->duration)+rand()/(float)RAND_MAX;//0.6;//(float)(rand()%100)/10.0f+0.003f;	// Random Fade Value
						particle[loop].x=PStm->x;//		emitx;				// Center On X Axis
						particle[loop].y=PStm->y;//		emity;				// Center On Y Axis
						particle[loop].z=PStm->z;//		emitz;				// Center On Z Axis
						particle[loop].xi=/*PStm->Cxspeed+*/(float)((rand()%60)-30.0f)*2.0;	// X Axis Speed And Direction
						particle[loop].yi=/*PStm->Cyspeed+*/(float)((rand()%60)-30.0f)*2.0;	// Y Axis Speed And Direction
						particle[loop].zi=/*PStm->Czspeed+*/(float)((rand()%60)-30.0f)*2.0;	// Z Axis Speed And Direction

						particle[loop].r=colors[PStm->col][0];			// Select Red From Color Table
						particle[loop].g=colors[PStm->col][1];			// Select Green From Color Table
						particle[loop].b=colors[PStm->col][2];			// Select Blue From Color Table
				}
			}


    	}

		PStm = PStm->partSysNext;

	}
		return 0;											// Everything Went OK
}




int mojo_particleDrawDyn(struct particleStm *PStm, float emitx, float emity, float emitz, float time)
{

	struct particleStm *tmpPStm;
	particles		*particle;
	float			pAlpha;
	int				pActiveNum;

	while( PStm != NULL ){

		PStm->accum += time;
		glBindTexture( GL_TEXTURE_2D, PStm->texid );

		//Prepare_Billboarding();

		if(PStm->rainbow && (PStm->delay>25))			// Space Or Rainbow Mode
		{
			PStm->delay=0;								// Reset The Rainbow Color Cycling Delay
			PStm->col++;								// Change The Particle Color
			//if (PStm->col>11)	PStm->col=0;			// If Color Is Too High Reset It
			PStm->col=PStm->col%15;
		}

		PStm->delay++;

		particle = PStm->partList;

		pActiveNum = 0;

		for (loop=0;loop<PStm->nbpart;loop++)					// Loop Through All The Particles
		{

			if (particle[loop].active)							// If The Particle Is Active
			{

				pActiveNum++;

				float x=particle[loop].x;						// Grab Our Particle X Position
				float y=particle[loop].y;						// Grab Our Particle Y Position
				float z=particle[loop].z;						// Particle Z Pos

				particle[loop].x+=(particle[loop].xi/PStm->Cslowdown)*time;// Move On The X Axis By X Speed
				particle[loop].y+=(particle[loop].yi/PStm->Cslowdown)*time;// Move On The Y Axis By Y Speed
				particle[loop].z+=(particle[loop].zi/PStm->Cslowdown)*time;// Move On The Z Axis By Z Speed


				particle[loop].xi+=particle[loop].xg*time;			// Take Pull On X Axis Into Account
				particle[loop].yi+=particle[loop].yg*time;			// Take Pull On Y Axis Into Account
				particle[loop].zi+=particle[loop].zg*time;			// Take Pull On Z Axis Into Account
				particle[loop].life-=particle[loop].fade*time;		// Reduce Particles Life By 'Fade'

				// Draw The Particle Using Our RGB Values, Fade The Particle Based On It's Life
				if(PStm->alpha == 1.0 )
					pAlpha = particle[loop].life;
				else
					pAlpha = PStm->alpha;



				glColor4f(particle[loop].r,particle[loop].g,particle[loop].b,pAlpha);
				QuadBillboarded(PStm->size, x, y, z, particle[loop].rot);


				if (particle[loop].life<0.0f)					// If Particle Is Burned Out
				{

					if(PStm->time >= 0.0)
					{

						if((PStm->accum < PStm->time) ){
							particle[loop].life=1.0f;// Give It New Life
						}else{
							particle[loop].life=0.0f;// Give It death state
							particle[loop].active=0;
						}
					}else{
						particle[loop].life=1.0f;// Give It New Life
					}

						particle[loop].fade=(1.0-PStm->duration)+rand()/(float)RAND_MAX;//0.6;//(float)(rand()%100)/10.0f+0.003f;	// Random Fade Value
						particle[loop].x=PStm->x;//		emitx;				// Center On X Axis
						particle[loop].y=PStm->y;//		emity;				// Center On Y Axis
						particle[loop].z=PStm->z;//		emitz;				// Center On Z Axis
						particle[loop].xi=/*PStm->Cxspeed+*/(float)((rand()%60)-30.0f)*2.0;	// X Axis Speed And Direction
						particle[loop].yi=/*PStm->Cyspeed+*/(float)((rand()%60)-30.0f)*2.0;	// Y Axis Speed And Direction
						particle[loop].zi=/*PStm->Czspeed+*/(float)((rand()%60)-30.0f)*2.0;	// Z Axis Speed And Direction

						particle[loop].r=colors[PStm->col][0];			// Select Red From Color Table
						particle[loop].g=colors[PStm->col][1];			// Select Green From Color Table
						particle[loop].b=colors[PStm->col][2];			// Select Blue From Color Table
				}

			}

    	}
/*
		if(pActiveNum == 0 )
		{
			tmpPStm = PStm;
			PStm = PStm->partSysNext;
		    mojoRemPStmSlot( FirstDynPStm, tmpPStm->id );
		}else{
*/			PStm = PStm->partSysNext;
//		}

	}
		return 0;											// Everything Went OK
}





int mojo_particleDrawSAFE(struct particleStm *PStm, float emitx, float emity, float emitz, float time)
{

	particles			*particle;


	while( PStm != NULL ){

		glBindTexture( GL_TEXTURE_2D, PStm->texid );

		Prepare_Billboarding();

		if(PStm->rainbow && (PStm->delay>25))			// Space Or Rainbow Mode
		{
			PStm->delay=0;						// Reset The Rainbow Color Cycling Delay
			PStm->col++;							// Change The Particle Color
			if (PStm->col>11)	PStm->col=0;				// If Color Is To High Reset It
		}

		PStm->delay++;


		particle = PStm->partList;


		for (loop=0;loop<PStm->nbpart;loop++)					// Loop Through All The Particles
		{
			if (particle[loop].active)							// If The Particle Is Active
			{
				float x=particle[loop].x;						// Grab Our Particle X Position
				float y=particle[loop].y;						// Grab Our Particle Y Position
				float z=particle[loop].z /*+zoom*/;					// Particle Z Pos + Zoom

				particle[loop].x+=(particle[loop].xi/PStm->Cslowdown)*time;// Move On The X Axis By X Speed
				particle[loop].y+=(particle[loop].yi/PStm->Cslowdown)*time;// Move On The Y Axis By Y Speed
				particle[loop].z+=(particle[loop].zi/PStm->Cslowdown)*time;// Move On The Z Axis By Z Speed


				particle[loop].xi+=particle[loop].xg*time;			// Take Pull On X Axis Into Account
				particle[loop].yi+=particle[loop].yg*time;			// Take Pull On Y Axis Into Account
				particle[loop].zi+=particle[loop].zg*time;			// Take Pull On Z Axis Into Account
				particle[loop].life-=particle[loop].fade*time;///(time*500.0);		// Reduce Particles Life By 'Fade'

/*			glBegin(GL_TRIANGLE_STRIP);						// Build Quad From A Triangle Strip
			    glTexCoord2d(1,1); glVertex3f(x+0.5f,y+0.5f,z); // Top Right
				glTexCoord2d(0,1); glVertex3f(x-0.5f,y+0.5f,z); // Top Left
				glTexCoord2d(1,0); glVertex3f(x+0.5f,y-0.5f,z); // Bottom Right
				glTexCoord2d(0,0); glVertex3f(x-0.5f,y-0.5f,z); // Bottom Left
			glEnd();										// Done Building Triangle Strip
*/
				// Draw The Particle Using Our RGB Values, Fade The Particle Based On It's Life
				glColor4f(particle[loop].r,particle[loop].g,particle[loop].b,particle[loop].life);
				QuadBillboarded(2.0f, x, y, z, 0.0);


				if (particle[loop].life<0.0f)					// If Particle Is Burned Out
				{
					particle[loop].life=1.0f;					// Give It New Life
					particle[loop].fade=0.2+rand()/(float)RAND_MAX;//0.3;//(float)(rand()%100)/10.0f+0.003f;	// Random Fade Value
					particle[loop].x=emitx;						// Center On X Axis
					particle[loop].y=emity;						// Center On Y Axis
					particle[loop].z=emitz;						// Center On Z Axis
					particle[loop].xi=PStm->Cxspeed+(float)((rand()%60)-30.0f);	// X Axis Speed And Direction
					particle[loop].yi=PStm->Cyspeed+(float)((rand()%60)-30.0f);	// Y Axis Speed And Direction
					particle[loop].zi=PStm->Czspeed+(float)((rand()%60)-30.0f);	// Z Axis Speed And Direction

			//particle[loop].xi=(float)((rand()%50)-26.0f)*2.0f;		// Random Speed On X Axis
			//particle[loop].yi=(float)((rand()%50)-25.0f)*2.0f;		// Random Speed On Y Axis
			//particle[loop].zi=(float)((rand()%50)-25.0f)*2.0f;		// Random Speed On Z Axis
					particle[loop].r=colors[PStm->col][0];			// Select Red From Color Table
					particle[loop].g=colors[PStm->col][1];			// Select Green From Color Table
					particle[loop].b=colors[PStm->col][2];			// Select Blue From Color Table
				}

			/*
			// If Number Pad 8 And Y Gravity Is Less Than 1.5 Increase Pull Upwards
			if (keys[VK_NUMPAD8] && (particle[loop].yg<1.5f)) particle[loop].yg+=0.01f;

			// If Number Pad 2 And Y Gravity Is Greater Than -1.5 Increase Pull Downwards
			if (keys[VK_NUMPAD2] && (particle[loop].yg>-1.5f)) particle[loop].yg-=0.01f;

			// If Number Pad 6 And X Gravity Is Less Than 1.5 Increase Pull Right
			if (keys[VK_NUMPAD6] && (particle[loop].xg<1.5f)) particle[loop].xg+=0.01f;

			// If Number Pad 4 And X Gravity Is Greater Than -1.5 Increase Pull Left
			if (keys[VK_NUMPAD4] && (particle[loop].xg>-1.5f)) particle[loop].xg-=0.01f;

			if (keys[VK_TAB])										// Tab Key Causes A Burst
			{
				particle[loop].x=0.0f;								// Center On X Axis
				particle[loop].y=0.0f;								// Center On Y Axis
				particle[loop].z=0.0f;								// Center On Z Axis
				particle[loop].xi=(float)((rand()%50)-26.0f)*10.0f;	// Random Speed On X Axis
				particle[loop].yi=(float)((rand()%50)-25.0f)*10.0f;	// Random Speed On Y Axis
				particle[loop].zi=(float)((rand()%50)-25.0f)*10.0f;	// Random Speed On Z Axis
			}
			*/
			}
    	}

		PStm = PStm->partSysNext;

	}
		return 0;											// Everything Went OK
}






int		mojoAddBillboardSlot( float isAlign, float life, int texId, float x, float y, float z, float size=15.0 )
{
	int i=0;

		//printf(" addedbullet\n");

		for(i=0;i<BILLBOARDMAXNUM;i++)
		{
			if(billboardSlot[i].bilState == 0)
			{

				printf(" adding billboard : %d\n", i);

				billboardSlot[i].isAligned = isAlign;
				billboardSlot[i].texId = texId;
				billboardSlot[i].isFading = 1;
				billboardSlot[i].bilState = 1;
				billboardSlot[i].bilLifeTotal = life;
				billboardSlot[i].bilLifeCurrent = 0.0;
				billboardSlot[i].bilSizeStart = 0.0;
				billboardSlot[i].bilSizeFinal = size;
				billboardSlot[i].bilPosX = x;
				billboardSlot[i].bilPosY = y;
				billboardSlot[i].bilPosZ = z;

				return i;
			}
		}

	return 0;
}


void		mojoUpdateBillboardList( double elapsed )
{
	int 	i=0;
	MATRIX 	MtxTmp;
	float 	size,	delta,	color;

	for(i=0;i<BILLBOARDMAXNUM;i++)
	{

		if(billboardSlot[i].bilLifeCurrent < billboardSlot[i].bilLifeTotal )
		{

			delta 	= billboardSlot[i].bilLifeCurrent / billboardSlot[i].bilLifeTotal;
			size	= billboardSlot[i].bilSizeStart + ( billboardSlot[i].bilSizeFinal*delta );
			billboardSlot[i].bilSizeCurrent = size;
			color	= 1.0 - delta;/*easeOut( delta, 0.0, 1.0,2.0)*/;
			//size = easeOut( delta, billboardSlot[i].bilSizeStart, 30.0, 2.0);

			MatrixLoadIdentity( &MtxTmp );

			MtxTmp.m[12] = billboardSlot[i].bilPosX;
			MtxTmp.m[13] = billboardSlot[i].bilPosY;
			MtxTmp.m[14] = billboardSlot[i].bilPosZ;

			//MatrixRotateY( &MtxTmp, bulletSlot[i].bulAngle );

			glPushMatrix();

			mojoGlBodyDraw( &MtxTmp );

			if(billboardSlot[i].isAligned){
				glBindTexture( GL_TEXTURE_2D, billboardSlot[i].texId );

				if(billboardSlot[i].isFading)
					glColor3f( color, color, color);

				glRotatef(i*5.0, 1.0,1.0,0.0);
				QuadBillboarded(size, 0.0, 0.0, 0.0, 0.0);
			}else{
				glBindTexture( GL_TEXTURE_2D, billboardSlot[i].texId );

				glColor3f( color, color, color);
				//glRotatef(i*20.0, 0.0,1.0,0.0);

				glBegin( GL_QUADS );
					glTexCoord2f(0.0f, 0.0f); glVertex3f(	-size,	0,	 size);
					glTexCoord2f(0.0f, 1.0f); glVertex3f(	 size, 	0,   size);
					glTexCoord2f(1.0f, 1.0f); glVertex3f( 	 size, 	0,	-size);
					glTexCoord2f(0.0f, 1.0f); glVertex3f(	-size, 	0, 	-size);
				glEnd();
				//mojoBoxDraw( 1.0, 1.0, 1.0, 0.0, 5.0, 0.0);
			}

			glPopMatrix();

			glColor3f( 1.0, 1.0, 1.0);
			billboardSlot[i].bilLifeCurrent += elapsed;


			//printf("dist #%f \n", locDist );

			if( billboardSlot[i].bilLifeCurrent > billboardSlot[i].bilLifeTotal )
			{
				billboardSlot[i].bilState = 0;
				printf(" killing billboard : %d\n", i);
			}//endif




		}//endif

	}//endfor

}


double easeOut (float t /*time*/, float b /*start*/,float c /*end*/,float d /*steps*/)
{
		return (double)-c *(t/=d)*(t-2) + b;
}


double easeIn (float t /*time*/, float b /*start*/,float c /*end*/,float d /*steps*/)
{

		return (double)c*(t/=d)*t + b;
}

double easeInOut (float t /*time*/, float b /*start*/,float c /*end*/,float d /*steps*/)
{

		if ((t/=d/2) < 1) return c/2*t*t + b;
		return (double)-c/2 * ((--t)*(t-2) - 1) + b;
}

void		mojoFreeBillboardList( void )
{
	int i=0;

	for(i=0;i<BILLBOARDMAXNUM;i++)
	{

				billboardSlot[i].isAligned = 0;
				billboardSlot[i].texId = 0;
				billboardSlot[i].isFading = 0;
				billboardSlot[i].bilState = 0;
				billboardSlot[i].bilLifeTotal = 1.0;
				billboardSlot[i].bilLifeCurrent = 0.0;
				billboardSlot[i].bilSizeStart = 0.0;
				billboardSlot[i].bilSizeFinal = 0.0;
	}
}





void QuadBillboarded(float Size, float x, float y, float z, float angle)
{
	float ax,ay,az;
	float bx,by,bz;
	float cx,cy,cz;
	float dx,dy,dz;

	// make new vertexes
	ax = x + ((-bb_rg_x - bb_up_x) * Size);
	ay = y + ((-bb_rg_y - bb_up_y) * Size);
	az = z + ((-bb_rg_z - bb_up_z) * Size);

	bx = x + ((bb_rg_x - bb_up_x) * Size);
	by = y + ((bb_rg_y - bb_up_y) * Size);
	bz = z + ((bb_rg_z - bb_up_z) * Size);

	cx = x + ((bb_rg_x + bb_up_x) * Size);
	cy = y + ((bb_rg_y + bb_up_y) * Size);
	cz = z + ((bb_rg_z + bb_up_z) * Size);

	dx = x + ((-bb_rg_x + bb_up_x) * Size);
	dy = y + ((-bb_rg_y + bb_up_y) * Size);
	dz = z + ((-bb_rg_z + bb_up_z) * Size);

	//sin(RADIAN(angle))*
	//cos(RADIAN(angle))*

	// draw quad

	glBegin( GL_QUADS );
		glTexCoord2f( 0.0f, 0.0f ); glVertex3f( ax, ay, az );
		glTexCoord2f( 0.0f, 1.0f ); glVertex3f( bx, by, bz );
		glTexCoord2f( 1.0f, 1.0f ); glVertex3f( cx, cy, cz );
		glTexCoord2f( 1.0f, 0.0f ); glVertex3f( dx, dy, dz );
	glEnd();

}




void Prepare_Billboarding( void )
{
    float mat[16];

	// get current matrix
	glGetFloatv(GL_MODELVIEW_MATRIX, mat);

	// take right & up vectors
	bb_rg_x = mat[0];
	bb_rg_y = mat[4];
	bb_rg_z = mat[8];
	bb_up_x = mat[1];
	bb_up_y = mat[5];
	bb_up_z = mat[9];
}





