/*
 * 	---------------- MoJo engine --------------
 *   		---- Basic Game Engine ----
 *
 * Author: Yann PARMENTIER <kohai@kohaistyle.com>
 *
 * ALL RIGHTS RESERVED
 *
 *	Uses
 * 			DMFloader  		( www.delgine.com )
 * 			GLFW 				( glfw.sourceforge.net )
 *			Fmod Sound API 		( www.fmod.org )
 *
 * To compile this project you must include the following libraries:
 * wlib.lib uuid.lib fmodlcc.lib glfw.lib glu32.lib ole32.lib olepro32.lib opengl32.lib
 * You need also the header file glfw.h in your GL/ compiler directory.
 *
 * File header: mojoParticles2.h
 *
 */



#ifndef _MOJOPARTICLES2_H_
#define _MOJOPARTICLES2_H_

#pragma once

#ifdef MEMWATCH
#include 	"memwatch.h"
#endif

// _______________________________________________________[ Struct Particle ]
typedef struct						// Create A Structure For Particle
{
	BOOL		active;					// Active (Yes/No)
	float	life;					// Particle Life
	float	fade;					// Fade Speed
	float	r;						// Red Value
	float	g;						// Green Value
	float	b;						// Blue Value
	float	x;						// X Position
	float	y;						// Y Position
	float	z;						// Z Position
	float	xi;						// X Direction
	float	yi;						// Y Direction
	float	zi;						// Z Direction
	float	xg;						// X Gravity
	float	yg;						// Y Gravity
	float	zg;						// Z Gravity
	float	rot;
}particles;							// Particles Structure


// _______________________________________________________[ Struct Particle SysTeM ]
struct particleStm					// Create A Structure For Particle
{
	int					nbpart;					// Active (Yes/No)
	int					id;
	float				time;					// Particle Life
	float				accum;					// Particle Life
	float				duration;					// Fade Speed
	int 				behaviour;
	int					type;
	float				size;					// Particle Size
	float				alpha;					// Particle alpha (if constant, otherwise based on life)
	int					texid;
	int 				bindId;

	float	x;						// X Position
	float	y;						// Y Position
	float	z;						// Z Position

	float	gx;						// X Position
	float	gy;						// Y Position
	float	gz;						// Z Position


	float	Cslowdown;				// Slow Down Particles
	float	Cxspeed;						// Base X Speed (To Allow Keyboard Direction Of Tail)
	float	Cyspeed;						// Base Y Speed (To Allow Keyboard Direction Of Tail)
	float	Czspeed;						// Base X Speed (To Allow Keyboard Direction Of Tail)

	GLuint	col;						// Current Color Selection
	GLuint	delay;						// Rainbow Effect Delay
	BOOL	rainbow;				// Rainbow Mode?

	particles			*partList;
	struct particleStm	*partSysNext;
};							// Particles System Structure




// _______________________________________________________[ Struct BillboardSlot ]
typedef struct{
	int				isAligned;
	int				isFading;
	int				texId;
	int				bindId;
	float			bilPosX, bilPosY, bilPosZ;
	float			bilRotX, bilRotY, bilRotZ;
	float			bilOffX, bilOffY, bilOffZ;
	int				bilState;
	float			bilLifeTotal, bilLifeCurrent;
	float			bilSizeCurrent;
	float			bilSizeStart;
	float			bilSizeFinal;
}BILLBOARD;


#define 	BILLBOARDMAXNUM 	50
BILLBOARD	billboardSlot[BILLBOARDMAXNUM];
BILLBOARD	billboardPreset[10];





struct particleStm	*mojoLoadPStmXmlCfg(char *filename);

struct particleStm	*mojoLoadPStmPresetXmlCfg(char *filename);

struct particleStm	*mojoAddPStmSlot(struct particleStm *oldhead, int nbpart);

struct particleStm	*mojoAddDynPStmSlot(struct particleStm *oldhead, struct particleStm *preset, int presetNum);

struct particleStm	*mojoGetDynPStm( void );

void				mojoRemPStmSlot(struct particleStm *oldhead, int id);

int 				mojoFreePStmList( struct particleStm *head);

int 				mojo_particleInit( struct particleStm *PStm, particles *particle );

int 				mojo_particleDrawStatic(struct particleStm *PStm,float emitx, float emity, float emitz, float time);

int 				mojo_particleDrawDyn(struct particleStm *PStm, float emitx,float emity, float emitz, float time);

int 				mojo_particleDrawSAFE(struct particleStm *PStm,
					float emitx, float emity, float emitz, float time);

int				mojoAddBillboardSlot( float isAlign, float life, int texId,
					float x, float y, float z, float size=15.0  );

void				mojoUpdateBillboardList( double elapsed );

void				mojoFreeBillboardList( void );

double easeOut (float t /*time*/, float b /*start*/,float c /*end*/,float d /*steps*/);
double easeIn (float t /*time*/, float b /*start*/,float c /*end*/,float d /*steps*/);
double easeInOut (float t /*time*/, float b /*start*/,float c /*end*/,float d /*steps*/);

// Rendu des particules
void 				QuadBillboarded(float Size, float x, float y, float z, float angle);

// Initialisation du "systeme" de particules
void 				Prepare_Billboarding( void );



#endif // _MOJOPARTICLES2__H_
