/*
 * 	---------------- MoJo engine --------------
 *   		---- Basic Game Engine ----
 *
 * Author: Yann PARMENTIER <kohai@kohaistyle.com>
 *
 * ALL RIGHTS RESERVED
 *
 *	Uses
 * 			DMFloader  		( www.delgine.com )
 * 			GLFW 				( glfw.sourceforge.net )
 *			Fmod Sound API 		( www.fmod.org )
 *
 * To compile this project you must include the following libraries:
 * wlib.lib uuid.lib fmodlcc.lib glfw.lib glu32.lib ole32.lib olepro32.lib opengl32.lib
 * You need also the header file glfw.h in your GL/ compiler directory.
 *
 * File header: mojoSupport.c
 *
 */



#include 	<stdio.h>
#include 	<stdlib.h>
#include 	<math.h>
#include 	"project.h"
#include 	"milkshapemodel.h"
#include 	<GL/glfw.h>
#include 	<GL/glext.h>
#include 	"mojomain.h"
#include 	"newton.h"
#include 	"mojoparticles2.h"
#include 	"mojolog.h"
#include 	"ezxml.h"



extern MODEL 							Bonus;
extern PFNGLMULTITEXCOORD2FARBPROC     	glMultiTexCoord2fARB;
extern PFNGLACTIVETEXTUREARBPROC       	glActiveTextureARB;
//extern NewtonCollision* 				newtonBulletCol;
//extern NewtonCollision* 				newtonEnmBulletCol;


// _______________________________________________________[ Struct BulletSlot ]
typedef struct{
	int				bulType;
	int				id;
	float			bulPosX, bulPosY, bulPosZ;
	float			bulAngle;
	int				bulState;
	float			bulLife;
	float			bulRadiusCol;
	float			bulForwardSpeed;
	float			bulSpeedFactor;
	float			distToStatic;
}BULLET;



#define 	BULLETMAXNUM 	15
float		bulletLife = 0.5;
float		bulletDelay = 0.7;
float		bulletMaxDelay = 0.2;
BULLET		bulletSlot[BULLETMAXNUM];





// _______________________________________________________[ Struct BonusSlot ]
typedef struct{
	int				bonType;
	int				id;
	int				bonTexId;
	int				spawnTexId;
	float			bonPosX, bonPosY, bonPosZ;
	float			bonAngle;
	int				bonState;
	int				bonDashPower;
	int				bonLifePower;
	int				bonWeaponPower;
	int				bonScorePower;

	float			bonLife;
	float			bonRadiusCol;
}BONUS;



#define 	BONUSMAXNUM 	10
BONUS		bonusSlotList[BONUSMAXNUM];
BONUS		bonusSlotPreset[BONUSMAXNUM];




// _______________________________________________________[ Global Num ]
unsigned int		iEnm=0;		// total number of enemies
unsigned int		iBind=0;	// total number of enemies



float				fadeLevel=0.0;
int					fadeState=0;






// _______________________________________________________[ Function Prototypes ]
int			mojoEnmFSM(struct ENEMY enm)
{




	return 0;
}





float		mojoGetAngle(float dx, float dy)
{

	int quadflag = 0;
	float locAngle = 0;

	if( dx<0 ){
		dx = -dx ;
		//dx = dx * -1;
		quadflag++;
	}

	if( dy<0 ){
		dy = -dy ;
		//dy = dy * -1;
	   	quadflag += 2;
	}

	locAngle = atan(dy/dx);

	// re-adjust angle to face in the orignal quadrant

	locAngle = DEGREE(locAngle);

	if ( quadflag==1 ) locAngle = 180.0-locAngle;
	if ( quadflag==2 ) locAngle = 360.0-locAngle;
	if ( quadflag==3 ) locAngle = 180.0+locAngle;

	return locAngle;
	//return WRAP(locAngle);
}




float	mojoGetDistance2D(float sx, float sy, float dx, float dy)
{

	return sqrt((sx - dx)*(sx - dx) + (sy - dy)*(sy - dy));

}



float	mojoGetDistance3D(float sx, float sy, float sz, float dx, float dy, float dz)
{

	return sqrt((sx - dx)*(sx - dx) + (sy - dy)*(sy - dy) + (sz - dz)*(sz - dz));

}



int		mojoGetBoxCollide(	float size1, float x1, float y1, float z1,
							float size2, float x2, float y2, float z2)
{

/*	float x1min = x1 - size1;
	float y1min = y1 - size1;
	float z1min = z1 - size1;

	float x1max = x1 + size1;
	float y1max = y1 + size1;
	float z1max = z1 + size1;
*/
	float x2min = x2 - size2;
	float y2min = y2 - size2;
	float z2min = z2 - size2;

	float x2max = x2 + size2;
	float y2max = y2 + size2;
	float z2max = z2 + size2;


	if( (x1 > x2min ) && ( y1 > y2min ) && (z1 > z2min ) )
		if( (x1 < x2max ) && ( y1 < y2max ) && (z1 < z2max ) )
			return 1;

	return 0;
}




void 			mojoGlBodyDraw(MATRIX *p)
{

	float m[16];

	m[0]=p->m[0];	m[4]=p->m[4];	m[8]=p->m[8];	m[12]=p->m[12];
	m[1]=p->m[1];	m[5]=p->m[5];	m[9]=p->m[9];	m[13]=p->m[13];
	m[2]=p->m[2];	m[6]=p->m[6];	m[10]=p->m[10];	m[14]=p->m[14];
	m[3]=p->m[3];	m[7]=p->m[7];	m[11]=p->m[11];	m[15]=p->m[15];

	//glTranslatef(m[12],m[13],m[14]);

	glMultMatrixf(m);

}




void 	mojoSetResourceDir(void)
{

	//sKr_params[1][strlen(sKr_params[1])-1] = '\0';
	//strcpy(mojoResDir, sKr_params[1]);

#if DEBUG
	if(debug){
	printf("\n#call mojoSetResourceDir\n\r" );
	}
#endif
}



int 	mojoLoadTexResource( char *filename )
{

	int free=0;

	free = mojoTexManGetFreeId();

	if(free != -1 )
	{
		mojoTexManLoad( filename, free, 1, GL_CLAMP_TO_EDGE );
	}

#if DEBUG
	if(debug){
	printf("\n#call mojoLoadTexResource\n\r" );
	}
#endif

	return free;
}



struct BIND	*mojoAddBindSlot(struct BIND *oldhead)
{

	struct BIND *newnode;

	if((newnode = malloc(sizeof(struct BIND))) == NULL )
	{
		printf("out of memory\n");
		exit(1);
	}

	newnode->next = NULL;
	newnode->id = iBind;

	iBind++;

	if(oldhead!=NULL)
	{
		oldhead->next = newnode;
	}

	return newnode;

}



int 		mojoUpdateBindSlot( struct BIND *head, int bindId, MATRIX mtx)
{

	//struct BIND *tmp;

	while(head != NULL )
	{
		if(head->id == bindId ) break;

		head = head->next;
	}

	memcpy( &(head)->Mtx, &mtx, sizeof(MATRIX));

	return 0;

}



int 		mojoFreeBindList( struct BIND *head)
{

	struct BIND *tmp;

	while(head != NULL )
	{
		//printf("#dealloc bind : %d, addr #%x \n",head->id, head);
		tmp = head->next;
		free(head);
		head = tmp;
		iBind = 0;
	}

	return 0;

}



void	mojoRemEnmSlot(struct ENEMY *oldhead, int id)
{

      struct ENEMY* prevnode=  NULL;
      struct ENEMY* tmp = NULL;

      while(oldhead != NULL)
      {
           if(oldhead->id == id) /* Si on trouve l'id */
           {
                tmp = oldhead; /* On memorise */

                if (prevnode) /* Si ce n'est pas le premier */
                     prevnode->next = oldhead->next;
                else /* Si c'est le premier */
                     oldhead = oldhead->next;

                free(tmp); /* On detruit le System de particules */
				return;
           }

           prevnode = oldhead; /* On memorise */
           oldhead = oldhead->next; /* Prochain System */
      }

}




struct ENEMY	*mojoAddEnmSlot(struct ENEMY *oldhead)
{

	struct ENEMY *newnode;

	if((newnode = malloc(sizeof(struct ENEMY))) == NULL )
	{
		printf("out of memory\n");
		exit(1);
	}


	newnode->next = NULL;


	if(oldhead!=NULL)
	{
		oldhead->next = newnode;
	}

	return newnode;

}




int 		mojoFreeEnmList( struct ENEMY *head)
{

	struct ENEMY *tmp;

	while(head != NULL )
	{
		printf("#dealloc enm : %d, pos #%f #%f #%f\n", head->id, head->enmMtx.m[12], head->enmMtx.m[13], head->enmMtx.m[14]);
		tmp = head->next;
		free(head);
		head = tmp;
		iEnm = 0;
	}

	return 0;

}




void		mojoBulletSetVar( float delay, float life)
{


	bulletMaxDelay = delay;
	bulletLife = life;

}





void		mojoAddBulletSlot( float elapsed, float angle, float dist, float x, float y, float z )
{
	int i=0;


	bulletDelay += elapsed;

	if(bulletDelay > bulletMaxDelay )
	{
		//printf(" addedbullet\n");

		for(i=0;i<BULLETMAXNUM;i++)
		{
			if(bulletSlot[i].bulState == 0)
			{

				printf(" adding bullet : %d\n", i);

				bulletSlot[i].bulType = 0;
				//bulletSlot[i].id = 0;
				bulletSlot[i].bulAngle = angle;
				bulletSlot[i].bulState = 1;
				bulletSlot[i].distToStatic = dist;
				bulletSlot[i].bulLife = bulletLife;
				bulletSlot[i].bulForwardSpeed = 150.0;
				bulletSlot[i].bulSpeedFactor = 0.0;
				bulletSlot[i].bulPosX = x;
				bulletSlot[i].bulPosY = y;
				bulletSlot[i].bulPosZ = z;
				bulletSlot[i].bulRadiusCol = 3.0;
				bulletDelay = 0.0;

				i = FSOUND_PlaySound(FSOUND_FREE, soundSlot[ playerSndFIRE ].sndPtr);

				return;
			}
		}

	}


}


void		mojoUpdateBulletList( double elapsed )
{
	int 	i=0, texNum=0;
	MATRIX 	MtxTmp;
	float	locDist;

	texNum = mojoTexManGetTextureId("impact.tga");

	glEnable(GL_BLEND);
	//glBindTexture( GL_TEXTURE_2D, texNum );

		//glEnable(GL_ALPHA_TEST);

	for(i=0;i<BULLETMAXNUM;i++)
	{

		if(bulletSlot[i].bulLife > 0.0 )
		{

			locDist = bulletSlot[i].bulForwardSpeed*elapsed;

			bulletSlot[i].bulPosX += sin(RADIAN(bulletSlot[i].bulAngle))*locDist;
			bulletSlot[i].bulPosZ += cos(RADIAN(bulletSlot[i].bulAngle))*locDist;

			MatrixLoadIdentity( &MtxTmp );

			MtxTmp.m[12] = bulletSlot[i].bulPosX;
			MtxTmp.m[13] = bulletSlot[i].bulPosY;
			MtxTmp.m[14] = bulletSlot[i].bulPosZ;

			MatrixRotateY( &MtxTmp, bulletSlot[i].bulAngle );

			glPushMatrix();

			mojoGlBodyDraw( &MtxTmp );
			glColor3f( 0.0, 0.0, 1.0);

			mojoBoxDraw( 0.5, 0.5, 0.5, 0.0, 6.0, 0.0);

			glPopMatrix();

			//glColor4f( 1.0, 1.0, 1.0, 0.5);
			//QuadBillboarded(2.0, MtxTmp.m[12], MtxTmp.m[13], MtxTmp.m[14], 0.0);

			bulletSlot[i].bulLife -= elapsed;
			bulletSlot[i].distToStatic -= locDist;


			//printf("dist #%f \n", locDist );

			if( ( bulletSlot[i].bulLife < 0.0 ) )
			{

				bulletSlot[i].bulLife = 0.0;
				bulletSlot[i].bulState = 0;
				//printf(" killing bullet : %d\n", i);

			}else if( bulletSlot[i].distToStatic+5.0 < 0.0 ){

				mojoAddBillboardSlot( 1, 0.5, texNum, bulletSlot[i].bulPosX, bulletSlot[i].bulPosY+6.0, bulletSlot[i].bulPosZ );
				bulletSlot[i].bulLife = 0.0;
				bulletSlot[i].bulState = 0;
				i = FSOUND_PlaySound(FSOUND_FREE, soundSlot[ bulletSndMISS ].sndPtr);
				//printf(" killing bullet : %d\n", i);

			}//endif




		}//endif

	}//endfor

		//glDisable(GL_ALPHA_TEST);

	glDisable(GL_BLEND);

}



int			mojoCheckBulletCollision( float radius, float x, float y, float z)
{

	int 	i=0;
	double 	dist=0;

	for(i=0;i<BULLETMAXNUM;i++)
	{
		if(bulletSlot[i].bulState>0)
		{

			if(mojoGetBoxCollide(	radius, x, y, z,
									bulletSlot[i].bulRadiusCol, bulletSlot[i].bulPosX, bulletSlot[i].bulPosY, bulletSlot[i].bulPosZ))
			{
				dist = mojoGetDistance3D( 	bulletSlot[i].bulPosX, bulletSlot[i].bulPosY, bulletSlot[i].bulPosZ,
											x, y, z );
				if(dist<(bulletSlot[i].bulRadiusCol + radius))
				{
					bulletSlot[i].bulState=0;
					return 1;
				}
			}

		}

	}

	return 0;
}




void		mojoFreeBulletList( void )
{
	int i=0;

	for(i=0;i<BULLETMAXNUM;i++)
	{

		bulletSlot[i].bulType = 0;
		bulletSlot[i].id = 0;
		bulletSlot[i].distToStatic = 100.0;
		bulletSlot[i].bulAngle = 0.0;
		bulletSlot[i].bulState = 0;
		bulletSlot[i].bulLife = 0.0;
		bulletSlot[i].bulForwardSpeed = 0.0;
		bulletSlot[i].bulSpeedFactor = 0.0;
	}
}




void		mojoAddBonusSlot( int bonusPresetId, float x, float y, float z )
{
	int i=0;


		for(i=0;i<BONUSMAXNUM;i++)
		{
			if(bonusSlotList[i].bonState == 0)
			{

				printf(" adding bonus : %d\n", i);

				bonusSlotList[i].bonType = bonusPresetId;
				bonusSlotList[i].bonTexId = bonusSlotPreset[bonusPresetId].bonTexId;
				bonusSlotList[i].spawnTexId = bonusSlotPreset[bonusPresetId].spawnTexId;
				bonusSlotList[i].bonAngle = 0.0;
				bonusSlotList[i].bonState = 1;
				bonusSlotList[i].bonLife = 8.0;
				bonusSlotList[i].bonPosX = x;
				bonusSlotList[i].bonPosY = y;
				bonusSlotList[i].bonPosZ = z;
				bonusSlotList[i].bonRadiusCol = 2.5;//bonusSlotPreset[bonusPresetId].bonRadiusCol;
				bonusSlotList[i].bonDashPower = bonusSlotPreset[bonusPresetId].bonDashPower;
				bonusSlotList[i].bonLifePower = bonusSlotPreset[bonusPresetId].bonLifePower;
				bonusSlotList[i].bonWeaponPower = bonusSlotPreset[bonusPresetId].bonWeaponPower;
				bonusSlotList[i].bonScorePower = bonusSlotPreset[bonusPresetId].bonScorePower;

				mojoAddBillboardSlot(1, 2.0, bonusSlotList[i].bonTexId, x, y-3.0, z );
				mojoAddBillboardSlot(0, 1.0, mojoTexManGetTextureId("ring_blue.tga"), x, y-3.0, z );

				return;
			}
		}



}




int			mojoCheckBonusCollision( float radius, float x, float y, float z)
{

	int 	i=0;
	double 	dist=0;

	for(i=0;i<BONUSMAXNUM;i++)
	{
		if(bonusSlotList[i].bonState>0)
		{

			if(mojoGetBoxCollide(	radius, x, y, z,
									bonusSlotList[i].bonRadiusCol, bonusSlotList[i].bonPosX, bonusSlotList[i].bonPosY, bonusSlotList[i].bonPosZ))
			{
//				dist = mojoGetDistance3D( 	bonusSlotList[i].bonPosX, bonusSlotList[i].bonPosY, bonusSlotList[i].bonPosZ,
//											x, y, z );
//				if(dist<(bonusSlotList[i].bonRadiusCol + radius))
//				{
					bonusSlotList[i].bonState=0;
					return bonusSlotList[i].bonType;
//				}
			}

		}

	}

	return -1;
}



void		mojoPlayerUpdate(struct PLAYER *player, int presetId)
{

	if((bonusSlotPreset[presetId].bonDashPower>0) && (bonusSlotPreset[presetId].bonDashPower>player->plyCurrentDashPower))
		player->plyCurrentDashPower = bonusSlotList[presetId].bonDashPower;

	printf("bonus Dpower: %i\n",bonusSlotPreset[presetId].bonDashPower);

	if((bonusSlotPreset[presetId].bonWeaponPower>0) && (bonusSlotPreset[presetId].bonWeaponPower>player->plyCurrentWPower))
		player->plyCurrentWPower = bonusSlotList[presetId].bonWeaponPower;

	printf("bonus WPower: %i\n",bonusSlotPreset[presetId].bonWeaponPower);

	if(bonusSlotPreset[presetId].bonScorePower>0)
		player->plyScore += bonusSlotPreset[presetId].bonScorePower;
	//printf("player Score: %i\n",player->plyScore);

	printf("bonus Score: %i\n",bonusSlotPreset[presetId].bonScorePower);

	if((bonusSlotPreset[presetId].bonLifePower>0) && (player->plyTotalLife < 100))
		player->plyTotalLife += bonusSlotPreset[presetId].bonLifePower;

	printf("bonus Life: %i\n",bonusSlotPreset[presetId].bonLifePower);

}




void		mojoUpdateBonusList( double elapsed )
{
	int 	i=0, texNum=0;
	MATRIX 	MtxTmp;

	//texNum = mojoTexManGetTextureId("impact.tga");

	//glEnable(GL_BLEND);
	//glEnable(GL_TEXTURE_2D);

	for(i=0;i<BONUSMAXNUM;i++)
	{

		if(bonusSlotList[i].bonState != 0.0 )
		{

		if(bonusSlotList[i].bonLife > 0.0 )
		{


			Bonus.pMaterials[0].texture = bonusSlotList[i].bonTexId;
			bonusSlotList[i].bonAngle += elapsed*60.0;

			MatrixLoadIdentity( &MtxTmp );

			MtxTmp.m[12] = bonusSlotList[i].bonPosX;
			MtxTmp.m[13] = bonusSlotList[i].bonPosY;
			MtxTmp.m[14] = bonusSlotList[i].bonPosZ;

			MatrixRotateY( &MtxTmp, bonusSlotList[i].bonAngle );

			glPushMatrix();

			mojoGlBodyDraw( &MtxTmp );
			glColor3f( 1.0, 1.0, 1.0);
			Model_Draw(&Bonus);

			glPopMatrix();

			//glColor3f( 1.0, 1.0, 1.0);
			bonusSlotList[i].bonLife -= elapsed;


			if( ( bonusSlotList[i].bonLife < 0.0 ) )
			{

				bonusSlotList[i].bonLife = 0.0;
				bonusSlotList[i].bonState = 0;
				//printf(" killing bullet : %d\n", i);

			}//endif

		}


		}//endif

	}//endfor

	//glDisable(GL_TEXTURE_2D);
	//glDisable(GL_BLEND);

}





void		mojoFreeBonusList( void )
{
	int i=0;

	for(i=0;i<BONUSMAXNUM;i++)
	{

		bonusSlotList[i].bonType = 0;
		bonusSlotList[i].id = 0;
		bonusSlotList[i].bonAngle = 0.0;
		bonusSlotList[i].bonState = 0;
		bonusSlotList[i].bonLife = 0.0;
	}
}



char			*mojoLoadLvlXmlCfg(char *filename, int node)
{

    ezxml_t 		xmlNode, resNode;
	char 			StrBuffer[200];
	FILE 			*cfgFile;


	// Xml Cfg Vars
	// -----------------------------------------------------
	if ( (cfgFile = fopen(filename, "rt")) == NULL ){

		sprintf(StrBuffer,"! mojo \tlog : Xml Definition parsing Error ! [ loading lvl in %s ]!\n",filename);
		mojolog (StrBuffer);
		return 0;
	}

    xmlNode = ezxml_parse_fp( cfgFile );

	sprintf(StrBuffer,"- mojo \tlog : Xml Definition parsing Success ! [ loading lvl in %s ]!\n",filename);
	mojolog (StrBuffer);

	fclose( cfgFile );

	resNode = NULL;

	switch (node)
	{
		case 0:
		resNode = ezxml_child(xmlNode, "leveldef");
		break;

		case 1:
		resNode = ezxml_child(xmlNode, "levelpresets");
		break;

		case 2:
		resNode = ezxml_child(xmlNode, "leveldmf");
		break;

		case 3:
		resNode = ezxml_child(xmlNode, "levelmusic");
		break;

		case 4:
		resNode = ezxml_child(xmlNode, "levelnext");
		break;

		case 5:
		resNode = ezxml_child(xmlNode, "leveltitle");
		break;
	}

	//printf(" res value : %s\n", resNode->txt);


	return resNode->txt;
}






int			mojoLoadResXmlCfg(char *filename)
{

    ezxml_t 		xmlNode, resNode;
	int				tmpInt;
    //char 			*xmlValue;
   	char            StrBuffer[200];
	FILE 			*cfgFile;


	// Xml Cfg Vars
	// -----------------------------------------------------
	if ( (cfgFile = fopen(filename, "rt")) == NULL ){

		sprintf(StrBuffer,"! mojo \tlog : Xml Definition parsing Error ! [ loading res in %s ]!\n",filename);
		mojolog (StrBuffer);
		return 0;
	}

    xmlNode = ezxml_parse_fp( cfgFile );

	sprintf(StrBuffer,"- mojo \tlog : Xml Definition parsing Success ! [ loading res in %s ]!\n",filename);
	mojolog (StrBuffer);

	fclose( cfgFile );

	resNode = NULL;
	resNode = ezxml_child(xmlNode, "texture");


	while(resNode!=NULL)
	{
		if(mojoTexManGetTextureId(resNode->txt) == -1 )
		{
			strcpy(mojoResDir,"./textures/\0");
			mojoLoadTexResource( resNode->txt );
			printf(" res value : %s\n", resNode->txt);
			sprintf(StrBuffer,"- mojo \tlog : Loading Res file ( %s )!\n",resNode->txt);
			mojolog (StrBuffer);
		}

		resNode = resNode->next;
	}



	resNode = NULL;
	resNode = ezxml_child(xmlNode, "sound");


	while(resNode!=NULL)
	{
		strcpy(mojoResDir,"./audio/\0");
		tmpInt = atoi(ezxml_attr( resNode , "id"));

		mojoSoundLoad( resNode->txt, tmpInt );
		printf(" res value : %s\n", resNode->txt);
		sprintf(StrBuffer,"- mojo \tlog : Loading Res file ( %s ) #id->%i!\n",resNode->txt, tmpInt);
		mojolog (StrBuffer);

		resNode = resNode->next;
	}


	//ezxml_free( tmpNode );		// free nodes in reverse order, to prevent an exception
	ezxml_free( resNode );
	ezxml_free( xmlNode );
	//free( xmlValue );


	return 0;
}




int			mojoLoadBonusPresetXmlCfg(char *filename)
{

    ezxml_t 		xmlNode, resNode, tmpNode;
    int 			Value;
   	char            StrBuffer[200];
	FILE 			*cfgFile;


		// Xml Cfg Vars
	// -----------------------------------------------------
	if ( (cfgFile = fopen(filename, "rt")) == NULL ){

		sprintf(StrBuffer,"! mojo \tlog : Xml Definition parsing Error ! [ loading bonus preset in %s ]!\n",filename);
		mojolog (StrBuffer);
		return 0;
	}

    xmlNode = ezxml_parse_fp( cfgFile );

	sprintf(StrBuffer,"- mojo \tlog : Xml Definition parsing Success ! [ loading bonus preset in %s ]!\n",filename);
	mojolog (StrBuffer);

	fclose( cfgFile );

	resNode = NULL;
	resNode = ezxml_child(xmlNode, "bonus");


	while(resNode!=NULL)
	{


		tmpNode = ezxml_child(resNode, "type");
		Value = atoi(tmpNode->txt);
		bonusSlotPreset[Value].bonType = Value;

		tmpNode = ezxml_child(resNode, "bontexname");
		bonusSlotPreset[Value].bonTexId = mojoTexManGetTextureId(tmpNode->txt);

		tmpNode = ezxml_child(resNode, "spawntexname");
		bonusSlotPreset[Value].spawnTexId = mojoTexManGetTextureId(tmpNode->txt);

		tmpNode = ezxml_child(resNode, "bonDashPower");
		bonusSlotPreset[Value].bonDashPower = atof(tmpNode->txt);

		tmpNode = ezxml_child(resNode, "bonLifePower");
		bonusSlotPreset[Value].bonLifePower = atof(tmpNode->txt);

		tmpNode = ezxml_child(resNode, "bonWeaponPower");
		bonusSlotPreset[Value].bonWeaponPower = atof(tmpNode->txt);

		tmpNode = ezxml_child(resNode, "bonScorePower");
		bonusSlotPreset[Value].bonScorePower = atof(tmpNode->txt);


		//tmpNode = ezxml_child(resNode, "duration");
		//bonusSlotPreset[Value].bonLife = atof(tmpNode->txt);

		//tmpNode = ezxml_child(resNode, "colradius");
		//bonusSlotPreset[Value].bonRadiusCol = atof(tmpNode->txt);


		printf(" bonus name : %i\n", Value);
		sprintf(StrBuffer,"- mojo \tlog : Loading Bonus preset ( %i )!\n", Value);
		mojolog (StrBuffer);

		resNode = resNode->next;
	}

	//ezxml_free( tmpNode );		// free nodes in reverse order, to prevent an exception
	ezxml_free( resNode );
	ezxml_free( xmlNode );
	//free( xmlValue );


	return 0;
}





struct ENEMY	*mojoLoadEnmXmlCfg(char *filename)
{

    ezxml_t 		xmlNode, tmpNode, enemyNode;
    //char 			*xmlValue;
   	char            StrBuffer[200];
	FILE 			*cfgFile;
	int				first=0;
	struct ENEMY	*tmpEnm=NULL;
	struct ENEMY	*firstEnm=NULL;

		// Xml Cfg Vars
	// -----------------------------------------------------
	if ( (cfgFile = fopen(filename, "rt")) == NULL ){

		sprintf(StrBuffer,"! mojo \tlog : Xml Definition parsing Error ! [ loading entities in %s ]!\n",filename);
		mojolog (StrBuffer);
		return firstEnm;
	}

    xmlNode = ezxml_parse_fp( cfgFile );

	sprintf(StrBuffer,"- mojo \tlog : Xml Definition parsing Success ! [ loading entities in %s ]!\n",filename);
	mojolog (StrBuffer);

	fclose( cfgFile );

	enemyNode = NULL;
	enemyNode = ezxml_child(xmlNode, "enemy");

	mojolog ("- mojo \tlog : Creating Enemy List!\n");


	while(enemyNode!=NULL)
	{

		tmpNode = NULL;
		iEnm++;

		tmpEnm = mojoAddEnmSlot( tmpEnm );

		//xmlValue = ezxml_attr(enemyNode, "id");
		tmpNode = ezxml_child(enemyNode, "type");
		tmpEnm->enmType = atoi(tmpNode->txt);

		tmpEnm->id = iEnm;
		// enemy type

		tmpEnm->enmType = atoi(tmpNode->txt);
		// enemy posX
		tmpNode = ezxml_child(enemyNode, "posx");
		tmpEnm->enmMtx.m[12] = atof(tmpNode->txt);
		// enemy posY
		tmpNode = ezxml_child(enemyNode, "posy");
		tmpEnm->enmMtx.m[13] = atof(tmpNode->txt);
		// enemy posZ
		tmpNode = ezxml_child(enemyNode, "posz");
		tmpEnm->enmMtx.m[14] = atof(tmpNode->txt);
		// Start Angle
		tmpNode = ezxml_child(enemyNode, "angle");
		tmpEnm->enmAngle = atof(tmpNode->txt);
		// Chase Speed
		tmpNode = ezxml_child(enemyNode, "forwardspeed");
		tmpEnm->enmForwardSpeed = atof(tmpNode->txt);
		// Rotation speed
		tmpNode = ezxml_child(enemyNode, "turnspeed");
		tmpEnm->enmTurnSpeed = atof(tmpNode->txt);
		// collision sphere radius
		tmpNode = ezxml_child(enemyNode, "colradius");
		tmpEnm->enmRadiusCol = atof(tmpNode->txt);
		// enemy life points
		tmpNode = ezxml_child(enemyNode, "life");
		tmpEnm->enmTotalLife = atof(tmpNode->txt);

		// enemy damage to player
		tmpNode = ezxml_child(enemyNode, "damage");
		tmpEnm->enmDamage = atof(tmpNode->txt);


		tmpEnm->enmState = 0;
		tmpEnm->enmDistToPlayer  = 0.0;
		tmpEnm->enmAngleToPlayer = 0.0;
		tmpEnm->enmRandomTimer = 0.0;
		tmpEnm->enmRandomAngle = 0.0;

		printf("Adding enm : %d\n", tmpEnm->id);
		//printf(" type value : %s\n", tmpNode->txt);
		enemyNode = enemyNode->next;




		if(first == 0)
		{
			firstEnm = tmpEnm;
			first = 1;
		}


	}

	//ezxml_free( tmpNode );		// free nodes in reverse order, to prevent an exception
	ezxml_free( enemyNode );
	ezxml_free( xmlNode );
	//free( xmlValue );


	return firstEnm;
}








void 		mojoSkyBoxDraw(float x, float y, float z, float width, float height, float length, int texid[])
{
	// This is the most important function of this tutorial.  This function
	// used to just create a silly colored cube in the RotateCube tutorial,
	// but now it creates something beautiful.  You'll notice we added
	// some more parameters to the function.  This way we can change the perspective
	// of the sky box.  It doesn't really look good if it's a perfect cube.  Some
	// textures look better at different ratios.  We assign the sky box textures
	// to each side of the box creating the illusion of a detailed 3D world.
	// You will notice I had to change the texture coordinates for every one
	// to be flipped correctly.  Also, depending on your culling, the vertex
	// order might need to be changed around.  I don't use culling in this tutorial
	// so it will work fine here, but be sure to remember this if you do.

	// Bind the BACK texture of the sky map to the BACK side of the cube
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, texid[1]);

	// Since we want the sky box to be centered around X, Y, and Z for ease,
	// we do a little math to accomplish this.  We just change the X, Y and Z
	// to perform this task.  If we just minus half the width, height and length
	// from x, y and z it will give us the desired result.  Now when we create the
	// box it will center it around (x, y, z)

	// This centers the sky box around (x, y, z)
	x = x - width  / 2;
	y = y - height / 2;
	z = z - length / 2;

	glColor3f(1.0,1.0,1.0);
   glDisable(GL_CULL_FACE);
	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);

		// Assign the texture coordinates and vertices for the BACK Side
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 0.0f); glVertex3f(x,			y,			z);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 1.0f); glVertex3f(x,			y + height, z);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 1.0f); glVertex3f(x + width, y + height, z);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 0.0f); glVertex3f(x + width, y,			z);
	glEnd();

	// Bind the FRONT texture of the sky map to the FRONT side of the box
	glBindTexture(GL_TEXTURE_2D, texid[0]);

	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);

		// Assign the texture coordinates and vertices for the FRONT Side
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 0.0f); glVertex3f(x,			y,			z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 1.0f); glVertex3f(x,			y + height, z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 1.0f); glVertex3f(x + width, y + height, z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 0.0f); glVertex3f(x + width, y,			z + length);
	glEnd();

	// Bind the BOTTOM texture of the sky map to the BOTTOM side of the box
	glBindTexture(GL_TEXTURE_2D, texid[3]);

	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);

		// Assign the texture coordinates and vertices for the BOTTOM Side
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 0.0f); glVertex3f(x,			y,			z);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 1.0f); glVertex3f(x,			y,			z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 1.0f); glVertex3f(x + width, y,			z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 0.0f); glVertex3f(x + width, y,			z);
	glEnd();

	// Bind the TOP texture of the sky map to the TOP side of the box
	glBindTexture(GL_TEXTURE_2D, texid[2]);

	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);

		// Assign the texture coordinates and vertices for the TOP Side
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 1.0f); glVertex3f(x,			y + height,	z);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 0.0f); glVertex3f(x,			y + height,	z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 0.0f); glVertex3f(x + width, y + height, z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 1.0f); glVertex3f(x + width, y + height, z);
	glEnd();

	// Bind the LEFT texture of the sky map to the LEFT side of the box
	glBindTexture(GL_TEXTURE_2D, texid[4]);

	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);

		// Assign the texture coordinates and vertices for the LEFT Side
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 0.0f); glVertex3f(x,			y,			z);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 0.0f); glVertex3f(x,			y,			z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 1.0f); glVertex3f(x,			y + height,	z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 1.0f); glVertex3f(x,			y + height,	z);
	glEnd();

	// Bind the RIGHT texture of the sky map to the RIGHT side of the box
	glBindTexture(GL_TEXTURE_2D, texid[5]);

	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);

		// Assign the texture coordinates and vertices for the RIGHT Side
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 0.0f); glVertex3f(x + width, y,			z);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 0.0f); glVertex3f(x + width, y,			z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 1.0f); glVertex3f(x + width, y + height,	z + length);
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 1.0f); glVertex3f(x + width, y + height,	z);
	glEnd();

	glDisable(GL_TEXTURE_2D);
   	glEnable(GL_CULL_FACE); 		   	// Enable back face culling
}




void		mojoQuadDraw2( float x, float y, float x1, float y1, int texId, float alpha)
{

	// Bind the FRONT texture of the sky map to the FRONT side of the box
	glBindTexture(GL_TEXTURE_2D, texId);

	// Start drawing the side as a QUAD
	glBegin(GL_QUADS);

		// Assign the texture coordinates and vertices for the FRONT Side
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 0.0f); glVertex2f(x,			y  );
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,1.0f, 1.0f); glVertex2f(x,			y1 );
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 1.0f); glVertex2f(x1, 		y1 );
		glMultiTexCoord2fARB(GL_TEXTURE0_ARB,0.0f, 0.0f); glVertex2f(x1 , 		y );
	glEnd();

}



void 		mojoBoxDraw(float length, float width, float height, float offX, float offY, float offZ)
{


	double x=length, y=width, z=height;


  	//glDisable (GL_TEXTURE_2D);
	//glColor3f(1.0,1.0,1.0);

	glBegin(GL_QUADS);
	glNormal3f( 0.0f, 0.0f, 1.0f);
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-x+offX, -y+offY,  z+offZ);
	glTexCoord2f(0.0f, 0.0f); glVertex3f( x+offX, -y+offY,  z+offZ);
	glTexCoord2f(1.0f, 1.0f); glVertex3f( x+offX,  y+offY,  z+offZ);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-x+offX,  y+offY,  z+offZ);

	glNormal3f( 0.0f, 0.0f,-1.0f);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-x+offX, -y+offY, -z+offZ);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(-x+offX,  y+offY, -z+offZ);
	glTexCoord2f(0.0f, 1.0f); glVertex3f( x+offX,  y+offY, -z+offZ);
	glTexCoord2f(0.0f, 0.0f); glVertex3f( x+offX, -y+offY, -z+offZ);

	glNormal3f( 0.0f, 1.0f, 0.0f);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-x+offX,  y+offY, -z+offZ);
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-x+offX,  y+offY,  z+offZ);
	glTexCoord2f(1.0f, 0.0f); glVertex3f( x+offX,  y+offY,  z+offZ);
	glTexCoord2f(1.0f, 1.0f); glVertex3f( x+offX,  y+offY, -z+offZ);

	glNormal3f( 0.0f,-1.0f, 0.0f);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(-x+offX, -y+offY, -z+offZ);
	glTexCoord2f(0.0f, 1.0f); glVertex3f( x+offX, -y+offY, -z+offZ);
	glTexCoord2f(0.0f, 0.0f); glVertex3f( x+offX, -y+offY,  z+offZ);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-x+offX, -y+offY,  z+offZ);

	glNormal3f( 1.0f, 0.0f, 0.0f);
	glTexCoord2f(1.0f, 0.0f); glVertex3f( x+offX, -y+offY, -z+offZ);
	glTexCoord2f(1.0f, 1.0f); glVertex3f( x+offX,  y+offY, -z+offZ);
	glTexCoord2f(0.0f, 1.0f); glVertex3f( x+offX,  y+offY,  z+offZ);
	glTexCoord2f(0.0f, 0.0f); glVertex3f( x+offX, -y+offY,  z+offZ);

	glNormal3f(-1.0f, 0.0f, 0.0f);
	glTexCoord2f(0.0f, 0.0f); glVertex3f(-x+offX, -y+offY, -z+offZ);
	glTexCoord2f(1.0f, 0.0f); glVertex3f(-x+offX, -y+offY,  z+offZ);
	glTexCoord2f(1.0f, 1.0f); glVertex3f(-x+offX,  y+offY,  z+offZ);
	glTexCoord2f(0.0f, 1.0f); glVertex3f(-x+offX,  y+offY, -z+offZ);
	glEnd();



}



void mojoCinemascope( int resX, int resY, int cinemascopeLo, int cinemascopeHi )
{


// Do Cinemascope
/////////////////////////////////////////////////////////////


//    glDisable(GL_TEXTURE_2D); // This Enable the Texture mapping
//	glBlendFunc(GL_ZERO, GL_SRC_COLOR);							// Select The Type Of Blending

	//_inF_glOrthoLoad();
	//glViewport(0, 0, resX, resY);
	glLoadIdentity();

	glColor3f(0.0f,0.0f,0.0f);
	glBegin(GL_QUADS);										// Start Drawing Our Screen Lines
		glVertex2d(0,0);									// Top Left Of Screen
		glVertex2d( resX, 0);								// Bottom Right Of Screen
		glVertex2d( resX,cinemascopeLo);					// Top Right Of Screen
		glVertex2d(	0, cinemascopeLo);						// Bottom Left Of Screen
	glEnd();

	glBegin(GL_QUADS);										// Start Drawing Our Screen
	   															// Lines
		glVertex2d(0,cinemascopeLo+cinemascopeHi);		// Top Left Of Screen
		glVertex2d( resX, cinemascopeLo+cinemascopeHi);	// Bottom Right Of Screen
		glVertex2d( resX,resY);								// Top Right Of Screen
		glVertex2d(	0, resY);								// Bottom Left Of Screen
	glEnd();


/*			glTexCoord2d(0,0);	glVertex2d(0,0);				// Top Left Of Screen
			glTexCoord2d(1,0);	glVertex2d( 800, 0);			// Bottom Right Of Screen
			glTexCoord2d(1,1);	glVertex2d( 800,100);			// Top Right Of Screen
			glTexCoord2d(0,1);	glVertex2d(	0, 100);			// Bottom Left Of Screen
*/

	//_inF_glOrthoUnload();

	//glDisable(GL_BLEND);										// Enable Blending


}



void	mojoFadeOut( float elapsed, float resX, float resY)
{


	if(fadeState == -1)
	{

		fadeLevel -= elapsed;

		if(fadeLevel<0.0)
		{
			mojoFadeSetState( 0 );
			return;
		}

		glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);
		glEnable(GL_BLEND);
		glColor4f( fadeLevel, fadeLevel, fadeLevel, fadeLevel);
		glBegin(GL_QUADS);										// Start Drawing Our Screen Lines
			glVertex2d(0,0);									// Top Left Of Screen
			glVertex2d( resX, 0);								// Bottom Right Of Screen
			glVertex2d( resX,resY);					// Top Right Of Screen
			glVertex2d(	0, resY);						// Bottom Left Of Screen
		glEnd();
		glDisable(GL_BLEND);
	}
}


int	mojoFadeIn( float elapsed, float resX, float resY)
{

	if(fadeState == 1)
	{

		fadeLevel += elapsed;


		if(fadeLevel>1.0)
		{
			mojoFadeSetState( 0 );
			return 1;
		}


		glBlendFunc(GL_ZERO, GL_ONE_MINUS_SRC_COLOR);
		glEnable(GL_BLEND);
		glColor4f( fadeLevel, fadeLevel, fadeLevel, fadeLevel);
		glBegin(GL_QUADS);										// Start Drawing Our Screen Lines
			glVertex2d(0,0);									// Top Left Of Screen
			glVertex2d( resX, 0);								// Bottom Right Of Screen
			glVertex2d( resX,resY);					// Top Right Of Screen
			glVertex2d(	0, resY);						// Bottom Left Of Screen
		glEnd();
		glDisable(GL_BLEND);


	}

	return 0;
}



void	mojoFadeSetState( int state /* -1=fadeOut, 0=none/stop, 1=fadeIn */ )
{


	fadeState = state;
	switch (state)
	{
		case -1:
			fadeLevel = 1.0;
		break;

		case 0:
			fadeLevel = 0.0;
		break;

		case 1:
			fadeLevel = 0.0;
		break;
	}


}



int		mojoFadeGetState( void )
{


	return fadeState;


}



void 	mojoGlowRTCreate(int textureSize, int RTid)											// Create An Empty Texture
{

	unsigned int* data;											// Stored Data
	unsigned int			texid;


	glGenTextures(1, &texid);

	// Create Storage Space For Texture Data (128x128x4)
	data = (unsigned int*) malloc((textureSize * textureSize)* 4 * sizeof(unsigned int));
	ZeroMemory(data,((textureSize * textureSize)* 4 * sizeof(unsigned int)));	// Clear Storage Memory
	//memset(data, 0, (textureSize * textureSize)* 4 * sizeof(unsigned int));
	textureGlowSlot[RTid].texId = texid;
	textureGlowSlot[RTid].size = textureSize;

	glBindTexture(GL_TEXTURE_2D, texid);					// Bind The Texture
	glTexImage2D(GL_TEXTURE_2D, 0, 4, textureSize, textureSize, 0,
		GL_RGBA, GL_UNSIGNED_BYTE, data);						// Build Texture Using Information In data
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);

		// trick : this avoid texture bleeding at the edges of the RenderTarget
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,  GL_CLAMP_TO_EDGE );
        glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,  GL_CLAMP_TO_EDGE );
	free ( data );												// Release data

}


void mojoGlowRTRender( int texIdSrc, int texIdDest)											// Renders To A Texture
{

	GLOWTEX	*ptr = &textureGlowSlot[texIdDest];
	int sizeDest = (int)ptr->size;


	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);						// Set The Clear Color To Medium Blue
	glClear(GL_COLOR_BUFFER_BIT );			// Clear The Screen And Depth Buffer

	glActiveTextureARB( GL_TEXTURE0_ARB );
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D,textureGlowSlot[texIdSrc].texId);					// Bind To The Blur Texture

	glColor4f(1.0, 1.0, 1.0, 1.0);
		glBegin(GL_QUADS);										// Start Drawing Our Quad
			glTexCoord2d(0,0);	glVertex2d(0,0);		// Top Left Of Screen
			glTexCoord2d(1,0);	glVertex2d( sizeDest, 0);	// Bottom Right Of Screen
			glTexCoord2d(1,1);	glVertex2d( sizeDest,sizeDest);	// Top Right Of Screen
			glTexCoord2d(0,1);	glVertex2d(	0, sizeDest);	// Bottom Left Of Screen
		glEnd();

	//glDisable(GL_TEXTURE_2D);

	glActiveTextureARB( GL_TEXTURE0_ARB );
	glBindTexture(GL_TEXTURE_2D,ptr->texId);					// Bind To The Blur Texture
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR); // param de texture
	//glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR); // param de texture


	// Copy Our ViewPort To The Blur Texture (From 0,0 To 128,128... No Border)
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, sizeDest, sizeDest);
	//glCopyTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, 0, 0, sizeDest, sizeDest, 0);



}

