/*
 * 	---------------- MoJo engine --------------
 *   		---- Basic Game Engine ----
 *
 * Author: Yann PARMENTIER <kohai@kohaistyle.com>
 *
 * ALL RIGHTS RESERVED
 *
 *	Uses
 * 			DMFloader  		( www.delgine.com )
 * 			GLFW 				( glfw.sourceforge.net )
 *			Fmod Sound API 		( www.fmod.org )
 *
 * To compile this project you must include the following libraries:
 * wlib.lib uuid.lib fmodlcc.lib glfw.lib glu32.lib ole32.lib olepro32.lib opengl32.lib
 * You need also the header file glfw.h in your GL/ compiler directory.
 *
 * File header: mojotexMan.c
 *
 */




#include 	<string.h>
#include 	<stdlib.h>
#include 	<GL/glfw.h>
#include 	<GL/glext.h>

#include 	"mojomain.h"

/**********************************************************
 *
 * VARIABLES DECLARATION
 *
 *********************************************************/


/**********************************************************
 *
 * FUNCTION LoadBitmap(char *)
 *
 * This function loads a bitmap file and return the OpenGL reference ID to use that texture
 *
 *********************************************************/

int mojoTexManLoad( char *filename, int tex_id, int mipmap, GLint flag )
{

	char 	tmp[25],ext[4];
	int		success;
	char	path[50];

		glBindTexture(GL_TEXTURE_2D, tex_id);
		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, flag );
        glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, flag );
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR );
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR );
		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, /*GL_BLEND*/ GL_MODULATE);
		//glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
		//glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	 	//glTexGeni(GL_S, GL_SPHERE_MAP, 0);
	 	//glTexGeni(GL_T, GL_SPHERE_MAP, 0);                          // All T texture coordinates will be generated for a sphere





	strcpy(path,mojoResDir);
	strcat(path,filename);

		if ( mipmap == 1)
		{
		success = glfwLoadTexture2D( path, GLFW_BUILD_MIPMAPS_BIT );
		}else{
		success = glfwLoadTexture2D( path, 0 );
		}

		_splitpath(filename,NULL,NULL,tmp,ext);
		strcat(tmp,"\0");
		strcat(tmp,".tga");


		if(success == 1)
		{
			textureSlot[tex_id].texLoaded = 1;
			strcpy( textureSlot[tex_id].texName, strlwr(tmp));
			textureSlot[tex_id].texId = tex_id;
		}

	return success;
}


int	mojoTexManGetFreeId( void )
{

	int i;

	for(i=0;i<MAX_TEX_SLOTS;i++)
	{

		if(textureSlot[i].texLoaded==0)
			return i;

	}

	return -1;
}



void mojoTexManSetTexture( int slot, char *name, int texId )
{
		textureSlot[slot].texLoaded = 1;
		strcpy( textureSlot[slot].texName, strlwr(name));
		textureSlot[slot].texId = texId;

}



int mojoTexManGetTextureId( char *name )
{

	int		id = -1, i;

	for(i=0; i< MAX_TEX_SLOTS; i++){
		if(!strcmpi(textureSlot[i].texName,strlwr(name))){
			id = i;
			break;
		}
	}


	return id;
}


void mojoTexManFreeTexture( int slot )
{

		glDeleteTextures(1,&textureSlot[slot].texId);
		textureSlot[slot].texLoaded = 0;
		textureSlot[slot].texName[0] = '\0';
		textureSlot[slot].texId = 0;

}




void mojoTexManFreeList( void )
{
	int i;

	for(i=0; i< MAX_TEX_SLOTS; i++){

		if(textureSlot[i].texLoaded)
		{
			glDeleteTextures(1,&textureSlot[i].texId);
			mojoTexManFreeTexture( i );
		}
	}

}
