#ifndef __PLASMA_VECTOR_H
#define __PLASMA_VECTOR_H




typedef struct VectorType VECTOR;

#ifndef PI
  #define PI 3.141592653589793F
#endif

/* array offsets */
#define X   0
#define Y   1
#define Z   2

struct VectorType {
  float v[3];
};

extern __inline void   VectorLoad(VECTOR *v, float x, float y, float z);
extern __inline void   VectorGet(VECTOR *v, float *x, float *y, float *z);
extern __inline void   VectorRotateX(VECTOR *v, float pitch);
extern __inline void   VectorRotateY(VECTOR *v, float yaw);
extern __inline void   VectorRotateZ(VECTOR *v, float roll);
extern __inline void   VectorCopy(VECTOR *v, VECTOR *w);
extern __inline void   VectorAdd(VECTOR *v, VECTOR *w);
extern __inline void   VectorSubtract(VECTOR *v, VECTOR *w);
extern __inline void   VectorMultiply(VECTOR *v, float scalar);
extern __inline void   VectorDivide(VECTOR *v, float scalar);
extern __inline int    VectorEqual(VECTOR *v, VECTOR *w);
extern __inline float  VectorDotProduct(VECTOR *v, VECTOR *w);
extern __inline float  VectorLength(VECTOR *v);
extern __inline void   VectorCrossProduct(VECTOR *v, VECTOR *v1, VECTOR *v2);
extern __inline void   VectorNormalize(VECTOR *v);
extern __inline float  VectorGetYaw(VECTOR *v);
extern __inline float  VectorGetPitch(VECTOR *v);
extern __inline float  VectorGetRoll(VECTOR *v);

//#ifdef UNTESTED_NEVER_USED

//#endif

/* ease of use alias */
#define VectorLength2(v) VectorDotProduct((v),(v))

/* angular macros */
#ifndef RADIAN
  	#define RADIAN(x) ((x)*PI/180.0f)
  	#define DEGREE(x) ((x)*180.0f/PI)
  	#define DEGREEADD(x, a)        do { (x) += (a); while ((x) > 180.0f) (x) -= 360.0f; } while(0)
  	#define DEGREESUBTRACT(x, a)   do { (x) -= (a); while ((x) <-180.0f) (x) += 360.0f; } while(0)
  	#define DEGREENORMALIZE(x)     do { while ((x) > 180.0f) (x) -= 360.0f; while ((x) < -180.0f) (x) += 360.0f; } while(0)
	#define WRAP(angle) (angle>0.0) ? fmod(angle,360.0) : 360.0-abs(angle)
	/* find minimum of a and b */
	#define MIN(a,b)	(((a)<(b))?(a):(b))

	/* find maximum of a and b */
	#define MAX(a,b)	(((a)>(b))?(a):(b))

	/* swap a and b (see Gem by Wyvill) */
	#define SWAP(a,b)	{ a^=b; b^=a; a^=b; }

	/* linear interpolation from l (when a=0) to h (when a=1)*/
	/* (equal to (a*h)+((1-a)*l) */
	#define LERP(a,l,h)	((l)+(((h)-(l))*(a)))

	/* clamp the input to the specified range */
	#define CLAMP(v,l,h)	((v)<(l) ? (l) : (v) > (h) ? (h) : v)


#endif
#endif

